/* valacompiler.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valacompiler.h>
#include <valacodecontext.h>
#include <valareport.h>
#include <stdio.h>
#include <valasourcefile.h>
#include <valasourcereference.h>
#include <valaparser.h>
#include <valasymbolbuilder.h>
#include <valaattributeprocessor.h>
#include <valasymbolresolver.h>
#include <valasemanticanalyzer.h>
#include <valamemorymanager.h>
#include <valacodegenerator.h>
#include <valainterfacewriter.h>

struct _ValaCompilerPrivate {
	ValaCodeContext* context;
};
#define VALA_COMPILER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_COMPILER, ValaCompilerPrivate))
enum  {
	VALA_COMPILER_DUMMY_PROPERTY,
};
static char* vala_compiler_directory;
static gboolean vala_compiler_version;
static char** vala_compiler_sources;
static char** vala_compiler_vapi_directories;
static char* vala_compiler_library;
static char** vala_compiler_packages;
static gboolean vala_compiler_disable_memory_management;
static const GOptionEntry VALA_COMPILER_options[] = {{"vapidir", 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &vala_compiler_vapi_directories, "Look for package bindings in DIRECTORY", "DIRECTORY..."}, {"pkg", 0, 0, G_OPTION_ARG_STRING_ARRAY, &vala_compiler_packages, "Include binding for PACKAGE", "PACKAGE..."}, {"library", 0, 0, G_OPTION_ARG_STRING, &vala_compiler_library, "Library name", "NAME"}, {"directory", 'd', 0, G_OPTION_ARG_FILENAME, &vala_compiler_directory, "Output directory", "DIRECTORY"}, {"version", 0, 0, G_OPTION_ARG_NONE, &vala_compiler_version, "Display version number", NULL}, {"disable-memory-management", 0, 0, G_OPTION_ARG_NONE, &vala_compiler_disable_memory_management, "Disable memory management", NULL}, {"", 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &vala_compiler_sources, NULL, "FILE..."}, {NULL}};
static int vala_compiler_quit (ValaCompiler* self);
static char* vala_compiler_get_package_path (ValaCompiler* self, char* pkg);
static gboolean vala_compiler_add_package (ValaCompiler* self, char* pkg);
static int vala_compiler_run (ValaCompiler* self);
static int vala_compiler_main (int argc, char** argv);
static void vala_compiler_dispose (GObject * obj);

static int vala_compiler_quit (ValaCompiler* self)
{
	g_return_val_if_fail (VALA_IS_COMPILER (self), 0);
	if (vala_report_get_errors () == 0) {
		fprintf (stdout, "Compilation succeeded - %d warning(s)\n", vala_report_get_warnings ());
		return 0;
	} else {
		fprintf (stdout, "Compilation failed: %d error(s), %d warning(s)\n", vala_report_get_errors (), vala_report_get_warnings ());
		return 1;
	}
}


static char* vala_compiler_get_package_path (ValaCompiler* self, char* pkg)
{
	g_return_val_if_fail (VALA_IS_COMPILER (self), NULL);
	g_return_val_if_fail (pkg != NULL, NULL);
	char* basename = g_strdup_printf ("%s.vala", pkg);
	if (vala_compiler_vapi_directories != NULL) {
		{
			char** vapidir_it;
			for (vapidir_it = vala_compiler_vapi_directories; *vapidir_it != NULL; vapidir_it = vapidir_it + 1) {
				char* vapidir = *vapidir_it;
				{
					char* filename = g_build_filename (vapidir, basename, NULL);
					if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
						char* __temp2 = NULL;
						char* __temp3 = NULL;
						return (__temp3 = (__temp2 = filename, (__temp2 == NULL ? __temp2 : g_strdup (__temp2))), (filename == NULL ? NULL : (filename = (g_free (filename), NULL))), __temp3);
					}
					(filename == NULL ? NULL : (filename = (g_free (filename), NULL)));
				}
			}
		}
	}
	char* filename = g_build_filename ("/usr/local/share/vala/vapi", basename, NULL);
	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
		char* __temp4 = NULL;
		char* __temp5 = NULL;
		return (__temp5 = (__temp4 = filename, (__temp4 == NULL ? __temp4 : g_strdup (__temp4))), (basename == NULL ? NULL : (basename = (g_free (basename), NULL))), (filename == NULL ? NULL : (filename = (g_free (filename), NULL))), __temp5);
	}
	char* __temp6 = NULL;
	filename = (__temp6 = g_build_filename ("/usr/share/vala/vapi", basename, NULL), (filename == NULL ? NULL : (filename = (g_free (filename), NULL))), __temp6);
	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
		char* __temp7 = NULL;
		char* __temp8 = NULL;
		return (__temp8 = (__temp7 = filename, (__temp7 == NULL ? __temp7 : g_strdup (__temp7))), (basename == NULL ? NULL : (basename = (g_free (basename), NULL))), (filename == NULL ? NULL : (filename = (g_free (filename), NULL))), __temp8);
	}
	gpointer __temp9;
	return (__temp9 = NULL, (basename == NULL ? NULL : (basename = (g_free (basename), NULL))), (filename == NULL ? NULL : (filename = (g_free (filename), NULL))), __temp9);
	(basename == NULL ? NULL : (basename = (g_free (basename), NULL)));
	(filename == NULL ? NULL : (filename = (g_free (filename), NULL)));
}


static gboolean vala_compiler_add_package (ValaCompiler* self, char* pkg)
{
	g_return_val_if_fail (VALA_IS_COMPILER (self), FALSE);
	g_return_val_if_fail (pkg != NULL, FALSE);
	char* package_path = vala_compiler_get_package_path (self, pkg);
	if (package_path == NULL) {
		gboolean __temp10;
		return (__temp10 = FALSE, (package_path == NULL ? NULL : (package_path = (g_free (package_path), NULL))), __temp10);
	}
	ValaSourceFile* __temp11 = NULL;
	vala_code_context_add_source_file (self->priv->context, (__temp11 = g_object_new (VALA_TYPE_SOURCE_FILE, "filename", package_path, "pkg", TRUE, NULL)));
	(__temp11 == NULL ? NULL : (__temp11 = (g_object_unref (__temp11), NULL)));
	gboolean __temp12;
	return (__temp12 = TRUE, (package_path == NULL ? NULL : (package_path = (g_free (package_path), NULL))), __temp12);
	(package_path == NULL ? NULL : (package_path = (g_free (package_path), NULL)));
}


static int vala_compiler_run (ValaCompiler* self)
{
	g_return_val_if_fail (VALA_IS_COMPILER (self), 0);
	ValaCodeContext* __temp13 = NULL;
	self->priv->context = (__temp13 = g_object_new (VALA_TYPE_CODE_CONTEXT, NULL), (self->priv->context == NULL ? NULL : (self->priv->context = (g_object_unref (self->priv->context), NULL))), __temp13);
	/* default package */
	if (!vala_compiler_add_package (self, "glib-2.0")) {
		vala_report_error (NULL, "glib-2.0 not found in specified Vala API directories");
	}
	if (vala_compiler_packages != NULL) {
		{
			char** package_it;
			for (package_it = vala_compiler_packages; *package_it != NULL; package_it = package_it + 1) {
				char* package = *package_it;
				{
					if (!vala_compiler_add_package (self, package)) {
						char* __temp14 = NULL;
						vala_report_error (NULL, (__temp14 = g_strdup_printf ("%s not found in specified Vala API directories", package)));
						(__temp14 == NULL ? NULL : (__temp14 = (g_free (__temp14), NULL)));
					}
				}
			}
		}
		char** __temp15 = NULL;
		vala_compiler_packages = (__temp15 = NULL, (vala_compiler_packages == NULL ? NULL : (vala_compiler_packages = (g_strfreev (vala_compiler_packages), NULL))), __temp15);
	}
	if (vala_report_get_errors () > 0) {
		return vala_compiler_quit (self);
	}
	{
		char** source_it;
		for (source_it = vala_compiler_sources; *source_it != NULL; source_it = source_it + 1) {
			char* source = *source_it;
			{
				if (g_file_test (source, G_FILE_TEST_EXISTS)) {
					ValaSourceFile* __temp17 = NULL;
					vala_code_context_add_source_file (self->priv->context, (__temp17 = g_object_new (VALA_TYPE_SOURCE_FILE, "filename", source, NULL)));
					(__temp17 == NULL ? NULL : (__temp17 = (g_object_unref (__temp17), NULL)));
				} else {
					char* __temp18 = NULL;
					vala_report_error (NULL, (__temp18 = g_strdup_printf ("%s not found", source)));
					(__temp18 == NULL ? NULL : (__temp18 = (g_free (__temp18), NULL)));
				}
			}
		}
	}
	char** __temp19 = NULL;
	vala_compiler_sources = (__temp19 = NULL, (vala_compiler_sources == NULL ? NULL : (vala_compiler_sources = (g_strfreev (vala_compiler_sources), NULL))), __temp19);
	if (vala_report_get_errors () > 0) {
		return vala_compiler_quit (self);
	}
	ValaParser* parser = g_object_new (VALA_TYPE_PARSER, NULL);
	vala_parser_parse (parser, self->priv->context);
	if (vala_report_get_errors () > 0) {
		int __temp21;
		return (__temp21 = vala_compiler_quit (self), (parser == NULL ? NULL : (parser = (g_object_unref (parser), NULL))), __temp21);
	}
	ValaSymbolBuilder* builder = g_object_new (VALA_TYPE_SYMBOL_BUILDER, NULL);
	vala_symbol_builder_build (builder, self->priv->context);
	if (vala_report_get_errors () > 0) {
		int __temp22;
		return (__temp22 = vala_compiler_quit (self), (parser == NULL ? NULL : (parser = (g_object_unref (parser), NULL))), (builder == NULL ? NULL : (builder = (g_object_unref (builder), NULL))), __temp22);
	}
	ValaAttributeProcessor* attributeprocessor = g_object_new (VALA_TYPE_ATTRIBUTE_PROCESSOR, NULL);
	vala_attribute_processor_process (attributeprocessor, self->priv->context);
	if (vala_report_get_errors () > 0) {
		int __temp23;
		return (__temp23 = vala_compiler_quit (self), (parser == NULL ? NULL : (parser = (g_object_unref (parser), NULL))), (builder == NULL ? NULL : (builder = (g_object_unref (builder), NULL))), (attributeprocessor == NULL ? NULL : (attributeprocessor = (g_object_unref (attributeprocessor), NULL))), __temp23);
	}
	ValaSymbolResolver* resolver = g_object_new (VALA_TYPE_SYMBOL_RESOLVER, NULL);
	vala_symbol_resolver_resolve (resolver, self->priv->context);
	if (vala_report_get_errors () > 0) {
		int __temp24;
		return (__temp24 = vala_compiler_quit (self), (parser == NULL ? NULL : (parser = (g_object_unref (parser), NULL))), (builder == NULL ? NULL : (builder = (g_object_unref (builder), NULL))), (attributeprocessor == NULL ? NULL : (attributeprocessor = (g_object_unref (attributeprocessor), NULL))), (resolver == NULL ? NULL : (resolver = (g_object_unref (resolver), NULL))), __temp24);
	}
	ValaSemanticAnalyzer* analyzer = g_object_new (VALA_TYPE_SEMANTIC_ANALYZER, "memory_management", !vala_compiler_disable_memory_management, NULL);
	vala_semantic_analyzer_analyze (analyzer, self->priv->context);
	if (vala_report_get_errors () > 0) {
		int __temp25;
		return (__temp25 = vala_compiler_quit (self), (parser == NULL ? NULL : (parser = (g_object_unref (parser), NULL))), (builder == NULL ? NULL : (builder = (g_object_unref (builder), NULL))), (attributeprocessor == NULL ? NULL : (attributeprocessor = (g_object_unref (attributeprocessor), NULL))), (resolver == NULL ? NULL : (resolver = (g_object_unref (resolver), NULL))), (analyzer == NULL ? NULL : (analyzer = (g_object_unref (analyzer), NULL))), __temp25);
	}
	if (!vala_compiler_disable_memory_management) {
		ValaMemoryManager* memory_manager = g_object_new (VALA_TYPE_MEMORY_MANAGER, NULL);
		vala_memory_manager_analyze (memory_manager, self->priv->context);
		if (vala_report_get_errors () > 0) {
			int __temp26;
			return (__temp26 = vala_compiler_quit (self), (memory_manager == NULL ? NULL : (memory_manager = (g_object_unref (memory_manager), NULL))), __temp26);
		}
		(memory_manager == NULL ? NULL : (memory_manager = (g_object_unref (memory_manager), NULL)));
	}
	ValaCodeGenerator* code_generator = g_object_new (VALA_TYPE_CODE_GENERATOR, "memory_management", !vala_compiler_disable_memory_management, NULL);
	vala_code_generator_emit (code_generator, self->priv->context);
	if (vala_report_get_errors () > 0) {
		int __temp27;
		return (__temp27 = vala_compiler_quit (self), (parser == NULL ? NULL : (parser = (g_object_unref (parser), NULL))), (builder == NULL ? NULL : (builder = (g_object_unref (builder), NULL))), (attributeprocessor == NULL ? NULL : (attributeprocessor = (g_object_unref (attributeprocessor), NULL))), (resolver == NULL ? NULL : (resolver = (g_object_unref (resolver), NULL))), (analyzer == NULL ? NULL : (analyzer = (g_object_unref (analyzer), NULL))), (code_generator == NULL ? NULL : (code_generator = (g_object_unref (code_generator), NULL))), __temp27);
	}
	if (vala_compiler_library != NULL) {
		ValaInterfaceWriter* interface_writer = g_object_new (VALA_TYPE_INTERFACE_WRITER, NULL);
		char* __temp28 = NULL;
		vala_interface_writer_write_file (interface_writer, self->priv->context, (__temp28 = g_strdup_printf ("%s.vala", vala_compiler_library)));
		(__temp28 == NULL ? NULL : (__temp28 = (g_free (__temp28), NULL)));
		char* __temp29 = NULL;
		vala_compiler_library = (__temp29 = NULL, (vala_compiler_library == NULL ? NULL : (vala_compiler_library = (g_free (vala_compiler_library), NULL))), __temp29);
		(interface_writer == NULL ? NULL : (interface_writer = (g_object_unref (interface_writer), NULL)));
	}
	int __temp30;
	return (__temp30 = vala_compiler_quit (self), (parser == NULL ? NULL : (parser = (g_object_unref (parser), NULL))), (builder == NULL ? NULL : (builder = (g_object_unref (builder), NULL))), (attributeprocessor == NULL ? NULL : (attributeprocessor = (g_object_unref (attributeprocessor), NULL))), (resolver == NULL ? NULL : (resolver = (g_object_unref (resolver), NULL))), (analyzer == NULL ? NULL : (analyzer = (g_object_unref (analyzer), NULL))), (code_generator == NULL ? NULL : (code_generator = (g_object_unref (code_generator), NULL))), __temp30);
	(parser == NULL ? NULL : (parser = (g_object_unref (parser), NULL)));
	(builder == NULL ? NULL : (builder = (g_object_unref (builder), NULL)));
	(attributeprocessor == NULL ? NULL : (attributeprocessor = (g_object_unref (attributeprocessor), NULL)));
	(resolver == NULL ? NULL : (resolver = (g_object_unref (resolver), NULL)));
	(analyzer == NULL ? NULL : (analyzer = (g_object_unref (analyzer), NULL)));
	(code_generator == NULL ? NULL : (code_generator = (g_object_unref (code_generator), NULL)));
}


static int vala_compiler_main (int argc, char** argv)
{
	GError* err = NULL;
	GOptionContext* opt_context = g_option_context_new ("- Vala Compiler");
	g_option_context_set_help_enabled (opt_context, TRUE);
	g_option_context_add_main_entries (opt_context, VALA_COMPILER_options, NULL);
	g_option_context_parse (opt_context, &argc, &argv, &err);
	if (err != NULL) {
		int __temp31;
		return (__temp31 = 1, (opt_context == NULL ? NULL : (opt_context = (g_option_context_free (opt_context), NULL))), __temp31);
	}
	if (vala_compiler_sources == NULL) {
		fprintf (stderr, "No source file specified.\n");
		int __temp32;
		return (__temp32 = 1, (opt_context == NULL ? NULL : (opt_context = (g_option_context_free (opt_context), NULL))), __temp32);
	}
	{
		char** source_it;
		for (source_it = vala_compiler_sources; *source_it != NULL; source_it = source_it + 1) {
			char* source = *source_it;
			{
				if (!g_str_has_suffix (source, ".vala")) {
					fprintf (stderr, "Only .vala source files supported.\n");
					int __temp33;
					return (__temp33 = 1, (opt_context == NULL ? NULL : (opt_context = (g_option_context_free (opt_context), NULL))), __temp33);
				}
			}
		}
	}
	ValaCompiler* compiler = g_object_new (VALA_TYPE_COMPILER, NULL);
	int __temp34;
	return (__temp34 = vala_compiler_run (compiler), (opt_context == NULL ? NULL : (opt_context = (g_option_context_free (opt_context), NULL))), (compiler == NULL ? NULL : (compiler = (g_object_unref (compiler), NULL))), __temp34);
	(opt_context == NULL ? NULL : (opt_context = (g_option_context_free (opt_context), NULL)));
	(compiler == NULL ? NULL : (compiler = (g_object_unref (compiler), NULL)));
}


int main (int argc, char ** argv)
{
	g_type_init ();
	return vala_compiler_main (argc, argv);
}


void vala_compiler_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaCompiler * self = VALA_COMPILER (object);
	switch (property_id) {
	}
}


void vala_compiler_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaCompiler * self = VALA_COMPILER (object);
	switch (property_id) {
	}
}


static void vala_compiler_class_init (ValaCompilerClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaCompilerPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_compiler_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_compiler_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_compiler_dispose;
}


static void vala_compiler_init (ValaCompiler * self)
{
	self->priv = VALA_COMPILER_GET_PRIVATE (self);
}


static void vala_compiler_dispose (GObject * obj)
{
	ValaCompiler * self = VALA_COMPILER (obj);
	(self->priv->context == NULL ? NULL : (self->priv->context = (g_object_unref (self->priv->context), NULL)));
	ValaCompilerClass * klass;
	GObjectClass * parent_class;
	klass = VALA_COMPILER_CLASS (g_type_class_peek (VALA_TYPE_COMPILER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_compiler_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCompilerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_compiler_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCompiler), 0, (GInstanceInitFunc) vala_compiler_init };
		g_define_type_id = g_type_register_static (G_TYPE_OBJECT, "ValaCompiler", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



