/* valatype.vala
 *
 * Copyright (C) 2006  Raffaele Sandrini
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Raffaele Sandrini <rasa@gmx.ch>
 */

#include <valaarray.h>

struct _ValaArrayPrivate {
	ValaDataType* _element_type;
	char* cname;
};
#define VALA_ARRAY_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_ARRAY, ValaArrayPrivate))
enum  {
	VALA_ARRAY_DUMMY_PROPERTY,
	VALA_ARRAY_ELEMENT_TYPE,
};
static char* vala_array_real_get_cname (ValaDataType* base);
static gboolean vala_array_real_is_reference_type (ValaDataType* base);
static char* vala_array_real_get_upper_case_cname (ValaDataType* base, char* infix);
static char* vala_array_real_get_lower_case_cname (ValaDataType* base, char* infix);
static char* vala_array_real_get_free_function (ValaDataType* base);
static GList* vala_array_real_get_cheader_filenames (ValaDataType* base);
static GObject * vala_array_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void vala_array_dispose (GObject * obj);

/**
 * Returns the name of this data type as it is used in C code.
 *
 * @return the name to be used in C code
 */
static char* vala_array_real_get_cname (ValaDataType* base)
{
	ValaArray * self = VALA_ARRAY (base);
	if (self->priv->cname == NULL) {
		char* __temp0 = NULL;
		self->priv->cname = (__temp0 = g_strdup_printf ("%s*", vala_data_type_get_cname (vala_array_get_element_type (self))), (self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL))), __temp0);
	}
	return self->priv->cname;
}


/**
 * Checks whether this data type has value or reference type semantics.
 *
 * @return true if this data type has reference type semantics
 */
static gboolean vala_array_real_is_reference_type (ValaDataType* base)
{
	ValaArray * self = VALA_ARRAY (base);
	return TRUE;
}


/**
 * Returns the C name of this data type in upper case. Words are
 * separated by underscores. The upper case C name of the namespace is
 * prefix of the result.
 *
 * @param infix a string to be placed between namespace and data type
 *              name or null
 * @return      the upper case name to be used in C code
 */
static char* vala_array_real_get_upper_case_cname (ValaDataType* base, char* infix)
{
	ValaArray * self = VALA_ARRAY (base);
	return NULL;
}


/**
 * Returns the C name of this data type in lower case. Words are
 * separated by underscores. The lower case C name of the namespace is
 * prefix of the result.
 *
 * @param infix a string to be placed between namespace and data type
 *              name or null
 * @return      the lower case name to be used in C code
 */
static char* vala_array_real_get_lower_case_cname (ValaDataType* base, char* infix)
{
	ValaArray * self = VALA_ARRAY (base);
	return NULL;
}


/**
 * Returns the C function name that frees instances of this data type.
 * This is only valid for data types with reference type semantics that
 * do not support reference counting. The specified C function must
 * accept one argument pointing to the instance to be freed.
 *
 * @return the name of the C function or null if this data type is not a
 *         reference type or if it supports reference counting
 */
static char* vala_array_real_get_free_function (ValaDataType* base)
{
	ValaArray * self = VALA_ARRAY (base);
	return "g_free";
}


/**
 * Returns a list of C header filenames users of this data type must
 * include.
 *
 * @return list of C header filenames for this data type
 */
static GList* vala_array_real_get_cheader_filenames (ValaDataType* base)
{
	ValaArray * self = VALA_ARRAY (base);
	return vala_data_type_get_cheader_filenames (vala_array_get_element_type (self));
}


ValaDataType* vala_array_get_element_type (ValaArray* self)
{
	g_return_val_if_fail (VALA_IS_ARRAY (self), NULL);
	return self->priv->_element_type;
}


void vala_array_set_element_type (ValaArray* self, ValaDataType* value)
{
	g_return_if_fail (VALA_IS_ARRAY (self));
	ValaDataType* __temp8 = NULL;
	self->priv->_element_type = (__temp8 = g_object_ref (value), (self->priv->_element_type == NULL ? NULL : (self->priv->_element_type = (g_object_unref (self->priv->_element_type), NULL))), __temp8);
}


static GObject * vala_array_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties)
{
	GObject * obj;
	ValaArrayClass * klass;
	GObjectClass * parent_class;
	klass = VALA_ARRAY_CLASS (g_type_class_peek (VALA_TYPE_ARRAY));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	ValaArray * self = VALA_ARRAY (obj);
	{
		char* __temp9 = NULL;
		vala_data_type_set_name (VALA_DATA_TYPE (self), (__temp9 = g_strdup_printf ("%s[]", vala_data_type_get_name (vala_array_get_element_type (self)))));
		(__temp9 == NULL ? NULL : (__temp9 = (g_free (__temp9), NULL)));
	}
	return obj;
}


void vala_array_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaArray * self = VALA_ARRAY (object);
	switch (property_id) {
		case VALA_ARRAY_ELEMENT_TYPE:
		g_value_set_object (value, vala_array_get_element_type (self));
		break;
	}
}


void vala_array_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaArray * self = VALA_ARRAY (object);
	switch (property_id) {
		case VALA_ARRAY_ELEMENT_TYPE:
		vala_array_set_element_type (self, g_value_get_object (value));
		break;
	}
}


static void vala_array_class_init (ValaArrayClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaArrayPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_array_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_array_set_property;
	G_OBJECT_CLASS (klass)->constructor = vala_array_constructor;
	G_OBJECT_CLASS (klass)->dispose = vala_array_dispose;
	VALA_DATA_TYPE_CLASS (klass)->get_cname = vala_array_real_get_cname;
	VALA_DATA_TYPE_CLASS (klass)->is_reference_type = vala_array_real_is_reference_type;
	VALA_DATA_TYPE_CLASS (klass)->get_upper_case_cname = vala_array_real_get_upper_case_cname;
	VALA_DATA_TYPE_CLASS (klass)->get_lower_case_cname = vala_array_real_get_lower_case_cname;
	VALA_DATA_TYPE_CLASS (klass)->get_free_function = vala_array_real_get_free_function;
	VALA_DATA_TYPE_CLASS (klass)->get_cheader_filenames = vala_array_real_get_cheader_filenames;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_ARRAY_ELEMENT_TYPE, g_param_spec_object ("element-type", "foo", "bar", VALA_TYPE_DATA_TYPE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
}


static void vala_array_init (ValaArray * self)
{
	self->priv = VALA_ARRAY_GET_PRIVATE (self);
}


static void vala_array_dispose (GObject * obj)
{
	ValaArray * self = VALA_ARRAY (obj);
	(self->priv->_element_type == NULL ? NULL : (self->priv->_element_type = (g_object_unref (self->priv->_element_type), NULL)));
	(self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL)));
	ValaArrayClass * klass;
	GObjectClass * parent_class;
	klass = VALA_ARRAY_CLASS (g_type_class_peek (VALA_TYPE_ARRAY));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_array_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaArrayClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_array_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaArray), 0, (GInstanceInitFunc) vala_array_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_DATA_TYPE, "ValaArray", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



