/* valacastexpression.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valacastexpression.h>
#include <valatypereference.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>

struct _ValaCastExpressionPrivate {
	ValaExpression* _inner;
	ValaTypeReference* _type_reference;
};
#define VALA_CAST_EXPRESSION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CAST_EXPRESSION, ValaCastExpressionPrivate))
enum  {
	VALA_CAST_EXPRESSION_DUMMY_PROPERTY,
	VALA_CAST_EXPRESSION_INNER,
	VALA_CAST_EXPRESSION_TYPE_REFERENCE,
};
static void vala_cast_expression_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_cast_expression_dispose (GObject * obj);

/**
 * Creates a new cast expression.
 *
 * @param inner expression to be casted
 * @param type  target type
 * @return      newly created cast expression
 */
ValaCastExpression* vala_cast_expression_new (ValaExpression* inner, ValaTypeReference* type, ValaSourceReference* source)
{
	g_return_val_if_fail (VALA_IS_EXPRESSION (inner), NULL);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (type), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_CAST_EXPRESSION, "inner", inner, "type_reference", type, "source_reference", source, NULL));
}


static void vala_cast_expression_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaCastExpression * self = VALA_CAST_EXPRESSION (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_node_accept (VALA_CODE_NODE (vala_cast_expression_get_inner (self)), visitor);
	vala_code_node_accept (VALA_CODE_NODE (vala_cast_expression_get_type_reference (self)), visitor);
	vala_code_visitor_visit_cast_expression (visitor, self);
}


ValaExpression* vala_cast_expression_get_inner (ValaCastExpression* self)
{
	g_return_val_if_fail (VALA_IS_CAST_EXPRESSION (self), NULL);
	return self->priv->_inner;
}


void vala_cast_expression_set_inner (ValaCastExpression* self, ValaExpression* value)
{
	g_return_if_fail (VALA_IS_CAST_EXPRESSION (self));
	ValaExpression* __temp2 = NULL;
	self->priv->_inner = (__temp2 = g_object_ref (value), (self->priv->_inner == NULL ? NULL : (self->priv->_inner = (g_object_unref (self->priv->_inner), NULL))), __temp2);
}


ValaTypeReference* vala_cast_expression_get_type_reference (ValaCastExpression* self)
{
	g_return_val_if_fail (VALA_IS_CAST_EXPRESSION (self), NULL);
	return self->priv->_type_reference;
}


void vala_cast_expression_set_type_reference (ValaCastExpression* self, ValaTypeReference* value)
{
	g_return_if_fail (VALA_IS_CAST_EXPRESSION (self));
	ValaTypeReference* __temp4 = NULL;
	self->priv->_type_reference = (__temp4 = g_object_ref (value), (self->priv->_type_reference == NULL ? NULL : (self->priv->_type_reference = (g_object_unref (self->priv->_type_reference), NULL))), __temp4);
}


void vala_cast_expression_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaCastExpression * self = VALA_CAST_EXPRESSION (object);
	switch (property_id) {
		case VALA_CAST_EXPRESSION_INNER:
		g_value_set_object (value, vala_cast_expression_get_inner (self));
		break;
		case VALA_CAST_EXPRESSION_TYPE_REFERENCE:
		g_value_set_object (value, vala_cast_expression_get_type_reference (self));
		break;
	}
}


void vala_cast_expression_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaCastExpression * self = VALA_CAST_EXPRESSION (object);
	switch (property_id) {
		case VALA_CAST_EXPRESSION_INNER:
		vala_cast_expression_set_inner (self, g_value_get_object (value));
		break;
		case VALA_CAST_EXPRESSION_TYPE_REFERENCE:
		vala_cast_expression_set_type_reference (self, g_value_get_object (value));
		break;
	}
}


static void vala_cast_expression_class_init (ValaCastExpressionClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaCastExpressionPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_cast_expression_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_cast_expression_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_cast_expression_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_cast_expression_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CAST_EXPRESSION_INNER, g_param_spec_object ("inner", "foo", "bar", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CAST_EXPRESSION_TYPE_REFERENCE, g_param_spec_object ("type-reference", "foo", "bar", VALA_TYPE_TYPE_REFERENCE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
}


static void vala_cast_expression_init (ValaCastExpression * self)
{
	self->priv = VALA_CAST_EXPRESSION_GET_PRIVATE (self);
}


static void vala_cast_expression_dispose (GObject * obj)
{
	ValaCastExpression * self = VALA_CAST_EXPRESSION (obj);
	(self->priv->_inner == NULL ? NULL : (self->priv->_inner = (g_object_unref (self->priv->_inner), NULL)));
	(self->priv->_type_reference == NULL ? NULL : (self->priv->_type_reference = (g_object_unref (self->priv->_type_reference), NULL)));
	ValaCastExpressionClass * klass;
	GObjectClass * parent_class;
	klass = VALA_CAST_EXPRESSION_CLASS (g_type_class_peek (VALA_TYPE_CAST_EXPRESSION));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_cast_expression_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCastExpressionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_cast_expression_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCastExpression), 0, (GInstanceInitFunc) vala_cast_expression_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_EXPRESSION, "ValaCastExpression", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



