/* valaclass.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valaclass.h>
#include <valaconstructor.h>
#include <valadestructor.h>
#include <valasourcereference.h>
#include <valatypereference.h>
#include <valatypeparameter.h>
#include <valaconstant.h>
#include <valafield.h>
#include <valamemberaccessibility.h>
#include <valamethod.h>
#include <valaproperty.h>
#include <valapropertyaccessor.h>
#include <valasignal.h>
#include <valacodevisitor.h>
#include <valanamespace.h>
#include <valaattribute.h>
#include <valanamedargument.h>
#include <valaliteralexpression.h>
#include <valaliteral.h>
#include <valastringliteral.h>

struct _ValaClassPrivate {
	ValaClass* _base_class;
	gboolean _is_abstract;
	char* cname;
	char* lower_case_csuffix;
	char* type_id;
	gboolean _has_private_fields;
	GList* type_parameters;
	GList* base_types;
	GList* constants;
	GList* fields;
	GList* methods;
	GList* properties;
	GList* signals;
	ValaConstructor* _constructor;
	ValaDestructor* _destructor;
};
#define VALA_CLASS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CLASS, ValaClassPrivate))
enum  {
	VALA_CLASS_DUMMY_PROPERTY,
	VALA_CLASS_BASE_CLASS,
	VALA_CLASS_IS_ABSTRACT,
	VALA_CLASS_HAS_PRIVATE_FIELDS,
	VALA_CLASS_CONSTRUCTOR,
	VALA_CLASS_DESTRUCTOR,
};
static void vala_class_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static char* vala_class_real_get_cname (ValaDataType* base);
static char* vala_class_get_lower_case_csuffix (ValaClass* self);
static void vala_class_set_lower_case_csuffix (ValaClass* self, char* csuffix);
static char* vala_class_real_get_lower_case_cname (ValaDataType* base, char* infix);
static char* vala_class_real_get_upper_case_cname (ValaDataType* base, char* infix);
static gboolean vala_class_real_is_reference_type (ValaDataType* base);
static void vala_class_process_ccode_attribute (ValaClass* self, ValaAttribute* a);
static char* vala_class_real_get_type_id (ValaDataType* base);
static char* vala_class_real_get_marshaller_type_name (ValaDataType* base);
static gboolean vala_class_real_is_reference_counting (ValaDataType* base);
static char* vala_class_real_get_ref_function (ValaDataType* base);
static char* vala_class_real_get_unref_function (ValaDataType* base);
static void vala_class_dispose (GObject * obj);

/**
 * Creates a new class.
 *
 * @param name   type name
 * @param source reference to source code
 * @return       newly created class
 */
ValaClass* vala_class_new (char* name, ValaSourceReference* source)
{
	g_return_val_if_fail (name != NULL, NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_CLASS, "name", name, "source_reference", source, NULL));
}


/**
 * Adds the specified class or interface to the list of base types of
 * this class.
 *
 * @param type a class or interface reference
 */
void vala_class_add_base_type (ValaClass* self, ValaTypeReference* type)
{
	g_return_if_fail (VALA_IS_CLASS (self));
	g_return_if_fail (VALA_IS_TYPE_REFERENCE (type));
	self->priv->base_types = g_list_append (self->priv->base_types, g_object_ref (type));
}


/**
 * Returns a copy of the base type list.
 *
 * @return list of base types
 */
GList* vala_class_get_base_types (ValaClass* self)
{
	g_return_val_if_fail (VALA_IS_CLASS (self), NULL);
	return g_list_copy (self->priv->base_types);
}


/**
 * Appends the specified parameter to the list of type parameters.
 *
 * @param p a type parameter
 */
void vala_class_add_type_parameter (ValaClass* self, ValaTypeParameter* p)
{
	g_return_if_fail (VALA_IS_CLASS (self));
	g_return_if_fail (VALA_IS_TYPE_PARAMETER (p));
	self->priv->type_parameters = g_list_append (self->priv->type_parameters, g_object_ref (p));
	p->type = VALA_DATA_TYPE (self);
}


/**
 * Adds the specified constant as a member to this class.
 *
 * @param c a constant
 */
void vala_class_add_constant (ValaClass* self, ValaConstant* c)
{
	g_return_if_fail (VALA_IS_CLASS (self));
	g_return_if_fail (VALA_IS_CONSTANT (c));
	self->priv->constants = g_list_append (self->priv->constants, g_object_ref (c));
}


/**
 * Adds the specified field as a member to this class.
 *
 * @param f a field
 */
void vala_class_add_field (ValaClass* self, ValaField* f)
{
	g_return_if_fail (VALA_IS_CLASS (self));
	g_return_if_fail (VALA_IS_FIELD (f));
	self->priv->fields = g_list_append (self->priv->fields, g_object_ref (f));
	if (f->access == VALA_MEMBER_ACCESSIBILITY_PRIVATE && vala_field_get_instance (f)) {
		self->priv->_has_private_fields = TRUE;
	}
}


/**
 * Returns a copy of the list of fields.
 *
 * @return list of fields
 */
GList* vala_class_get_fields (ValaClass* self)
{
	g_return_val_if_fail (VALA_IS_CLASS (self), NULL);
	return g_list_copy (self->priv->fields);
}


/**
 * Adds the specified method as a member to this class.
 *
 * @param m a method
 */
void vala_class_add_method (ValaClass* self, ValaMethod* m)
{
	g_return_if_fail (VALA_IS_CLASS (self));
	g_return_if_fail (VALA_IS_METHOD (m));
	self->priv->methods = g_list_append (self->priv->methods, g_object_ref (m));
}


/**
 * Returns a copy of the list of methods.
 *
 * @return list of methods
 */
GList* vala_class_get_methods (ValaClass* self)
{
	g_return_val_if_fail (VALA_IS_CLASS (self), NULL);
	return g_list_copy (self->priv->methods);
}


/**
 * Adds the specified property as a member to this class.
 *
 * @param prop a property
 */
void vala_class_add_property (ValaClass* self, ValaProperty* prop)
{
	g_return_if_fail (VALA_IS_CLASS (self));
	g_return_if_fail (VALA_IS_PROPERTY (prop));
	self->priv->properties = g_list_append (self->priv->properties, g_object_ref (prop));
	if (vala_property_get_set_accessor (prop) != NULL && vala_property_accessor_get_body (vala_property_get_set_accessor (prop)) == NULL) {
		/* automatic property accessor body generation */
		char* __temp4 = NULL;
		ValaField* __temp5 = NULL;
		ValaField* f = (__temp5 = g_object_new (VALA_TYPE_FIELD, "name", (__temp4 = g_strdup_printf ("_%s", vala_property_get_name (prop))), "type_reference", vala_property_get_type_reference (prop), "source_reference", vala_code_node_get_source_reference (VALA_CODE_NODE (prop)), NULL), (__temp4 == NULL ? NULL : (__temp4 = (g_free (__temp4), NULL))), __temp5);
		f->access = VALA_MEMBER_ACCESSIBILITY_PRIVATE;
		vala_class_add_field (self, f);
		(f == NULL ? NULL : (f = (g_object_unref (f), NULL)));
	}
}


/**
 * Returns a copy of the list of properties.
 *
 * @return list of properties
 */
GList* vala_class_get_properties (ValaClass* self)
{
	g_return_val_if_fail (VALA_IS_CLASS (self), NULL);
	return g_list_copy (self->priv->properties);
}


/**
 * Adds the specified signal as a member to this class.
 *
 * @param sig a signal
 */
void vala_class_add_signal (ValaClass* self, ValaSignal* sig)
{
	g_return_if_fail (VALA_IS_CLASS (self));
	g_return_if_fail (VALA_IS_SIGNAL (sig));
	self->priv->signals = g_list_append (self->priv->signals, g_object_ref (sig));
}


/**
 * Returns a copy of the list of signals.
 *
 * @return list of signals
 */
GList* vala_class_get_signals (ValaClass* self)
{
	g_return_val_if_fail (VALA_IS_CLASS (self), NULL);
	return g_list_copy (self->priv->signals);
}


static void vala_class_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaClass * self = VALA_CLASS (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_begin_class (visitor, self);
	{
		GList * type_it;
		for (type_it = self->priv->base_types; type_it != NULL; type_it = type_it->next) {
			ValaTypeReference* type = type_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (type), visitor);
			}
		}
	}
	{
		GList * p_it;
		for (p_it = self->priv->type_parameters; p_it != NULL; p_it = p_it->next) {
			ValaTypeParameter* p = p_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (p), visitor);
			}
		}
	}
	{
		GList * f_it;
		for (f_it = self->priv->fields; f_it != NULL; f_it = f_it->next) {
			ValaField* f = f_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (f), visitor);
			}
		}
	}
	{
		GList * c_it;
		for (c_it = self->priv->constants; c_it != NULL; c_it = c_it->next) {
			ValaConstant* c = c_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (c), visitor);
			}
		}
	}
	{
		GList * m_it;
		for (m_it = self->priv->methods; m_it != NULL; m_it = m_it->next) {
			ValaMethod* m = m_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (m), visitor);
			}
		}
	}
	{
		GList * prop_it;
		for (prop_it = self->priv->properties; prop_it != NULL; prop_it = prop_it->next) {
			ValaProperty* prop = prop_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (prop), visitor);
			}
		}
	}
	{
		GList * sig_it;
		for (sig_it = self->priv->signals; sig_it != NULL; sig_it = sig_it->next) {
			ValaSignal* sig = sig_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (sig), visitor);
			}
		}
	}
	if (vala_class_get_constructor (self) != NULL) {
		vala_code_node_accept (VALA_CODE_NODE (vala_class_get_constructor (self)), visitor);
	}
	if (vala_class_get_destructor (self) != NULL) {
		vala_code_node_accept (VALA_CODE_NODE (vala_class_get_destructor (self)), visitor);
	}
	vala_code_visitor_visit_end_class (visitor, self);
}


static char* vala_class_real_get_cname (ValaDataType* base)
{
	ValaClass * self = VALA_CLASS (base);
	if (self->priv->cname == NULL) {
		char* __temp8 = NULL;
		self->priv->cname = (__temp8 = g_strdup_printf ("%s%s", vala_namespace_get_cprefix (VALA_DATA_TYPE (self)->namespace), vala_data_type_get_name (VALA_DATA_TYPE (self))), (self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL))), __temp8);
	}
	return self->priv->cname;
}


/**
 * Sets the name of this class as it is used in C code.
 *
 * @param cname the name to be used in C code
 */
void vala_class_set_cname (ValaClass* self, char* cname)
{
	g_return_if_fail (VALA_IS_CLASS (self));
	g_return_if_fail (cname != NULL);
	char* __temp10 = NULL;
	self->priv->cname = (__temp10 = g_strdup (cname), (self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL))), __temp10);
}


static char* vala_class_get_lower_case_csuffix (ValaClass* self)
{
	g_return_val_if_fail (VALA_IS_CLASS (self), NULL);
	if (self->priv->lower_case_csuffix == NULL) {
		char* __temp11 = NULL;
		self->priv->lower_case_csuffix = (__temp11 = vala_namespace_camel_case_to_lower_case (vala_data_type_get_name (VALA_DATA_TYPE (self))), (self->priv->lower_case_csuffix == NULL ? NULL : (self->priv->lower_case_csuffix = (g_free (self->priv->lower_case_csuffix), NULL))), __temp11);
	}
	return self->priv->lower_case_csuffix;
}


static void vala_class_set_lower_case_csuffix (ValaClass* self, char* csuffix)
{
	g_return_if_fail (VALA_IS_CLASS (self));
	g_return_if_fail (csuffix != NULL);
	char* __temp13 = NULL;
	self->priv->lower_case_csuffix = (__temp13 = g_strdup (csuffix), (self->priv->lower_case_csuffix == NULL ? NULL : (self->priv->lower_case_csuffix = (g_free (self->priv->lower_case_csuffix), NULL))), __temp13);
}


static char* vala_class_real_get_lower_case_cname (ValaDataType* base, char* infix)
{
	ValaClass * self = VALA_CLASS (base);
	if (infix == NULL) {
		infix = "";
	}
	return g_strdup_printf ("%s%s%s", vala_namespace_get_lower_case_cprefix (VALA_DATA_TYPE (self)->namespace), infix, vala_class_get_lower_case_csuffix (self));
}


static char* vala_class_real_get_upper_case_cname (ValaDataType* base, char* infix)
{
	ValaClass * self = VALA_CLASS (base);
	char* __temp15 = NULL;
	char* __temp16 = NULL;
	return (__temp16 = g_utf8_strup ((__temp15 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (self), infix)), -1), (__temp15 == NULL ? NULL : (__temp15 = (g_free (__temp15), NULL))), __temp16);
}


static gboolean vala_class_real_is_reference_type (ValaDataType* base)
{
	ValaClass * self = VALA_CLASS (base);
	return TRUE;
}


static void vala_class_process_ccode_attribute (ValaClass* self, ValaAttribute* a)
{
	g_return_if_fail (VALA_IS_CLASS (self));
	g_return_if_fail (VALA_IS_ATTRIBUTE (a));
	{
		GList * arg_it;
		for (arg_it = a->args; arg_it != NULL; arg_it = arg_it->next) {
			ValaNamedArgument* arg = arg_it->data;
			{
				if (g_utf8_collate (vala_named_argument_get_name (arg), "cname") == 0) {
					/* this will already be checked during semantic analysis */
					if (VALA_IS_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))) {
						ValaLiteral* lit = vala_literal_expression_get_literal ((VALA_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))));
						if (VALA_IS_STRING_LITERAL (lit)) {
							char* __temp19 = NULL;
							vala_class_set_cname (self, (__temp19 = vala_string_literal_eval ((VALA_STRING_LITERAL (lit)))));
							(__temp19 == NULL ? NULL : (__temp19 = (g_free (__temp19), NULL)));
						}
					}
				} else if (g_utf8_collate (vala_named_argument_get_name (arg), "cheader_filename") == 0) {
					/* this will already be checked during semantic analysis */
					if (VALA_IS_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))) {
						ValaLiteral* lit = vala_literal_expression_get_literal ((VALA_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))));
						if (VALA_IS_STRING_LITERAL (lit)) {
							char* val = vala_string_literal_eval ((VALA_STRING_LITERAL (lit)));
							{
								char** filename_it;
								for (filename_it = g_strsplit (val, ",", 0); *filename_it != NULL; filename_it = filename_it + 1) {
									char* filename = *filename_it;
									{
										vala_data_type_add_cheader_filename (VALA_DATA_TYPE (self), filename);
									}
								}
							}
							(val == NULL ? NULL : (val = (g_free (val), NULL)));
						}
					}
				}
			}
		}
	}
}


/**
 * Process all associated attributes.
 */
void vala_class_process_attributes (ValaClass* self)
{
	g_return_if_fail (VALA_IS_CLASS (self));
	{
		GList * a_it;
		for (a_it = VALA_CODE_NODE (self)->attributes; a_it != NULL; a_it = a_it->next) {
			ValaAttribute* a = a_it->data;
			{
				if (g_utf8_collate (vala_attribute_get_name (a), "CCode") == 0) {
					vala_class_process_ccode_attribute (self, a);
				}
			}
		}
	}
}


static char* vala_class_real_get_type_id (ValaDataType* base)
{
	ValaClass * self = VALA_CLASS (base);
	if (self->priv->type_id == NULL) {
		char* __temp20 = NULL;
		self->priv->type_id = (__temp20 = vala_data_type_get_upper_case_cname (VALA_DATA_TYPE (self), "TYPE_"), (self->priv->type_id == NULL ? NULL : (self->priv->type_id = (g_free (self->priv->type_id), NULL))), __temp20);
	}
	return self->priv->type_id;
}


static char* vala_class_real_get_marshaller_type_name (ValaDataType* base)
{
	ValaClass * self = VALA_CLASS (base);
	return "OBJECT";
}


static gboolean vala_class_real_is_reference_counting (ValaDataType* base)
{
	ValaClass * self = VALA_CLASS (base);
	return TRUE;
}


static char* vala_class_real_get_ref_function (ValaDataType* base)
{
	ValaClass * self = VALA_CLASS (base);
	return "g_object_ref";
}


static char* vala_class_real_get_unref_function (ValaDataType* base)
{
	ValaClass * self = VALA_CLASS (base);
	return "g_object_unref";
}


ValaClass* vala_class_get_base_class (ValaClass* self)
{
	g_return_val_if_fail (VALA_IS_CLASS (self), NULL);
	return self->priv->_base_class;
}


void vala_class_set_base_class (ValaClass* self, ValaClass* value)
{
	g_return_if_fail (VALA_IS_CLASS (self));
	ValaClass* __temp28 = NULL;
	ValaClass* __temp27 = NULL;
	self->priv->_base_class = (__temp28 = (__temp27 = value, (__temp27 == NULL ? __temp27 : g_object_ref (__temp27))), (self->priv->_base_class == NULL ? NULL : (self->priv->_base_class = (g_object_unref (self->priv->_base_class), NULL))), __temp28);
}


gboolean vala_class_get_is_abstract (ValaClass* self)
{
	g_return_val_if_fail (VALA_IS_CLASS (self), FALSE);
	return self->priv->_is_abstract;
}


void vala_class_set_is_abstract (ValaClass* self, gboolean value)
{
	g_return_if_fail (VALA_IS_CLASS (self));
	self->priv->_is_abstract = value;
}


gboolean vala_class_get_has_private_fields (ValaClass* self)
{
	g_return_val_if_fail (VALA_IS_CLASS (self), FALSE);
	return self->priv->_has_private_fields;
}


ValaConstructor* vala_class_get_constructor (ValaClass* self)
{
	g_return_val_if_fail (VALA_IS_CLASS (self), NULL);
	return self->priv->_constructor;
}


void vala_class_set_constructor (ValaClass* self, ValaConstructor* value)
{
	g_return_if_fail (VALA_IS_CLASS (self));
	ValaConstructor* __temp33 = NULL;
	ValaConstructor* __temp32 = NULL;
	self->priv->_constructor = (__temp33 = (__temp32 = value, (__temp32 == NULL ? __temp32 : g_object_ref (__temp32))), (self->priv->_constructor == NULL ? NULL : (self->priv->_constructor = (g_object_unref (self->priv->_constructor), NULL))), __temp33);
}


ValaDestructor* vala_class_get_destructor (ValaClass* self)
{
	g_return_val_if_fail (VALA_IS_CLASS (self), NULL);
	return self->priv->_destructor;
}


void vala_class_set_destructor (ValaClass* self, ValaDestructor* value)
{
	g_return_if_fail (VALA_IS_CLASS (self));
	ValaDestructor* __temp36 = NULL;
	ValaDestructor* __temp35 = NULL;
	self->priv->_destructor = (__temp36 = (__temp35 = value, (__temp35 == NULL ? __temp35 : g_object_ref (__temp35))), (self->priv->_destructor == NULL ? NULL : (self->priv->_destructor = (g_object_unref (self->priv->_destructor), NULL))), __temp36);
}


void vala_class_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaClass * self = VALA_CLASS (object);
	switch (property_id) {
		case VALA_CLASS_BASE_CLASS:
		g_value_set_object (value, vala_class_get_base_class (self));
		break;
		case VALA_CLASS_IS_ABSTRACT:
		g_value_set_boolean (value, vala_class_get_is_abstract (self));
		break;
		case VALA_CLASS_HAS_PRIVATE_FIELDS:
		g_value_set_boolean (value, vala_class_get_has_private_fields (self));
		break;
		case VALA_CLASS_CONSTRUCTOR:
		g_value_set_object (value, vala_class_get_constructor (self));
		break;
		case VALA_CLASS_DESTRUCTOR:
		g_value_set_object (value, vala_class_get_destructor (self));
		break;
	}
}


void vala_class_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaClass * self = VALA_CLASS (object);
	switch (property_id) {
		case VALA_CLASS_BASE_CLASS:
		vala_class_set_base_class (self, g_value_get_object (value));
		break;
		case VALA_CLASS_IS_ABSTRACT:
		vala_class_set_is_abstract (self, g_value_get_boolean (value));
		break;
		case VALA_CLASS_CONSTRUCTOR:
		vala_class_set_constructor (self, g_value_get_object (value));
		break;
		case VALA_CLASS_DESTRUCTOR:
		vala_class_set_destructor (self, g_value_get_object (value));
		break;
	}
}


static void vala_class_class_init (ValaClassClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaClassPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_class_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_class_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_class_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_class_real_accept;
	VALA_DATA_TYPE_CLASS (klass)->get_cname = vala_class_real_get_cname;
	VALA_DATA_TYPE_CLASS (klass)->get_lower_case_cname = vala_class_real_get_lower_case_cname;
	VALA_DATA_TYPE_CLASS (klass)->get_upper_case_cname = vala_class_real_get_upper_case_cname;
	VALA_DATA_TYPE_CLASS (klass)->is_reference_type = vala_class_real_is_reference_type;
	VALA_DATA_TYPE_CLASS (klass)->get_type_id = vala_class_real_get_type_id;
	VALA_DATA_TYPE_CLASS (klass)->get_marshaller_type_name = vala_class_real_get_marshaller_type_name;
	VALA_DATA_TYPE_CLASS (klass)->is_reference_counting = vala_class_real_is_reference_counting;
	VALA_DATA_TYPE_CLASS (klass)->get_ref_function = vala_class_real_get_ref_function;
	VALA_DATA_TYPE_CLASS (klass)->get_unref_function = vala_class_real_get_unref_function;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CLASS_BASE_CLASS, g_param_spec_object ("base-class", "foo", "bar", VALA_TYPE_CLASS, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CLASS_IS_ABSTRACT, g_param_spec_boolean ("is-abstract", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CLASS_HAS_PRIVATE_FIELDS, g_param_spec_boolean ("has-private-fields", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CLASS_CONSTRUCTOR, g_param_spec_object ("constructor", "foo", "bar", VALA_TYPE_CONSTRUCTOR, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CLASS_DESTRUCTOR, g_param_spec_object ("destructor", "foo", "bar", VALA_TYPE_DESTRUCTOR, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_class_init (ValaClass * self)
{
	self->priv = VALA_CLASS_GET_PRIVATE (self);
}


static void vala_class_dispose (GObject * obj)
{
	ValaClass * self = VALA_CLASS (obj);
	(self->priv->_base_class == NULL ? NULL : (self->priv->_base_class = (g_object_unref (self->priv->_base_class), NULL)));
	(self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL)));
	(self->priv->lower_case_csuffix == NULL ? NULL : (self->priv->lower_case_csuffix = (g_free (self->priv->lower_case_csuffix), NULL)));
	(self->priv->type_id == NULL ? NULL : (self->priv->type_id = (g_free (self->priv->type_id), NULL)));
	(self->priv->type_parameters == NULL ? NULL : (self->priv->type_parameters = (g_list_foreach (self->priv->type_parameters, (GFunc) g_free, NULL), g_list_free (self->priv->type_parameters), NULL)));
	(self->priv->base_types == NULL ? NULL : (self->priv->base_types = (g_list_foreach (self->priv->base_types, (GFunc) g_object_unref, NULL), g_list_free (self->priv->base_types), NULL)));
	(self->priv->constants == NULL ? NULL : (self->priv->constants = (g_list_foreach (self->priv->constants, (GFunc) g_object_unref, NULL), g_list_free (self->priv->constants), NULL)));
	(self->priv->fields == NULL ? NULL : (self->priv->fields = (g_list_foreach (self->priv->fields, (GFunc) g_object_unref, NULL), g_list_free (self->priv->fields), NULL)));
	(self->priv->methods == NULL ? NULL : (self->priv->methods = (g_list_foreach (self->priv->methods, (GFunc) g_object_unref, NULL), g_list_free (self->priv->methods), NULL)));
	(self->priv->properties == NULL ? NULL : (self->priv->properties = (g_list_foreach (self->priv->properties, (GFunc) g_object_unref, NULL), g_list_free (self->priv->properties), NULL)));
	(self->priv->signals == NULL ? NULL : (self->priv->signals = (g_list_foreach (self->priv->signals, (GFunc) g_object_unref, NULL), g_list_free (self->priv->signals), NULL)));
	(self->priv->_constructor == NULL ? NULL : (self->priv->_constructor = (g_object_unref (self->priv->_constructor), NULL)));
	(self->priv->_destructor == NULL ? NULL : (self->priv->_destructor = (g_object_unref (self->priv->_destructor), NULL)));
	ValaClassClass * klass;
	GObjectClass * parent_class;
	klass = VALA_CLASS_CLASS (g_type_class_peek (VALA_TYPE_CLASS));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_class_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaClassClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_class_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaClass), 0, (GInstanceInitFunc) vala_class_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_DATA_TYPE, "ValaClass", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



