/* valaconstant.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valaconstant.h>
#include <valatypereference.h>
#include <valaexpression.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>
#include <valasymbol.h>
#include <valadatatype.h>
#include <valanamespace.h>

struct _ValaConstantPrivate {
	char* _name;
	ValaTypeReference* _type_reference;
	ValaExpression* _initializer;
	char* cname;
};
#define VALA_CONSTANT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CONSTANT, ValaConstantPrivate))
enum  {
	VALA_CONSTANT_DUMMY_PROPERTY,
	VALA_CONSTANT_NAME,
	VALA_CONSTANT_TYPE_REFERENCE,
	VALA_CONSTANT_INITIALIZER,
};
static void vala_constant_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_constant_dispose (GObject * obj);

/**
 * Creates a new constant.
 *
 * @param name   constant name
 * @param type   constant type
 * @param init   constant value
 * @param source reference to source code
 * @return       newly created constant
 */
ValaConstant* vala_constant_new (char* name, ValaTypeReference* type, ValaExpression* init, ValaSourceReference* source)
{
	g_return_val_if_fail (name != NULL, NULL);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (type), NULL);
	g_return_val_if_fail (VALA_IS_EXPRESSION (init), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_CONSTANT, "name", name, "type_reference", type, "initializer", init, "source_reference", source, NULL));
}


static void vala_constant_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaConstant * self = VALA_CONSTANT (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_node_accept (VALA_CODE_NODE (vala_constant_get_type_reference (self)), visitor);
	vala_code_node_accept (VALA_CODE_NODE (vala_constant_get_initializer (self)), visitor);
	vala_code_visitor_visit_constant (visitor, self);
}


/**
 * Returns the name of this constant as it is used in C code.
 *
 * @return the name to be used in C code
 */
char* vala_constant_get_cname (ValaConstant* self)
{
	g_return_val_if_fail (VALA_IS_CONSTANT (self), NULL);
	if (self->priv->cname == NULL) {
		if (VALA_IS_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (self))->parent_symbol))) {
			ValaDataType* t = VALA_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (self))->parent_symbol));
			char* __temp2 = NULL;
			char* __temp1 = NULL;
			self->priv->cname = (__temp2 = g_strdup_printf ("%s_%s", (__temp1 = vala_data_type_get_upper_case_cname (t, NULL)), vala_constant_get_name (self)), (self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL))), __temp2);
			(__temp1 == NULL ? NULL : (__temp1 = (g_free (__temp1), NULL)));
		} else {
			ValaNamespace* ns = VALA_NAMESPACE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (self))->parent_symbol));
			char* __temp4 = NULL;
			char* __temp3 = NULL;
			self->priv->cname = (__temp4 = g_strdup_printf ("%s%s", (__temp3 = g_utf8_strup (vala_namespace_get_cprefix (ns), -1)), vala_constant_get_name (self)), (self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL))), __temp4);
			(__temp3 == NULL ? NULL : (__temp3 = (g_free (__temp3), NULL)));
		}
	}
	return self->priv->cname;
}


char* vala_constant_get_name (ValaConstant* self)
{
	g_return_val_if_fail (VALA_IS_CONSTANT (self), NULL);
	return self->priv->_name;
}


void vala_constant_set_name (ValaConstant* self, char* value)
{
	g_return_if_fail (VALA_IS_CONSTANT (self));
	char* __temp7 = NULL;
	self->priv->_name = (__temp7 = g_strdup (value), (self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL))), __temp7);
}


ValaTypeReference* vala_constant_get_type_reference (ValaConstant* self)
{
	g_return_val_if_fail (VALA_IS_CONSTANT (self), NULL);
	return self->priv->_type_reference;
}


void vala_constant_set_type_reference (ValaConstant* self, ValaTypeReference* value)
{
	g_return_if_fail (VALA_IS_CONSTANT (self));
	ValaTypeReference* __temp9 = NULL;
	self->priv->_type_reference = (__temp9 = g_object_ref (value), (self->priv->_type_reference == NULL ? NULL : (self->priv->_type_reference = (g_object_unref (self->priv->_type_reference), NULL))), __temp9);
}


ValaExpression* vala_constant_get_initializer (ValaConstant* self)
{
	g_return_val_if_fail (VALA_IS_CONSTANT (self), NULL);
	return self->priv->_initializer;
}


void vala_constant_set_initializer (ValaConstant* self, ValaExpression* value)
{
	g_return_if_fail (VALA_IS_CONSTANT (self));
	ValaExpression* __temp11 = NULL;
	self->priv->_initializer = (__temp11 = g_object_ref (value), (self->priv->_initializer == NULL ? NULL : (self->priv->_initializer = (g_object_unref (self->priv->_initializer), NULL))), __temp11);
}


void vala_constant_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaConstant * self = VALA_CONSTANT (object);
	switch (property_id) {
		case VALA_CONSTANT_NAME:
		g_value_set_string (value, vala_constant_get_name (self));
		break;
		case VALA_CONSTANT_TYPE_REFERENCE:
		g_value_set_object (value, vala_constant_get_type_reference (self));
		break;
		case VALA_CONSTANT_INITIALIZER:
		g_value_set_object (value, vala_constant_get_initializer (self));
		break;
	}
}


void vala_constant_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaConstant * self = VALA_CONSTANT (object);
	switch (property_id) {
		case VALA_CONSTANT_NAME:
		vala_constant_set_name (self, g_value_get_string (value));
		break;
		case VALA_CONSTANT_TYPE_REFERENCE:
		vala_constant_set_type_reference (self, g_value_get_object (value));
		break;
		case VALA_CONSTANT_INITIALIZER:
		vala_constant_set_initializer (self, g_value_get_object (value));
		break;
	}
}


static void vala_constant_class_init (ValaConstantClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaConstantPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_constant_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_constant_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_constant_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_constant_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CONSTANT_NAME, g_param_spec_string ("name", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CONSTANT_TYPE_REFERENCE, g_param_spec_object ("type-reference", "foo", "bar", VALA_TYPE_TYPE_REFERENCE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_CONSTANT_INITIALIZER, g_param_spec_object ("initializer", "foo", "bar", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
}


static void vala_constant_init (ValaConstant * self)
{
	self->priv = VALA_CONSTANT_GET_PRIVATE (self);
}


static void vala_constant_dispose (GObject * obj)
{
	ValaConstant * self = VALA_CONSTANT (obj);
	(self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL)));
	(self->priv->_type_reference == NULL ? NULL : (self->priv->_type_reference = (g_object_unref (self->priv->_type_reference), NULL)));
	(self->priv->_initializer == NULL ? NULL : (self->priv->_initializer = (g_object_unref (self->priv->_initializer), NULL)));
	(self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL)));
	ValaConstantClass * klass;
	GObjectClass * parent_class;
	klass = VALA_CONSTANT_CLASS (g_type_class_peek (VALA_TYPE_CONSTANT));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_constant_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaConstantClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_constant_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaConstant), 0, (GInstanceInitFunc) vala_constant_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_NODE, "ValaConstant", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



