/* valatype.vala
 *
 * Copyright (C) 2006  Jürg Billeter, Raffaele Sandrini
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 *	Raffaele Sandrini <rasa@gmx.ch>
 */

#include <valadatatype.h>
#include <valaarray.h>
#include <valanamespace.h>
#include <valasymbol.h>

struct _ValaDataTypePrivate {
	char* _name;
	GList* cheader_filenames;
	ValaArray* array_type;
};
#define VALA_DATA_TYPE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_DATA_TYPE, ValaDataTypePrivate))
enum  {
	VALA_DATA_TYPE_DUMMY_PROPERTY,
	VALA_DATA_TYPE_NAME,
};
static gboolean vala_data_type_real_is_reference_type (ValaDataType* self);
static char* vala_data_type_real_get_dup_function (ValaDataType* self);
static char* vala_data_type_real_get_free_function (ValaDataType* self);
static gboolean vala_data_type_real_is_reference_counting (ValaDataType* self);
static char* vala_data_type_real_get_ref_function (ValaDataType* self);
static char* vala_data_type_real_get_unref_function (ValaDataType* self);
static char* vala_data_type_real_get_type_id (ValaDataType* self);
static char* vala_data_type_real_get_marshaller_type_name (ValaDataType* self);
static GList* vala_data_type_real_get_cheader_filenames (ValaDataType* self);
static void vala_data_type_dispose (GObject * obj);

char* vala_data_type_get_cname (ValaDataType* self)
{
	VALA_DATA_TYPE_GET_CLASS (self)->get_cname (self);
}


/**
 * Checks whether this data type has value or reference type semantics.
 *
 * @return true if this data type has reference type semantics
 */
static gboolean vala_data_type_real_is_reference_type (ValaDataType* self)
{
	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), FALSE);
	return FALSE;
}


gboolean vala_data_type_is_reference_type (ValaDataType* self)
{
	VALA_DATA_TYPE_GET_CLASS (self)->is_reference_type (self);
}


/**
 * Returns the C function name that duplicates instances of this data
 * type. The specified C function must accept one argument referencing
 * the instance of this data type and return a reference to the
 * duplicate.
 *
 * @return the name of the C function if supported or null otherwise
 */
static char* vala_data_type_real_get_dup_function (ValaDataType* self)
{
	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), NULL);
	return NULL;
}


char* vala_data_type_get_dup_function (ValaDataType* self)
{
	VALA_DATA_TYPE_GET_CLASS (self)->get_dup_function (self);
}


/**
 * Returns the C function name that frees instances of this data type.
 * This is only valid for data types with reference type semantics that
 * do not support reference counting. The specified C function must
 * accept one argument pointing to the instance to be freed.
 *
 * @return the name of the C function or null if this data type is not a
 *         reference type or if it supports reference counting
 */
static char* vala_data_type_real_get_free_function (ValaDataType* self)
{
	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), NULL);
	return NULL;
}


char* vala_data_type_get_free_function (ValaDataType* self)
{
	VALA_DATA_TYPE_GET_CLASS (self)->get_free_function (self);
}


/**
 * Checks whether this data type supports reference counting. This is
 * only valid for reference types.
 *
 * @return true if this data type supports reference counting
 */
static gboolean vala_data_type_real_is_reference_counting (ValaDataType* self)
{
	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), FALSE);
	return FALSE;
}


gboolean vala_data_type_is_reference_counting (ValaDataType* self)
{
	VALA_DATA_TYPE_GET_CLASS (self)->is_reference_counting (self);
}


/**
 * Returns the C function name that increments the reference count of
 * instances of this data type. This is only valid for data types
 * supporting reference counting. The specified C function must accept
 * one argument referencing the instance of this data type and return
 * the reference.
 *
 * @return the name of the C function or null if this data type does not
 *         support reference counting
 */
static char* vala_data_type_real_get_ref_function (ValaDataType* self)
{
	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), NULL);
	return NULL;
}


char* vala_data_type_get_ref_function (ValaDataType* self)
{
	VALA_DATA_TYPE_GET_CLASS (self)->get_ref_function (self);
}


/**
 * Returns the C function name that decrements the reference count of
 * instances of this data type. This is only valid for data types
 * supporting reference counting. The specified C function must accept
 * one argument referencing the instance of this data type.
 *
 * @return the name of the C function or null if this data type does not
 *         support reference counting
 */
static char* vala_data_type_real_get_unref_function (ValaDataType* self)
{
	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), NULL);
	return NULL;
}


char* vala_data_type_get_unref_function (ValaDataType* self)
{
	VALA_DATA_TYPE_GET_CLASS (self)->get_unref_function (self);
}


/**
 * Returns the C symbol representing the runtime type id for this data
 * type. The specified symbol must express a registered GType.
 *
 * @return the name of the GType name in C code or null if this data
 *         type is not registered with GType
 */
static char* vala_data_type_real_get_type_id (ValaDataType* self)
{
	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), NULL);
	return NULL;
}


char* vala_data_type_get_type_id (ValaDataType* self)
{
	VALA_DATA_TYPE_GET_CLASS (self)->get_type_id (self);
}


/**
 * Returns the name of this data type as used in C code marshallers
 *
 * @return type name for marshallers
 */
static char* vala_data_type_real_get_marshaller_type_name (ValaDataType* self)
{
	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), NULL);
	return NULL;
}


char* vala_data_type_get_marshaller_type_name (ValaDataType* self)
{
	VALA_DATA_TYPE_GET_CLASS (self)->get_marshaller_type_name (self);
}


char* vala_data_type_get_upper_case_cname (ValaDataType* self, char* infix)
{
	VALA_DATA_TYPE_GET_CLASS (self)->get_upper_case_cname (self, infix);
}


char* vala_data_type_get_lower_case_cname (ValaDataType* self, char* infix)
{
	VALA_DATA_TYPE_GET_CLASS (self)->get_lower_case_cname (self, infix);
}


/**
 * Returns a list of C header filenames users of this data type must
 * include.
 *
 * @return list of C header filenames for this data type
 */
static GList* vala_data_type_real_get_cheader_filenames (ValaDataType* self)
{
	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), NULL);
	if (self->priv->cheader_filenames == NULL) {
		{
			GList * filename_it;
			for (filename_it = vala_namespace_get_cheader_filenames (self->namespace); filename_it != NULL; filename_it = filename_it->next) {
				char* filename = filename_it->data;
				{
					/* default to header filenames of the namespace */
					vala_data_type_add_cheader_filename (self, filename);
				}
			}
		}
	}
	return g_list_copy (self->priv->cheader_filenames);
}


GList* vala_data_type_get_cheader_filenames (ValaDataType* self)
{
	VALA_DATA_TYPE_GET_CLASS (self)->get_cheader_filenames (self);
}


/**
 * Adds a filename to the list of C header filenames users of this data
 * type must include.
 *
 * @param filename a C header filename
 */
void vala_data_type_add_cheader_filename (ValaDataType* self, char* filename)
{
	g_return_if_fail (VALA_IS_DATA_TYPE (self));
	g_return_if_fail (filename != NULL);
	self->priv->cheader_filenames = g_list_append (self->priv->cheader_filenames, g_strdup (filename));
}


/**
 * Retrieves for a given DataType its corresponding Array.
 */
ValaArray* vala_data_type_get_array (ValaDataType* self)
{
	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), NULL);
	if (self->priv->array_type == NULL) {
		ValaArray* __temp9 = NULL;
		self->priv->array_type = (__temp9 = g_object_new (VALA_TYPE_ARRAY, "element_type", self, NULL), (self->priv->array_type == NULL ? NULL : (self->priv->array_type = (g_object_unref (self->priv->array_type), NULL))), __temp9);
	}
	/* create a new Symbol */
	ValaSymbol* __temp10 = NULL;
	vala_code_node_set_symbol (VALA_CODE_NODE (self->priv->array_type), (__temp10 = g_object_new (VALA_TYPE_SYMBOL, "node", self->priv->array_type, NULL)));
	(__temp10 == NULL ? NULL : (__temp10 = (g_object_unref (__temp10), NULL)));
	vala_symbol_add (vala_code_node_get_symbol (VALA_CODE_NODE (self))->parent_symbol, vala_data_type_get_name (VALA_DATA_TYPE (self->priv->array_type)), vala_code_node_get_symbol (VALA_CODE_NODE (self->priv->array_type)));
	/* link the array type to the same source as the container type */
	vala_code_node_set_source_reference (VALA_CODE_NODE (self->priv->array_type), vala_code_node_get_source_reference (VALA_CODE_NODE (self)));
	/* link the namespace */
	VALA_DATA_TYPE (self->priv->array_type)->namespace = self->namespace;
	return self->priv->array_type;
}


char* vala_data_type_get_name (ValaDataType* self)
{
	g_return_val_if_fail (VALA_IS_DATA_TYPE (self), NULL);
	return self->priv->_name;
}


void vala_data_type_set_name (ValaDataType* self, char* value)
{
	g_return_if_fail (VALA_IS_DATA_TYPE (self));
	char* __temp13 = NULL;
	self->priv->_name = (__temp13 = g_strdup (value), (self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL))), __temp13);
}


void vala_data_type_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaDataType * self = VALA_DATA_TYPE (object);
	switch (property_id) {
		case VALA_DATA_TYPE_NAME:
		g_value_set_string (value, vala_data_type_get_name (self));
		break;
	}
}


void vala_data_type_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaDataType * self = VALA_DATA_TYPE (object);
	switch (property_id) {
		case VALA_DATA_TYPE_NAME:
		vala_data_type_set_name (self, g_value_get_string (value));
		break;
	}
}


static void vala_data_type_class_init (ValaDataTypeClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaDataTypePrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_data_type_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_data_type_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_data_type_dispose;
	VALA_DATA_TYPE_CLASS (klass)->is_reference_type = vala_data_type_real_is_reference_type;
	VALA_DATA_TYPE_CLASS (klass)->get_dup_function = vala_data_type_real_get_dup_function;
	VALA_DATA_TYPE_CLASS (klass)->get_free_function = vala_data_type_real_get_free_function;
	VALA_DATA_TYPE_CLASS (klass)->is_reference_counting = vala_data_type_real_is_reference_counting;
	VALA_DATA_TYPE_CLASS (klass)->get_ref_function = vala_data_type_real_get_ref_function;
	VALA_DATA_TYPE_CLASS (klass)->get_unref_function = vala_data_type_real_get_unref_function;
	VALA_DATA_TYPE_CLASS (klass)->get_type_id = vala_data_type_real_get_type_id;
	VALA_DATA_TYPE_CLASS (klass)->get_marshaller_type_name = vala_data_type_real_get_marshaller_type_name;
	VALA_DATA_TYPE_CLASS (klass)->get_cheader_filenames = vala_data_type_real_get_cheader_filenames;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_DATA_TYPE_NAME, g_param_spec_string ("name", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
}


static void vala_data_type_init (ValaDataType * self)
{
	self->priv = VALA_DATA_TYPE_GET_PRIVATE (self);
}


static void vala_data_type_dispose (GObject * obj)
{
	ValaDataType * self = VALA_DATA_TYPE (obj);
	(self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL)));
	(self->priv->cheader_filenames == NULL ? NULL : (self->priv->cheader_filenames = (g_list_foreach (self->priv->cheader_filenames, (GFunc) g_free, NULL), g_list_free (self->priv->cheader_filenames), NULL)));
	(self->priv->array_type == NULL ? NULL : (self->priv->array_type = (g_object_unref (self->priv->array_type), NULL)));
	ValaDataTypeClass * klass;
	GObjectClass * parent_class;
	klass = VALA_DATA_TYPE_CLASS (g_type_class_peek (VALA_TYPE_DATA_TYPE));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_data_type_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaDataTypeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_data_type_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaDataType), 0, (GInstanceInitFunc) vala_data_type_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_NODE, "ValaDataType", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	}
	return g_define_type_id;
}



