/* valaenumvalue.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valaenumvalue.h>
#include <valaexpression.h>
#include <valacodevisitor.h>
#include <valasymbol.h>
#include <valaenum.h>

struct _ValaEnumValuePrivate {
	char* _name;
	ValaExpression* _value;
	char* cname;
};
#define VALA_ENUM_VALUE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_ENUM_VALUE, ValaEnumValuePrivate))
enum  {
	VALA_ENUM_VALUE_DUMMY_PROPERTY,
	VALA_ENUM_VALUE_NAME,
	VALA_ENUM_VALUE_VALUE,
};
static void vala_enum_value_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_enum_value_dispose (GObject * obj);

/**
 * Creates a new enum value.
 *
 * @param name enum value name
 * @return     newly created enum value
 */
ValaEnumValue* vala_enum_value_new (char* name)
{
	g_return_val_if_fail (name != NULL, NULL);
	return (g_object_new (VALA_TYPE_ENUM_VALUE, "name", name, NULL));
}


/**
 * Creates a new enum value with the specified numerical representation.
 *
 * @param name  enum value name
 * @param value numerical representation
 * @return      newly created enum value
 */
ValaEnumValue* vala_enum_value_new_with_value (char* name, ValaExpression* value)
{
	g_return_val_if_fail (name != NULL, NULL);
	g_return_val_if_fail (value == NULL || VALA_IS_EXPRESSION (value), NULL);
	return (g_object_new (VALA_TYPE_ENUM_VALUE, "name", name, "value", value, NULL));
}


static void vala_enum_value_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaEnumValue * self = VALA_ENUM_VALUE (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_enum_value (visitor, self);
}


/**
 * Returns the name of this enum value as it is used in C code.
 *
 * @return the name to be used in C code
 */
char* vala_enum_value_get_cname (ValaEnumValue* self)
{
	g_return_val_if_fail (VALA_IS_ENUM_VALUE (self), NULL);
	if (self->priv->cname == NULL) {
		ValaEnum* en = VALA_ENUM (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (self))->parent_symbol));
		char* __temp2 = NULL;
		self->priv->cname = (__temp2 = g_strdup_printf ("%s%s", vala_enum_get_cprefix (en), vala_enum_value_get_name (self)), (self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL))), __temp2);
	}
	return self->priv->cname;
}


char* vala_enum_value_get_name (ValaEnumValue* self)
{
	g_return_val_if_fail (VALA_IS_ENUM_VALUE (self), NULL);
	return self->priv->_name;
}


void vala_enum_value_set_name (ValaEnumValue* self, char* value)
{
	g_return_if_fail (VALA_IS_ENUM_VALUE (self));
	char* __temp5 = NULL;
	self->priv->_name = (__temp5 = g_strdup (value), (self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL))), __temp5);
}


ValaExpression* vala_enum_value_get_value (ValaEnumValue* self)
{
	g_return_val_if_fail (VALA_IS_ENUM_VALUE (self), NULL);
	return self->priv->_value;
}


void vala_enum_value_set_value (ValaEnumValue* self, ValaExpression* value)
{
	g_return_if_fail (VALA_IS_ENUM_VALUE (self));
	ValaExpression* __temp8 = NULL;
	ValaExpression* __temp7 = NULL;
	self->priv->_value = (__temp8 = (__temp7 = value, (__temp7 == NULL ? __temp7 : g_object_ref (__temp7))), (self->priv->_value == NULL ? NULL : (self->priv->_value = (g_object_unref (self->priv->_value), NULL))), __temp8);
}


void vala_enum_value_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaEnumValue * self = VALA_ENUM_VALUE (object);
	switch (property_id) {
		case VALA_ENUM_VALUE_NAME:
		g_value_set_string (value, vala_enum_value_get_name (self));
		break;
		case VALA_ENUM_VALUE_VALUE:
		g_value_set_object (value, vala_enum_value_get_value (self));
		break;
	}
}


void vala_enum_value_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaEnumValue * self = VALA_ENUM_VALUE (object);
	switch (property_id) {
		case VALA_ENUM_VALUE_NAME:
		vala_enum_value_set_name (self, g_value_get_string (value));
		break;
		case VALA_ENUM_VALUE_VALUE:
		vala_enum_value_set_value (self, g_value_get_object (value));
		break;
	}
}


static void vala_enum_value_class_init (ValaEnumValueClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaEnumValuePrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_enum_value_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_enum_value_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_enum_value_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_enum_value_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_ENUM_VALUE_NAME, g_param_spec_string ("name", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_ENUM_VALUE_VALUE, g_param_spec_object ("value", "foo", "bar", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_enum_value_init (ValaEnumValue * self)
{
	self->priv = VALA_ENUM_VALUE_GET_PRIVATE (self);
}


static void vala_enum_value_dispose (GObject * obj)
{
	ValaEnumValue * self = VALA_ENUM_VALUE (obj);
	(self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL)));
	(self->priv->_value == NULL ? NULL : (self->priv->_value = (g_object_unref (self->priv->_value), NULL)));
	(self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL)));
	ValaEnumValueClass * klass;
	GObjectClass * parent_class;
	klass = VALA_ENUM_VALUE_CLASS (g_type_class_peek (VALA_TYPE_ENUM_VALUE));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_enum_value_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaEnumValueClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_enum_value_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaEnumValue), 0, (GInstanceInitFunc) vala_enum_value_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_NODE, "ValaEnumValue", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



