/* valafield.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valafield.h>
#include <valatypereference.h>
#include <valaexpression.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>
#include <valaattribute.h>
#include <valanamedargument.h>
#include <valaliteralexpression.h>
#include <valaliteral.h>
#include <valastringliteral.h>

struct _ValaFieldPrivate {
	char* _name;
	ValaTypeReference* _type_reference;
	ValaExpression* _initializer;
	char* cname;
	gboolean _instance;
};
#define VALA_FIELD_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_FIELD, ValaFieldPrivate))
enum  {
	VALA_FIELD_DUMMY_PROPERTY,
	VALA_FIELD_NAME,
	VALA_FIELD_TYPE_REFERENCE,
	VALA_FIELD_INITIALIZER,
	VALA_FIELD_INSTANCE,
};
static void vala_field_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_field_set_cname (ValaField* self, char* cname);
static void vala_field_process_ccode_attribute (ValaField* self, ValaAttribute* a);
static void vala_field_dispose (GObject * obj);

/**
 * Creates a new field.
 *
 * @param name   field name
 * @param type   field type
 * @param init   initializer expression
 * @param source reference to source code
 * @return       newly created field
 */
ValaField* vala_field_new (char* name, ValaTypeReference* type, ValaExpression* init, ValaSourceReference* source)
{
	g_return_val_if_fail (name != NULL, NULL);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (type), NULL);
	g_return_val_if_fail (init == NULL || VALA_IS_EXPRESSION (init), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_FIELD, "name", name, "type_reference", type, "initializer", init, "source_reference", source, NULL));
}


static void vala_field_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaField * self = VALA_FIELD (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_node_accept (VALA_CODE_NODE (vala_field_get_type_reference (self)), visitor);
	if (vala_field_get_initializer (self) != NULL) {
		vala_code_node_accept (VALA_CODE_NODE (vala_field_get_initializer (self)), visitor);
	}
	vala_code_visitor_visit_field (visitor, self);
}


/**
 * Returns the name of this field as it is used in C code.
 *
 * @return the name to be used in C code
 */
char* vala_field_get_cname (ValaField* self)
{
	g_return_val_if_fail (VALA_IS_FIELD (self), NULL);
	if (self->priv->cname == NULL) {
		char* __temp1 = NULL;
		self->priv->cname = (__temp1 = g_strdup (vala_field_get_name (self)), (self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL))), __temp1);
	}
	return self->priv->cname;
}


static void vala_field_set_cname (ValaField* self, char* cname)
{
	g_return_if_fail (VALA_IS_FIELD (self));
	g_return_if_fail (cname != NULL);
	char* __temp3 = NULL;
	self->priv->cname = (__temp3 = g_strdup (cname), (self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL))), __temp3);
}


static void vala_field_process_ccode_attribute (ValaField* self, ValaAttribute* a)
{
	g_return_if_fail (VALA_IS_FIELD (self));
	g_return_if_fail (VALA_IS_ATTRIBUTE (a));
	{
		GList * arg_it;
		for (arg_it = a->args; arg_it != NULL; arg_it = arg_it->next) {
			ValaNamedArgument* arg = arg_it->data;
			{
				if (g_utf8_collate (vala_named_argument_get_name (arg), "cname") == 0) {
					/* this will already be checked during semantic analysis */
					if (VALA_IS_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))) {
						ValaLiteral* lit = vala_literal_expression_get_literal ((VALA_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))));
						if (VALA_IS_STRING_LITERAL (lit)) {
							char* __temp4 = NULL;
							vala_field_set_cname (self, (__temp4 = vala_string_literal_eval ((VALA_STRING_LITERAL (lit)))));
							(__temp4 == NULL ? NULL : (__temp4 = (g_free (__temp4), NULL)));
						}
					}
				}
			}
		}
	}
}


/**
 * Process all associated attributes.
 */
void vala_field_process_attributes (ValaField* self)
{
	g_return_if_fail (VALA_IS_FIELD (self));
	{
		GList * a_it;
		for (a_it = VALA_CODE_NODE (self)->attributes; a_it != NULL; a_it = a_it->next) {
			ValaAttribute* a = a_it->data;
			{
				if (g_utf8_collate (vala_attribute_get_name (a), "CCode") == 0) {
					vala_field_process_ccode_attribute (self, a);
				}
			}
		}
	}
}


char* vala_field_get_name (ValaField* self)
{
	g_return_val_if_fail (VALA_IS_FIELD (self), NULL);
	return self->priv->_name;
}


void vala_field_set_name (ValaField* self, char* value)
{
	g_return_if_fail (VALA_IS_FIELD (self));
	char* __temp6 = NULL;
	self->priv->_name = (__temp6 = g_strdup (value), (self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL))), __temp6);
}


ValaTypeReference* vala_field_get_type_reference (ValaField* self)
{
	g_return_val_if_fail (VALA_IS_FIELD (self), NULL);
	return self->priv->_type_reference;
}


void vala_field_set_type_reference (ValaField* self, ValaTypeReference* value)
{
	g_return_if_fail (VALA_IS_FIELD (self));
	ValaTypeReference* __temp8 = NULL;
	self->priv->_type_reference = (__temp8 = g_object_ref (value), (self->priv->_type_reference == NULL ? NULL : (self->priv->_type_reference = (g_object_unref (self->priv->_type_reference), NULL))), __temp8);
}


ValaExpression* vala_field_get_initializer (ValaField* self)
{
	g_return_val_if_fail (VALA_IS_FIELD (self), NULL);
	return self->priv->_initializer;
}


void vala_field_set_initializer (ValaField* self, ValaExpression* value)
{
	g_return_if_fail (VALA_IS_FIELD (self));
	ValaExpression* __temp11 = NULL;
	ValaExpression* __temp10 = NULL;
	self->priv->_initializer = (__temp11 = (__temp10 = value, (__temp10 == NULL ? __temp10 : g_object_ref (__temp10))), (self->priv->_initializer == NULL ? NULL : (self->priv->_initializer = (g_object_unref (self->priv->_initializer), NULL))), __temp11);
}


gboolean vala_field_get_instance (ValaField* self)
{
	g_return_val_if_fail (VALA_IS_FIELD (self), FALSE);
	return self->priv->_instance;
}


void vala_field_set_instance (ValaField* self, gboolean value)
{
	g_return_if_fail (VALA_IS_FIELD (self));
	self->priv->_instance = value;
}


void vala_field_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaField * self = VALA_FIELD (object);
	switch (property_id) {
		case VALA_FIELD_NAME:
		g_value_set_string (value, vala_field_get_name (self));
		break;
		case VALA_FIELD_TYPE_REFERENCE:
		g_value_set_object (value, vala_field_get_type_reference (self));
		break;
		case VALA_FIELD_INITIALIZER:
		g_value_set_object (value, vala_field_get_initializer (self));
		break;
		case VALA_FIELD_INSTANCE:
		g_value_set_boolean (value, vala_field_get_instance (self));
		break;
	}
}


void vala_field_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaField * self = VALA_FIELD (object);
	switch (property_id) {
		case VALA_FIELD_NAME:
		vala_field_set_name (self, g_value_get_string (value));
		break;
		case VALA_FIELD_TYPE_REFERENCE:
		vala_field_set_type_reference (self, g_value_get_object (value));
		break;
		case VALA_FIELD_INITIALIZER:
		vala_field_set_initializer (self, g_value_get_object (value));
		break;
		case VALA_FIELD_INSTANCE:
		vala_field_set_instance (self, g_value_get_boolean (value));
		break;
	}
}


static void vala_field_class_init (ValaFieldClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaFieldPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_field_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_field_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_field_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_field_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_FIELD_NAME, g_param_spec_string ("name", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_FIELD_TYPE_REFERENCE, g_param_spec_object ("type-reference", "foo", "bar", VALA_TYPE_TYPE_REFERENCE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_FIELD_INITIALIZER, g_param_spec_object ("initializer", "foo", "bar", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_FIELD_INSTANCE, g_param_spec_boolean ("instance", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_field_init (ValaField * self)
{
	self->priv = VALA_FIELD_GET_PRIVATE (self);
	self->priv->_instance = TRUE;
}


static void vala_field_dispose (GObject * obj)
{
	ValaField * self = VALA_FIELD (obj);
	(self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL)));
	(self->priv->_type_reference == NULL ? NULL : (self->priv->_type_reference = (g_object_unref (self->priv->_type_reference), NULL)));
	(self->priv->_initializer == NULL ? NULL : (self->priv->_initializer = (g_object_unref (self->priv->_initializer), NULL)));
	(self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL)));
	ValaFieldClass * klass;
	GObjectClass * parent_class;
	klass = VALA_FIELD_CLASS (g_type_class_peek (VALA_TYPE_FIELD));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_field_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaFieldClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_field_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaField), 0, (GInstanceInitFunc) vala_field_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_NODE, "ValaField", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



