/* valaflags.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valaflags.h>
#include <valasourcereference.h>
#include <valaflagsvalue.h>
#include <valacodevisitor.h>
#include <valanamespace.h>

struct _ValaFlagsPrivate {
	GList* values;
	char* cname;
};
#define VALA_FLAGS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_FLAGS, ValaFlagsPrivate))
enum  {
	VALA_FLAGS_DUMMY_PROPERTY,
};
static void vala_flags_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static char* vala_flags_real_get_cname (ValaDataType* base);
static char* vala_flags_real_get_upper_case_cname (ValaDataType* base, char* infix);
static gboolean vala_flags_real_is_reference_type (ValaDataType* base);
static void vala_flags_dispose (GObject * obj);

/**
 * Creates a new flags.
 *
 * @param name   type name
 * @param source reference to source code
 * @return       newly created flags
 */
ValaFlags* vala_flags_new (char* name, ValaSourceReference* source)
{
	g_return_val_if_fail (name != NULL, NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_FLAGS, "name", name, "source_reference", source, NULL));
}


/**
 * Appends the specified flags value to the list of values.
 *
 * @param value a flags value
 */
void vala_flags_add_value (ValaFlags* self, ValaFlagsValue* value)
{
	g_return_if_fail (VALA_IS_FLAGS (self));
	g_return_if_fail (VALA_IS_FLAGS_VALUE (value));
	self->priv->values = g_list_append (self->priv->values, g_object_ref (value));
}


static void vala_flags_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaFlags * self = VALA_FLAGS (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_begin_flags (visitor, self);
	{
		GList * value_it;
		for (value_it = self->priv->values; value_it != NULL; value_it = value_it->next) {
			ValaFlagsValue* value = value_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (value), visitor);
			}
		}
	}
	vala_code_visitor_visit_end_flags (visitor, self);
}


static char* vala_flags_real_get_cname (ValaDataType* base)
{
	ValaFlags * self = VALA_FLAGS (base);
	if (self->priv->cname == NULL) {
		char* __temp1 = NULL;
		self->priv->cname = (__temp1 = g_strdup_printf ("%s%s", vala_namespace_get_cprefix (VALA_DATA_TYPE (self)->namespace), vala_data_type_get_name (VALA_DATA_TYPE (self))), (self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL))), __temp1);
	}
	return self->priv->cname;
}


static char* vala_flags_real_get_upper_case_cname (ValaDataType* base, char* infix)
{
	ValaFlags * self = VALA_FLAGS (base);
	char* __temp4 = NULL;
	char* __temp3 = NULL;
	char* __temp5 = NULL;
	return (__temp5 = g_utf8_strup ((__temp4 = g_strdup_printf ("%s%s", vala_namespace_get_lower_case_cprefix (VALA_DATA_TYPE (self)->namespace), (__temp3 = vala_namespace_camel_case_to_lower_case (vala_data_type_get_name (VALA_DATA_TYPE (self)))))), -1), (__temp4 == NULL ? NULL : (__temp4 = (g_free (__temp4), NULL))), (__temp3 == NULL ? NULL : (__temp3 = (g_free (__temp3), NULL))), __temp5);
}


static gboolean vala_flags_real_is_reference_type (ValaDataType* base)
{
	ValaFlags * self = VALA_FLAGS (base);
	return FALSE;
}


void vala_flags_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaFlags * self = VALA_FLAGS (object);
	switch (property_id) {
	}
}


void vala_flags_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaFlags * self = VALA_FLAGS (object);
	switch (property_id) {
	}
}


static void vala_flags_class_init (ValaFlagsClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaFlagsPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_flags_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_flags_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_flags_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_flags_real_accept;
	VALA_DATA_TYPE_CLASS (klass)->get_cname = vala_flags_real_get_cname;
	VALA_DATA_TYPE_CLASS (klass)->get_upper_case_cname = vala_flags_real_get_upper_case_cname;
	VALA_DATA_TYPE_CLASS (klass)->is_reference_type = vala_flags_real_is_reference_type;
}


static void vala_flags_init (ValaFlags * self)
{
	self->priv = VALA_FLAGS_GET_PRIVATE (self);
}


static void vala_flags_dispose (GObject * obj)
{
	ValaFlags * self = VALA_FLAGS (obj);
	(self->priv->values == NULL ? NULL : (self->priv->values = (g_list_foreach (self->priv->values, (GFunc) g_object_unref, NULL), g_list_free (self->priv->values), NULL)));
	(self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL)));
	ValaFlagsClass * klass;
	GObjectClass * parent_class;
	klass = VALA_FLAGS_CLASS (g_type_class_peek (VALA_TYPE_FLAGS));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_flags_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaFlagsClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_flags_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaFlags), 0, (GInstanceInitFunc) vala_flags_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_DATA_TYPE, "ValaFlags", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



