/* valaforstatement.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valaforstatement.h>
#include <valaexpression.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>

struct _ValaForStatementPrivate {
	ValaExpression* _condition;
	ValaStatement* _body;
	GList* initializer;
	GList* iterator;
};
#define VALA_FOR_STATEMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_FOR_STATEMENT, ValaForStatementPrivate))
enum  {
	VALA_FOR_STATEMENT_DUMMY_PROPERTY,
	VALA_FOR_STATEMENT_CONDITION,
	VALA_FOR_STATEMENT_BODY,
};
static void vala_for_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_for_statement_dispose (GObject * obj);

/**
 * Creates a new for statement.
 *
 * @param cond   loop condition
 * @param body   loop body
 * @param source reference to source code
 * @return       newly created for statement
 */
ValaForStatement* vala_for_statement_new (ValaExpression* cond, ValaStatement* body, ValaSourceReference* source)
{
	g_return_val_if_fail (cond == NULL || VALA_IS_EXPRESSION (cond), NULL);
	g_return_val_if_fail (body == NULL || VALA_IS_STATEMENT (body), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_FOR_STATEMENT, "condition", cond, "body", body, "source_reference", source, NULL));
}


/**
 * Appends the specified expression to the list of initializers.
 *
 * @param init an initializer expression
 */
void vala_for_statement_add_initializer (ValaForStatement* self, ValaExpression* init)
{
	g_return_if_fail (VALA_IS_FOR_STATEMENT (self));
	g_return_if_fail (VALA_IS_EXPRESSION (init));
	self->priv->initializer = g_list_append (self->priv->initializer, g_object_ref (init));
}


/**
 * Returns a copy of the list of initializers.
 *
 * @return initializer list
 */
GList* vala_for_statement_get_initializer (ValaForStatement* self)
{
	g_return_val_if_fail (VALA_IS_FOR_STATEMENT (self), NULL);
	return g_list_copy (self->priv->initializer);
}


/**
 * Appends the specified expression to the iterator.
 *
 * @param iter an iterator expression
 */
void vala_for_statement_add_iterator (ValaForStatement* self, ValaExpression* iter)
{
	g_return_if_fail (VALA_IS_FOR_STATEMENT (self));
	g_return_if_fail (VALA_IS_EXPRESSION (iter));
	self->priv->iterator = g_list_append (self->priv->iterator, g_object_ref (iter));
}


/**
 * Returns a copy of the iterator.
 *
 * @return iterator
 */
GList* vala_for_statement_get_iterator (ValaForStatement* self)
{
	g_return_val_if_fail (VALA_IS_FOR_STATEMENT (self), NULL);
	return g_list_copy (self->priv->iterator);
}


static void vala_for_statement_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaForStatement * self = VALA_FOR_STATEMENT (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	{
		GList * init_expr_it;
		for (init_expr_it = self->priv->initializer; init_expr_it != NULL; init_expr_it = init_expr_it->next) {
			ValaExpression* init_expr = init_expr_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (init_expr), visitor);
			}
		}
	}
	vala_code_node_accept (VALA_CODE_NODE (vala_for_statement_get_condition (self)), visitor);
	vala_code_visitor_visit_end_full_expression (visitor, vala_for_statement_get_condition (self));
	{
		GList * it_expr_it;
		for (it_expr_it = self->priv->iterator; it_expr_it != NULL; it_expr_it = it_expr_it->next) {
			ValaExpression* it_expr = it_expr_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (it_expr), visitor);
			}
		}
	}
	vala_code_node_accept (VALA_CODE_NODE (vala_for_statement_get_body (self)), visitor);
	vala_code_visitor_visit_for_statement (visitor, self);
}


ValaExpression* vala_for_statement_get_condition (ValaForStatement* self)
{
	g_return_val_if_fail (VALA_IS_FOR_STATEMENT (self), NULL);
	return self->priv->_condition;
}


void vala_for_statement_set_condition (ValaForStatement* self, ValaExpression* value)
{
	g_return_if_fail (VALA_IS_FOR_STATEMENT (self));
	ValaExpression* __temp5 = NULL;
	ValaExpression* __temp4 = NULL;
	self->priv->_condition = (__temp5 = (__temp4 = value, (__temp4 == NULL ? __temp4 : g_object_ref (__temp4))), (self->priv->_condition == NULL ? NULL : (self->priv->_condition = (g_object_unref (self->priv->_condition), NULL))), __temp5);
}


ValaStatement* vala_for_statement_get_body (ValaForStatement* self)
{
	g_return_val_if_fail (VALA_IS_FOR_STATEMENT (self), NULL);
	return self->priv->_body;
}


void vala_for_statement_set_body (ValaForStatement* self, ValaStatement* value)
{
	g_return_if_fail (VALA_IS_FOR_STATEMENT (self));
	ValaStatement* __temp8 = NULL;
	ValaStatement* __temp7 = NULL;
	self->priv->_body = (__temp8 = (__temp7 = value, (__temp7 == NULL ? __temp7 : g_object_ref (__temp7))), (self->priv->_body == NULL ? NULL : (self->priv->_body = (g_object_unref (self->priv->_body), NULL))), __temp8);
}


void vala_for_statement_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaForStatement * self = VALA_FOR_STATEMENT (object);
	switch (property_id) {
		case VALA_FOR_STATEMENT_CONDITION:
		g_value_set_object (value, vala_for_statement_get_condition (self));
		break;
		case VALA_FOR_STATEMENT_BODY:
		g_value_set_object (value, vala_for_statement_get_body (self));
		break;
	}
}


void vala_for_statement_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaForStatement * self = VALA_FOR_STATEMENT (object);
	switch (property_id) {
		case VALA_FOR_STATEMENT_CONDITION:
		vala_for_statement_set_condition (self, g_value_get_object (value));
		break;
		case VALA_FOR_STATEMENT_BODY:
		vala_for_statement_set_body (self, g_value_get_object (value));
		break;
	}
}


static void vala_for_statement_class_init (ValaForStatementClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaForStatementPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_for_statement_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_for_statement_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_for_statement_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_for_statement_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_FOR_STATEMENT_CONDITION, g_param_spec_object ("condition", "foo", "bar", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_FOR_STATEMENT_BODY, g_param_spec_object ("body", "foo", "bar", VALA_TYPE_STATEMENT, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_for_statement_init (ValaForStatement * self)
{
	self->priv = VALA_FOR_STATEMENT_GET_PRIVATE (self);
}


static void vala_for_statement_dispose (GObject * obj)
{
	ValaForStatement * self = VALA_FOR_STATEMENT (obj);
	(self->priv->_condition == NULL ? NULL : (self->priv->_condition = (g_object_unref (self->priv->_condition), NULL)));
	(self->priv->_body == NULL ? NULL : (self->priv->_body = (g_object_unref (self->priv->_body), NULL)));
	(self->priv->initializer == NULL ? NULL : (self->priv->initializer = (g_list_foreach (self->priv->initializer, (GFunc) g_object_unref, NULL), g_list_free (self->priv->initializer), NULL)));
	(self->priv->iterator == NULL ? NULL : (self->priv->iterator = (g_list_foreach (self->priv->iterator, (GFunc) g_object_unref, NULL), g_list_free (self->priv->iterator), NULL)));
	ValaForStatementClass * klass;
	GObjectClass * parent_class;
	klass = VALA_FOR_STATEMENT_CLASS (g_type_class_peek (VALA_TYPE_FOR_STATEMENT));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_for_statement_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaForStatementClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_for_statement_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaForStatement), 0, (GInstanceInitFunc) vala_for_statement_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_STATEMENT, "ValaForStatement", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



