/* valainitializerlist.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valainitializerlist.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>

struct _ValaInitializerListPrivate {
	GList* initializers;
};
#define VALA_INITIALIZER_LIST_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_INITIALIZER_LIST, ValaInitializerListPrivate))
enum  {
	VALA_INITIALIZER_LIST_DUMMY_PROPERTY,
};
static void vala_initializer_list_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_initializer_list_dispose (GObject * obj);

/**
 * Appends the specified expression to this initializer list.
 *
 * @param expr an expression
 */
void vala_initializer_list_append (ValaInitializerList* self, ValaExpression* expr)
{
	g_return_if_fail (VALA_IS_INITIALIZER_LIST (self));
	g_return_if_fail (VALA_IS_EXPRESSION (expr));
	self->priv->initializers = g_list_append (self->priv->initializers, g_object_ref (expr));
}


/**
 * Returns a copy of the expression list.
 *
 * @return expression list
 */
GList* vala_initializer_list_get_initializers (ValaInitializerList* self)
{
	g_return_val_if_fail (VALA_IS_INITIALIZER_LIST (self), NULL);
	return g_list_copy (self->priv->initializers);
}


/**
 * Creates a new initializer list.
 *
 * @param source reference to source code
 * @return       newly created initializer list
 */
ValaInitializerList* vala_initializer_list_new (ValaSourceReference* source)
{
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_INITIALIZER_LIST, "source_reference", source, NULL));
}


static void vala_initializer_list_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaInitializerList * self = VALA_INITIALIZER_LIST (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	{
		GList * expr_it;
		for (expr_it = self->priv->initializers; expr_it != NULL; expr_it = expr_it->next) {
			ValaExpression* expr = expr_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (expr), visitor);
			}
		}
	}
	vala_code_visitor_visit_initializer_list (visitor, self);
}


void vala_initializer_list_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaInitializerList * self = VALA_INITIALIZER_LIST (object);
	switch (property_id) {
	}
}


void vala_initializer_list_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaInitializerList * self = VALA_INITIALIZER_LIST (object);
	switch (property_id) {
	}
}


static void vala_initializer_list_class_init (ValaInitializerListClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaInitializerListPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_initializer_list_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_initializer_list_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_initializer_list_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_initializer_list_real_accept;
}


static void vala_initializer_list_init (ValaInitializerList * self)
{
	self->priv = VALA_INITIALIZER_LIST_GET_PRIVATE (self);
}


static void vala_initializer_list_dispose (GObject * obj)
{
	ValaInitializerList * self = VALA_INITIALIZER_LIST (obj);
	(self->priv->initializers == NULL ? NULL : (self->priv->initializers = (g_list_foreach (self->priv->initializers, (GFunc) g_object_unref, NULL), g_list_free (self->priv->initializers), NULL)));
	ValaInitializerListClass * klass;
	GObjectClass * parent_class;
	klass = VALA_INITIALIZER_LIST_CLASS (g_type_class_peek (VALA_TYPE_INITIALIZER_LIST));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_initializer_list_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaInitializerListClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_initializer_list_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaInitializerList), 0, (GInstanceInitFunc) vala_initializer_list_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_EXPRESSION, "ValaInitializerList", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



