/* valainterface.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valainterface.h>
#include <valasourcereference.h>
#include <valatypeparameter.h>
#include <valatypereference.h>
#include <valamethod.h>
#include <valaproperty.h>
#include <valasignal.h>
#include <valanamespace.h>
#include <valacodevisitor.h>

struct _ValaInterfacePrivate {
	GList* type_parameters;
	GList* base_types;
	GList* methods;
	GList* properties;
	GList* signals;
	char* cname;
	char* lower_case_csuffix;
};
#define VALA_INTERFACE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_INTERFACE, ValaInterfacePrivate))
enum  {
	VALA_INTERFACE_DUMMY_PROPERTY,
};
static char* vala_interface_real_get_cname (ValaDataType* base);
static char* vala_interface_real_get_lower_case_cname (ValaDataType* base, char* infix);
static char* vala_interface_real_get_upper_case_cname (ValaDataType* base, char* infix);
static void vala_interface_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static gboolean vala_interface_real_is_reference_type (ValaDataType* base);
static gboolean vala_interface_real_is_reference_counting (ValaDataType* base);
static char* vala_interface_real_get_ref_function (ValaDataType* base);
static char* vala_interface_real_get_unref_function (ValaDataType* base);
static void vala_interface_dispose (GObject * obj);

/**
 * Creates a new interface.
 *
 * @param name   type name
 * @param source reference to source code
 * @return       newly created interface
 */
ValaInterface* vala_interface_new (char* name, ValaSourceReference* source)
{
	g_return_val_if_fail (name != NULL, NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_INTERFACE, "name", name, "source_reference", source, NULL));
}


/**
 * Appends the specified parameter to the list of type parameters.
 *
 * @param p a type parameter
 */
void vala_interface_add_type_parameter (ValaInterface* self, ValaTypeParameter* p)
{
	g_return_if_fail (VALA_IS_INTERFACE (self));
	g_return_if_fail (VALA_IS_TYPE_PARAMETER (p));
	self->priv->type_parameters = g_list_append (self->priv->type_parameters, g_object_ref (p));
	p->type = VALA_DATA_TYPE (self);
}


/**
 * Adds the specified interface to the list of prerequisites of this
 * interface.
 *
 * @param type an interface reference
 */
void vala_interface_add_base_type (ValaInterface* self, ValaTypeReference* type)
{
	g_return_if_fail (VALA_IS_INTERFACE (self));
	g_return_if_fail (VALA_IS_TYPE_REFERENCE (type));
	self->priv->base_types = g_list_append (self->priv->base_types, g_object_ref (type));
}


/**
 * Adds the specified method as a member to this interface.
 *
 * @param m a method
 */
void vala_interface_add_method (ValaInterface* self, ValaMethod* m)
{
	g_return_if_fail (VALA_IS_INTERFACE (self));
	g_return_if_fail (VALA_IS_METHOD (m));
	self->priv->methods = g_list_append (self->priv->methods, g_object_ref (m));
}


/**
 * Returns a copy of the list of methods.
 *
 * @return list of methods
 */
GList* vala_interface_get_methods (ValaInterface* self)
{
	g_return_val_if_fail (VALA_IS_INTERFACE (self), NULL);
	return g_list_copy (self->priv->methods);
}


/**
 * Adds the specified property as a member to this interface.
 *
 * @param prop a property
 */
void vala_interface_add_property (ValaInterface* self, ValaProperty* prop)
{
	g_return_if_fail (VALA_IS_INTERFACE (self));
	g_return_if_fail (VALA_IS_PROPERTY (prop));
	self->priv->properties = g_list_append (self->priv->properties, g_object_ref (prop));
}


/**
 * Returns a copy of the list of properties.
 *
 * @return list of properties
 */
GList* vala_interface_get_properties (ValaInterface* self)
{
	g_return_val_if_fail (VALA_IS_INTERFACE (self), NULL);
	return g_list_copy (self->priv->properties);
}


/**
 * Adds the specified signal as a member to this interface.
 *
 * @param sig a signal
 */
void vala_interface_add_signal (ValaInterface* self, ValaSignal* sig)
{
	g_return_if_fail (VALA_IS_INTERFACE (self));
	g_return_if_fail (VALA_IS_SIGNAL (sig));
	self->priv->signals = g_list_append (self->priv->signals, g_object_ref (sig));
}


/**
 * Returns a copy of the list of signals.
 *
 * @return list of signals
 */
GList* vala_interface_get_signals (ValaInterface* self)
{
	g_return_val_if_fail (VALA_IS_INTERFACE (self), NULL);
	return g_list_copy (self->priv->signals);
}


static char* vala_interface_real_get_cname (ValaDataType* base)
{
	ValaInterface * self = VALA_INTERFACE (base);
	if (self->priv->cname == NULL) {
		char* __temp4 = NULL;
		self->priv->cname = (__temp4 = g_strdup_printf ("%s%s", vala_namespace_get_cprefix (VALA_DATA_TYPE (self)->namespace), vala_data_type_get_name (VALA_DATA_TYPE (self))), (self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL))), __temp4);
	}
	return self->priv->cname;
}


/**
 * Returns the string to be prepended to the name of members of this
 * interface when used in C code.
 *
 * @return the suffix to be used in C code
 */
char* vala_interface_get_lower_case_csuffix (ValaInterface* self)
{
	g_return_val_if_fail (VALA_IS_INTERFACE (self), NULL);
	if (self->priv->lower_case_csuffix == NULL) {
		char* __temp6 = NULL;
		self->priv->lower_case_csuffix = (__temp6 = vala_namespace_camel_case_to_lower_case (vala_data_type_get_name (VALA_DATA_TYPE (self))), (self->priv->lower_case_csuffix == NULL ? NULL : (self->priv->lower_case_csuffix = (g_free (self->priv->lower_case_csuffix), NULL))), __temp6);
	}
	return self->priv->lower_case_csuffix;
}


/**
 * Sets the string to be prepended to the name of members of this
 * interface when used in C code.
 *
 * @param csuffix the suffix to be used in C code
 */
void vala_interface_set_lower_case_csuffix (ValaInterface* self, char* csuffix)
{
	g_return_if_fail (VALA_IS_INTERFACE (self));
	g_return_if_fail (csuffix != NULL);
	char* __temp8 = NULL;
	self->priv->lower_case_csuffix = (__temp8 = g_strdup (csuffix), (self->priv->lower_case_csuffix == NULL ? NULL : (self->priv->lower_case_csuffix = (g_free (self->priv->lower_case_csuffix), NULL))), __temp8);
}


static char* vala_interface_real_get_lower_case_cname (ValaDataType* base, char* infix)
{
	ValaInterface * self = VALA_INTERFACE (base);
	if (infix == NULL) {
		infix = "";
	}
	return g_strdup_printf ("%s%s%s", vala_namespace_get_lower_case_cprefix (VALA_DATA_TYPE (self)->namespace), infix, vala_interface_get_lower_case_csuffix (self));
}


static char* vala_interface_real_get_upper_case_cname (ValaDataType* base, char* infix)
{
	ValaInterface * self = VALA_INTERFACE (base);
	char* __temp10 = NULL;
	char* __temp11 = NULL;
	return (__temp11 = g_utf8_strup ((__temp10 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (self), infix)), -1), (__temp10 == NULL ? NULL : (__temp10 = (g_free (__temp10), NULL))), __temp11);
}


static void vala_interface_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaInterface * self = VALA_INTERFACE (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_begin_interface (visitor, self);
	{
		GList * type_it;
		for (type_it = self->priv->base_types; type_it != NULL; type_it = type_it->next) {
			ValaTypeReference* type = type_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (type), visitor);
			}
		}
	}
	{
		GList * p_it;
		for (p_it = self->priv->type_parameters; p_it != NULL; p_it = p_it->next) {
			ValaTypeParameter* p = p_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (p), visitor);
			}
		}
	}
	{
		GList * m_it;
		for (m_it = self->priv->methods; m_it != NULL; m_it = m_it->next) {
			ValaMethod* m = m_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (m), visitor);
			}
		}
	}
	{
		GList * prop_it;
		for (prop_it = self->priv->properties; prop_it != NULL; prop_it = prop_it->next) {
			ValaProperty* prop = prop_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (prop), visitor);
			}
		}
	}
	vala_code_visitor_visit_end_interface (visitor, self);
}


static gboolean vala_interface_real_is_reference_type (ValaDataType* base)
{
	ValaInterface * self = VALA_INTERFACE (base);
	return TRUE;
}


static gboolean vala_interface_real_is_reference_counting (ValaDataType* base)
{
	ValaInterface * self = VALA_INTERFACE (base);
	return TRUE;
}


static char* vala_interface_real_get_ref_function (ValaDataType* base)
{
	ValaInterface * self = VALA_INTERFACE (base);
	return "g_object_ref";
}


static char* vala_interface_real_get_unref_function (ValaDataType* base)
{
	ValaInterface * self = VALA_INTERFACE (base);
	return "g_object_unref";
}


void vala_interface_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaInterface * self = VALA_INTERFACE (object);
	switch (property_id) {
	}
}


void vala_interface_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaInterface * self = VALA_INTERFACE (object);
	switch (property_id) {
	}
}


static void vala_interface_class_init (ValaInterfaceClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaInterfacePrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_interface_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_interface_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_interface_dispose;
	VALA_DATA_TYPE_CLASS (klass)->get_cname = vala_interface_real_get_cname;
	VALA_DATA_TYPE_CLASS (klass)->get_lower_case_cname = vala_interface_real_get_lower_case_cname;
	VALA_DATA_TYPE_CLASS (klass)->get_upper_case_cname = vala_interface_real_get_upper_case_cname;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_interface_real_accept;
	VALA_DATA_TYPE_CLASS (klass)->is_reference_type = vala_interface_real_is_reference_type;
	VALA_DATA_TYPE_CLASS (klass)->is_reference_counting = vala_interface_real_is_reference_counting;
	VALA_DATA_TYPE_CLASS (klass)->get_ref_function = vala_interface_real_get_ref_function;
	VALA_DATA_TYPE_CLASS (klass)->get_unref_function = vala_interface_real_get_unref_function;
}


static void vala_interface_init (ValaInterface * self)
{
	self->priv = VALA_INTERFACE_GET_PRIVATE (self);
}


static void vala_interface_dispose (GObject * obj)
{
	ValaInterface * self = VALA_INTERFACE (obj);
	(self->priv->type_parameters == NULL ? NULL : (self->priv->type_parameters = (g_list_foreach (self->priv->type_parameters, (GFunc) g_free, NULL), g_list_free (self->priv->type_parameters), NULL)));
	(self->priv->base_types == NULL ? NULL : (self->priv->base_types = (g_list_foreach (self->priv->base_types, (GFunc) g_object_unref, NULL), g_list_free (self->priv->base_types), NULL)));
	(self->priv->methods == NULL ? NULL : (self->priv->methods = (g_list_foreach (self->priv->methods, (GFunc) g_object_unref, NULL), g_list_free (self->priv->methods), NULL)));
	(self->priv->properties == NULL ? NULL : (self->priv->properties = (g_list_foreach (self->priv->properties, (GFunc) g_object_unref, NULL), g_list_free (self->priv->properties), NULL)));
	(self->priv->signals == NULL ? NULL : (self->priv->signals = (g_list_foreach (self->priv->signals, (GFunc) g_object_unref, NULL), g_list_free (self->priv->signals), NULL)));
	(self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL)));
	(self->priv->lower_case_csuffix == NULL ? NULL : (self->priv->lower_case_csuffix = (g_free (self->priv->lower_case_csuffix), NULL)));
	ValaInterfaceClass * klass;
	GObjectClass * parent_class;
	klass = VALA_INTERFACE_CLASS (g_type_class_peek (VALA_TYPE_INTERFACE));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_interface_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaInterfaceClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_interface_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaInterface), 0, (GInstanceInitFunc) vala_interface_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_DATA_TYPE, "ValaInterface", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



