/* valainterfacewriter.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valainterfacewriter.h>
#include <stdio.h>
#include <valadatatype.h>
#include <valamemberaccessibility.h>
#include <valatypereference.h>
#include <valacodenode.h>
#include <valasymbol.h>
#include <valaformalparameter.h>

struct _ValaInterfaceWriterPrivate {
	FILE* stream;
	int indent;
	gboolean bol;
	gboolean internal_scope;
	char* current_cheader_filename;
};
#define VALA_INTERFACE_WRITER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_INTERFACE_WRITER, ValaInterfaceWriterPrivate))
enum  {
	VALA_INTERFACE_WRITER_DUMMY_PROPERTY,
};
static void vala_interface_writer_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file);
static void vala_interface_writer_real_visit_begin_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
static void vala_interface_writer_real_visit_end_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
static void vala_interface_writer_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_interface_writer_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_interface_writer_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vala_interface_writer_real_visit_end_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vala_interface_writer_real_visit_begin_enum (ValaCodeVisitor* base, ValaEnum* en);
static void vala_interface_writer_real_visit_end_enum (ValaCodeVisitor* base, ValaEnum* en);
static void vala_interface_writer_real_visit_enum_value (ValaCodeVisitor* base, ValaEnumValue* ev);
static void vala_interface_writer_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c);
static void vala_interface_writer_real_visit_field (ValaCodeVisitor* base, ValaField* f);
static void vala_interface_writer_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m);
static void vala_interface_writer_real_visit_begin_property (ValaCodeVisitor* base, ValaProperty* prop);
static void vala_interface_writer_write_indent (ValaInterfaceWriter* self);
static void vala_interface_writer_write_identifier (ValaInterfaceWriter* self, char* s);
static void vala_interface_writer_write_string (ValaInterfaceWriter* self, char* s);
static void vala_interface_writer_write_newline (ValaInterfaceWriter* self);
static void vala_interface_writer_write_begin_block (ValaInterfaceWriter* self);
static void vala_interface_writer_write_end_block (ValaInterfaceWriter* self);
static void vala_interface_writer_dispose (GObject * obj);

/**
 * Writes the public interface of the specified code context into the
 * specified file.
 *
 * @param context  a code context
 * @param filename a relative or absolute filename
 */
void vala_interface_writer_write_file (ValaInterfaceWriter* self, ValaCodeContext* context, char* filename)
{
	g_return_if_fail (VALA_IS_INTERFACE_WRITER (self));
	g_return_if_fail (VALA_IS_CODE_CONTEXT (context));
	g_return_if_fail (filename != NULL);
	FILE* __temp0 = NULL;
	self->priv->stream = (__temp0 = fopen (filename, "w"), (self->priv->stream == NULL ? NULL : (self->priv->stream = (fclose (self->priv->stream), NULL))), __temp0);
	{
		GList * file_it;
		for (file_it = vala_code_context_get_source_files (context); file_it != NULL; file_it = file_it->next) {
			ValaSourceFile* file = file_it->data;
			{
				/* we're only interested in non-pkg source files */
				if (!vala_source_file_get_pkg (file)) {
					vala_source_file_accept (file, VALA_CODE_VISITOR (self));
				}
			}
		}
	}
	FILE* __temp1 = NULL;
	self->priv->stream = (__temp1 = NULL, (self->priv->stream == NULL ? NULL : (self->priv->stream = (fclose (self->priv->stream), NULL))), __temp1);
}


static void vala_interface_writer_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* source_file)
{
	ValaInterfaceWriter * self = VALA_INTERFACE_WRITER (base);
	g_return_if_fail (VALA_IS_SOURCE_FILE (source_file));
	char* __temp3 = NULL;
	char* __temp2 = NULL;
	self->priv->current_cheader_filename = (__temp3 = (__temp2 = vala_source_file_get_cheader_filename (source_file), (__temp2 == NULL ? __temp2 : g_strdup (__temp2))), (self->priv->current_cheader_filename == NULL ? NULL : (self->priv->current_cheader_filename = (g_free (self->priv->current_cheader_filename), NULL))), __temp3);
}


static void vala_interface_writer_real_visit_begin_namespace (ValaCodeVisitor* base, ValaNamespace* ns)
{
	ValaInterfaceWriter * self = VALA_INTERFACE_WRITER (base);
	g_return_if_fail (VALA_IS_NAMESPACE (ns));
	if (vala_namespace_get_name (ns) == NULL) {
		return;
	}
	vala_interface_writer_write_indent (self);
	char* __temp4 = NULL;
	vala_interface_writer_write_string (self, (__temp4 = g_strdup_printf ("[CCode (cheader_filename = \"%s\")]", self->priv->current_cheader_filename)));
	(__temp4 == NULL ? NULL : (__temp4 = (g_free (__temp4), NULL)));
	vala_interface_writer_write_newline (self);
	vala_interface_writer_write_indent (self);
	vala_interface_writer_write_string (self, "namespace ");
	vala_interface_writer_write_identifier (self, vala_namespace_get_name (ns));
	vala_interface_writer_write_begin_block (self);
}


static void vala_interface_writer_real_visit_end_namespace (ValaCodeVisitor* base, ValaNamespace* ns)
{
	ValaInterfaceWriter * self = VALA_INTERFACE_WRITER (base);
	g_return_if_fail (VALA_IS_NAMESPACE (ns));
	if (vala_namespace_get_name (ns) == NULL) {
		return;
	}
	vala_interface_writer_write_end_block (self);
	vala_interface_writer_write_newline (self);
}


static void vala_interface_writer_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaInterfaceWriter * self = VALA_INTERFACE_WRITER (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	if (VALA_DATA_TYPE (cl)->access != VALA_MEMBER_ACCESSIBILITY_PUBLIC) {
		self->priv->internal_scope = TRUE;
		return;
	}
	vala_interface_writer_write_indent (self);
	vala_interface_writer_write_string (self, "public ");
	if (vala_class_get_is_abstract (cl)) {
		vala_interface_writer_write_string (self, "abstract ");
	}
	vala_interface_writer_write_string (self, "class ");
	vala_interface_writer_write_identifier (self, vala_data_type_get_name (VALA_DATA_TYPE (cl)));
	GList* base_types = vala_class_get_base_types (cl);
	if (base_types != NULL) {
		vala_interface_writer_write_string (self, " : ");
		gboolean first = TRUE;
		{
			GList * base_type_it;
			for (base_type_it = base_types; base_type_it != NULL; base_type_it = base_type_it->next) {
				ValaTypeReference* base_type = base_type_it->data;
				{
					if (!first) {
						vala_interface_writer_write_string (self, ", ");
					} else {
						first = FALSE;
					}
					char* __temp5 = NULL;
					vala_interface_writer_write_string (self, (__temp5 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (base_type->type)))));
					(__temp5 == NULL ? NULL : (__temp5 = (g_free (__temp5), NULL)));
				}
			}
		}
	}
	vala_interface_writer_write_begin_block (self);
	(base_types == NULL ? NULL : (base_types = (g_list_free (base_types), NULL)));
}


static void vala_interface_writer_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaInterfaceWriter * self = VALA_INTERFACE_WRITER (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	if (VALA_DATA_TYPE (cl)->access != VALA_MEMBER_ACCESSIBILITY_PUBLIC) {
		self->priv->internal_scope = FALSE;
		return;
	}
	vala_interface_writer_write_end_block (self);
	vala_interface_writer_write_newline (self);
}


static void vala_interface_writer_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st)
{
	ValaInterfaceWriter * self = VALA_INTERFACE_WRITER (base);
	g_return_if_fail (VALA_IS_STRUCT (st));
	if (VALA_DATA_TYPE (st)->access != VALA_MEMBER_ACCESSIBILITY_PUBLIC) {
		self->priv->internal_scope = TRUE;
		return;
	}
	vala_interface_writer_write_indent (self);
	vala_interface_writer_write_string (self, "public struct ");
	vala_interface_writer_write_identifier (self, vala_data_type_get_name (VALA_DATA_TYPE (st)));
	vala_interface_writer_write_begin_block (self);
}


static void vala_interface_writer_real_visit_end_struct (ValaCodeVisitor* base, ValaStruct* st)
{
	ValaInterfaceWriter * self = VALA_INTERFACE_WRITER (base);
	g_return_if_fail (VALA_IS_STRUCT (st));
	if (VALA_DATA_TYPE (st)->access != VALA_MEMBER_ACCESSIBILITY_PUBLIC) {
		self->priv->internal_scope = FALSE;
		return;
	}
	vala_interface_writer_write_end_block (self);
	vala_interface_writer_write_newline (self);
}


static void vala_interface_writer_real_visit_begin_enum (ValaCodeVisitor* base, ValaEnum* en)
{
	ValaInterfaceWriter * self = VALA_INTERFACE_WRITER (base);
	g_return_if_fail (VALA_IS_ENUM (en));
	if (VALA_DATA_TYPE (en)->access != VALA_MEMBER_ACCESSIBILITY_PUBLIC) {
		self->priv->internal_scope = TRUE;
		return;
	}
	vala_interface_writer_write_indent (self);
	vala_interface_writer_write_string (self, "public enum ");
	vala_interface_writer_write_identifier (self, vala_data_type_get_name (VALA_DATA_TYPE (en)));
	vala_interface_writer_write_begin_block (self);
}


static void vala_interface_writer_real_visit_end_enum (ValaCodeVisitor* base, ValaEnum* en)
{
	ValaInterfaceWriter * self = VALA_INTERFACE_WRITER (base);
	g_return_if_fail (VALA_IS_ENUM (en));
	if (VALA_DATA_TYPE (en)->access != VALA_MEMBER_ACCESSIBILITY_PUBLIC) {
		self->priv->internal_scope = FALSE;
		return;
	}
	vala_interface_writer_write_end_block (self);
	vala_interface_writer_write_newline (self);
}


static void vala_interface_writer_real_visit_enum_value (ValaCodeVisitor* base, ValaEnumValue* ev)
{
	ValaInterfaceWriter * self = VALA_INTERFACE_WRITER (base);
	g_return_if_fail (VALA_IS_ENUM_VALUE (ev));
	if (self->priv->internal_scope) {
		return;
	}
	vala_interface_writer_write_indent (self);
	vala_interface_writer_write_identifier (self, vala_enum_value_get_name (ev));
	vala_interface_writer_write_string (self, ",");
	vala_interface_writer_write_newline (self);
}


static void vala_interface_writer_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c)
{
	ValaInterfaceWriter * self = VALA_INTERFACE_WRITER (base);
	g_return_if_fail (VALA_IS_CONSTANT (c));
}


static void vala_interface_writer_real_visit_field (ValaCodeVisitor* base, ValaField* f)
{
	ValaInterfaceWriter * self = VALA_INTERFACE_WRITER (base);
	g_return_if_fail (VALA_IS_FIELD (f));
	if (self->priv->internal_scope || f->access != VALA_MEMBER_ACCESSIBILITY_PUBLIC) {
		return;
	}
	vala_interface_writer_write_indent (self);
	vala_interface_writer_write_string (self, "public ");
	if (vala_type_reference_get_is_weak (vala_field_get_type_reference (f))) {
		vala_interface_writer_write_string (self, "weak ");
	}
	char* __temp6 = NULL;
	vala_interface_writer_write_string (self, (__temp6 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (vala_field_get_type_reference (f)->type)))));
	(__temp6 == NULL ? NULL : (__temp6 = (g_free (__temp6), NULL)));
	GList* type_args = vala_type_reference_get_type_arguments (vala_field_get_type_reference (f));
	if (type_args != NULL) {
		vala_interface_writer_write_string (self, "<");
		{
			GList * type_arg_it;
			for (type_arg_it = type_args; type_arg_it != NULL; type_arg_it = type_arg_it->next) {
				ValaTypeReference* type_arg = type_arg_it->data;
				{
					if (vala_type_reference_get_is_weak (type_arg)) {
						vala_interface_writer_write_string (self, "weak ");
					}
					char* __temp7 = NULL;
					vala_interface_writer_write_string (self, (__temp7 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (type_arg->type)))));
					(__temp7 == NULL ? NULL : (__temp7 = (g_free (__temp7), NULL)));
				}
			}
		}
		vala_interface_writer_write_string (self, ">");
	}
	vala_interface_writer_write_string (self, " ");
	vala_interface_writer_write_identifier (self, vala_field_get_name (f));
	vala_interface_writer_write_string (self, ";");
	vala_interface_writer_write_newline (self);
	(type_args == NULL ? NULL : (type_args = (g_list_free (type_args), NULL)));
}


static void vala_interface_writer_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m)
{
	ValaInterfaceWriter * self = VALA_INTERFACE_WRITER (base);
	g_return_if_fail (VALA_IS_METHOD (m));
	if (self->priv->internal_scope || m->access != VALA_MEMBER_ACCESSIBILITY_PUBLIC || vala_method_get_overrides (m)) {
		return;
	}
	vala_interface_writer_write_indent (self);
	vala_interface_writer_write_string (self, "public ");
	if (!vala_method_get_instance (m)) {
		vala_interface_writer_write_string (self, "static ");
	} else if (vala_method_get_is_abstract (m)) {
		vala_interface_writer_write_string (self, "abstract ");
	} else if (vala_method_get_is_virtual (m)) {
		vala_interface_writer_write_string (self, "virtual ");
	}
	ValaDataType* type = vala_method_get_return_type (m)->type;
	if (type == NULL) {
		vala_interface_writer_write_string (self, "void");
	} else {
		if (vala_type_reference_get_is_ref (vala_method_get_return_type (m))) {
			vala_interface_writer_write_string (self, "ref ");
		}
		char* __temp8 = NULL;
		vala_interface_writer_write_string (self, (__temp8 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (vala_method_get_return_type (m)->type)))));
		(__temp8 == NULL ? NULL : (__temp8 = (g_free (__temp8), NULL)));
	}
	vala_interface_writer_write_string (self, " ");
	vala_interface_writer_write_identifier (self, vala_method_get_name (m));
	vala_interface_writer_write_string (self, " (");
	gboolean first = TRUE;
	{
		GList * param_it;
		for (param_it = vala_method_get_parameters (m); param_it != NULL; param_it = param_it->next) {
			ValaFormalParameter* param = param_it->data;
			{
				if (!first) {
					vala_interface_writer_write_string (self, ", ");
				} else {
					first = FALSE;
				}
				if (vala_type_reference_get_is_ref (vala_formal_parameter_get_type_reference (param))) {
					vala_interface_writer_write_string (self, "ref ");
				} else if (vala_type_reference_get_is_out (vala_formal_parameter_get_type_reference (param))) {
					vala_interface_writer_write_string (self, "out ");
				}
				char* __temp9 = NULL;
				vala_interface_writer_write_string (self, (__temp9 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (vala_formal_parameter_get_type_reference (param)->type)))));
				(__temp9 == NULL ? NULL : (__temp9 = (g_free (__temp9), NULL)));
				GList* type_args = vala_type_reference_get_type_arguments (vala_formal_parameter_get_type_reference (param));
				if (type_args != NULL) {
					vala_interface_writer_write_string (self, "<");
					{
						GList * type_arg_it;
						for (type_arg_it = type_args; type_arg_it != NULL; type_arg_it = type_arg_it->next) {
							ValaTypeReference* type_arg = type_arg_it->data;
							{
								if (vala_type_reference_get_is_ref (type_arg)) {
									vala_interface_writer_write_string (self, "ref ");
								}
								char* __temp10 = NULL;
								vala_interface_writer_write_string (self, (__temp10 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (type_arg->type)))));
								(__temp10 == NULL ? NULL : (__temp10 = (g_free (__temp10), NULL)));
							}
						}
					}
					vala_interface_writer_write_string (self, ">");
				}
				vala_interface_writer_write_string (self, " ");
				vala_interface_writer_write_identifier (self, vala_formal_parameter_get_name (param));
				(type_args == NULL ? NULL : (type_args = (g_list_free (type_args), NULL)));
			}
		}
	}
	vala_interface_writer_write_string (self, ");");
	vala_interface_writer_write_newline (self);
}


static void vala_interface_writer_real_visit_begin_property (ValaCodeVisitor* base, ValaProperty* prop)
{
	ValaInterfaceWriter * self = VALA_INTERFACE_WRITER (base);
	g_return_if_fail (VALA_IS_PROPERTY (prop));
	if (self->priv->internal_scope) {
		return;
	}
	vala_interface_writer_write_indent (self);
	vala_interface_writer_write_string (self, "public ");
	if (vala_type_reference_get_is_weak (vala_property_get_type_reference (prop))) {
		vala_interface_writer_write_string (self, "weak ");
	}
	char* __temp11 = NULL;
	vala_interface_writer_write_string (self, (__temp11 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (vala_property_get_type_reference (prop)->type)))));
	(__temp11 == NULL ? NULL : (__temp11 = (g_free (__temp11), NULL)));
	GList* type_args = vala_type_reference_get_type_arguments (vala_property_get_type_reference (prop));
	if (type_args != NULL) {
		vala_interface_writer_write_string (self, "<");
		{
			GList * type_arg_it;
			for (type_arg_it = type_args; type_arg_it != NULL; type_arg_it = type_arg_it->next) {
				ValaTypeReference* type_arg = type_arg_it->data;
				{
					if (vala_type_reference_get_is_weak (type_arg)) {
						vala_interface_writer_write_string (self, "weak ");
					}
					char* __temp12 = NULL;
					vala_interface_writer_write_string (self, (__temp12 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (type_arg->type)))));
					(__temp12 == NULL ? NULL : (__temp12 = (g_free (__temp12), NULL)));
				}
			}
		}
		vala_interface_writer_write_string (self, ">");
	}
	vala_interface_writer_write_string (self, " ");
	vala_interface_writer_write_identifier (self, vala_property_get_name (prop));
	vala_interface_writer_write_string (self, " { get; set construct; }");
	vala_interface_writer_write_newline (self);
	(type_args == NULL ? NULL : (type_args = (g_list_free (type_args), NULL)));
}


static void vala_interface_writer_write_indent (ValaInterfaceWriter* self)
{
	g_return_if_fail (VALA_IS_INTERFACE_WRITER (self));
	int i;
	if (!self->priv->bol) {
		fputc ('\n', self->priv->stream);
	}
	for (i = 0; i < self->priv->indent; i++) {
		fputc ('\t', self->priv->stream);
	}
	self->priv->bol = FALSE;
}


static void vala_interface_writer_write_identifier (ValaInterfaceWriter* self, char* s)
{
	g_return_if_fail (VALA_IS_INTERFACE_WRITER (self));
	g_return_if_fail (s != NULL);
	if (g_utf8_collate (s, "namespace") == 0) {
		fputc ('@', self->priv->stream);
	}
	vala_interface_writer_write_string (self, s);
}


static void vala_interface_writer_write_string (ValaInterfaceWriter* self, char* s)
{
	g_return_if_fail (VALA_IS_INTERFACE_WRITER (self));
	g_return_if_fail (s != NULL);
	fprintf (self->priv->stream, "%s", s);
	self->priv->bol = FALSE;
}


static void vala_interface_writer_write_newline (ValaInterfaceWriter* self)
{
	g_return_if_fail (VALA_IS_INTERFACE_WRITER (self));
	fputc ('\n', self->priv->stream);
	self->priv->bol = TRUE;
}


static void vala_interface_writer_write_begin_block (ValaInterfaceWriter* self)
{
	g_return_if_fail (VALA_IS_INTERFACE_WRITER (self));
	if (!self->priv->bol) {
		fputc (' ', self->priv->stream);
	} else {
		vala_interface_writer_write_indent (self);
	}
	fputc ('{', self->priv->stream);
	vala_interface_writer_write_newline (self);
	self->priv->indent++;
}


static void vala_interface_writer_write_end_block (ValaInterfaceWriter* self)
{
	g_return_if_fail (VALA_IS_INTERFACE_WRITER (self));
	self->priv->indent--;
	vala_interface_writer_write_indent (self);
	fprintf (self->priv->stream, "}");
}


void vala_interface_writer_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaInterfaceWriter * self = VALA_INTERFACE_WRITER (object);
	switch (property_id) {
	}
}


void vala_interface_writer_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaInterfaceWriter * self = VALA_INTERFACE_WRITER (object);
	switch (property_id) {
	}
}


static void vala_interface_writer_class_init (ValaInterfaceWriterClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaInterfaceWriterPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_interface_writer_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_interface_writer_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_interface_writer_dispose;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_source_file = vala_interface_writer_real_visit_begin_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_namespace = vala_interface_writer_real_visit_begin_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_namespace = vala_interface_writer_real_visit_end_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_class = vala_interface_writer_real_visit_begin_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_class = vala_interface_writer_real_visit_end_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_struct = vala_interface_writer_real_visit_begin_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_struct = vala_interface_writer_real_visit_end_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_enum = vala_interface_writer_real_visit_begin_enum;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_enum = vala_interface_writer_real_visit_end_enum;
	VALA_CODE_VISITOR_CLASS (klass)->visit_enum_value = vala_interface_writer_real_visit_enum_value;
	VALA_CODE_VISITOR_CLASS (klass)->visit_constant = vala_interface_writer_real_visit_constant;
	VALA_CODE_VISITOR_CLASS (klass)->visit_field = vala_interface_writer_real_visit_field;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_method = vala_interface_writer_real_visit_begin_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_property = vala_interface_writer_real_visit_begin_property;
}


static void vala_interface_writer_init (ValaInterfaceWriter * self)
{
	self->priv = VALA_INTERFACE_WRITER_GET_PRIVATE (self);
	self->priv->bol = TRUE;
	self->priv->internal_scope = FALSE;
}


static void vala_interface_writer_dispose (GObject * obj)
{
	ValaInterfaceWriter * self = VALA_INTERFACE_WRITER (obj);
	(self->priv->stream == NULL ? NULL : (self->priv->stream = (fclose (self->priv->stream), NULL)));
	(self->priv->current_cheader_filename == NULL ? NULL : (self->priv->current_cheader_filename = (g_free (self->priv->current_cheader_filename), NULL)));
	ValaInterfaceWriterClass * klass;
	GObjectClass * parent_class;
	klass = VALA_INTERFACE_WRITER_CLASS (g_type_class_peek (VALA_TYPE_INTERFACE_WRITER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_interface_writer_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaInterfaceWriterClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_interface_writer_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaInterfaceWriter), 0, (GInstanceInitFunc) vala_interface_writer_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "ValaInterfaceWriter", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



