/* valamemberaccess.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valamemberaccess.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>

struct _ValaMemberAccessPrivate {
	ValaExpression* _inner;
	char* _member_name;
};
#define VALA_MEMBER_ACCESS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_MEMBER_ACCESS, ValaMemberAccessPrivate))
enum  {
	VALA_MEMBER_ACCESS_DUMMY_PROPERTY,
	VALA_MEMBER_ACCESS_INNER,
	VALA_MEMBER_ACCESS_MEMBER_NAME,
};
static void vala_member_access_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_member_access_dispose (GObject * obj);

/**
 * Creates a new member access expression.
 *
 * @param inner  parent of the member
 * @param member member name
 * @param source reference to source code
 * @return       newly created member access expression
 */
ValaMemberAccess* vala_member_access_new (ValaExpression* inner, char* member, ValaSourceReference* source)
{
	g_return_val_if_fail (inner == NULL || VALA_IS_EXPRESSION (inner), NULL);
	g_return_val_if_fail (member != NULL, NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return g_object_new (VALA_TYPE_MEMBER_ACCESS, "inner", inner, "member_name", member, "source_reference", source, NULL);
}


static void vala_member_access_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaMemberAccess * self = VALA_MEMBER_ACCESS (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	if (vala_member_access_get_inner (self) != NULL) {
		vala_code_node_accept (VALA_CODE_NODE (vala_member_access_get_inner (self)), visitor);
	}
	vala_code_visitor_visit_member_access (visitor, self);
}


ValaExpression* vala_member_access_get_inner (ValaMemberAccess* self)
{
	g_return_val_if_fail (VALA_IS_MEMBER_ACCESS (self), NULL);
	return self->priv->_inner;
}


void vala_member_access_set_inner (ValaMemberAccess* self, ValaExpression* value)
{
	g_return_if_fail (VALA_IS_MEMBER_ACCESS (self));
	ValaExpression* __temp3 = NULL;
	ValaExpression* __temp2 = NULL;
	self->priv->_inner = (__temp3 = (__temp2 = value, (__temp2 == NULL ? __temp2 : g_object_ref (__temp2))), (self->priv->_inner == NULL ? NULL : (self->priv->_inner = (g_object_unref (self->priv->_inner), NULL))), __temp3);
}


char* vala_member_access_get_member_name (ValaMemberAccess* self)
{
	g_return_val_if_fail (VALA_IS_MEMBER_ACCESS (self), NULL);
	return self->priv->_member_name;
}


void vala_member_access_set_member_name (ValaMemberAccess* self, char* value)
{
	g_return_if_fail (VALA_IS_MEMBER_ACCESS (self));
	char* __temp5 = NULL;
	self->priv->_member_name = (__temp5 = g_strdup (value), (self->priv->_member_name == NULL ? NULL : (self->priv->_member_name = (g_free (self->priv->_member_name), NULL))), __temp5);
}


void vala_member_access_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaMemberAccess * self = VALA_MEMBER_ACCESS (object);
	switch (property_id) {
		case VALA_MEMBER_ACCESS_INNER:
		g_value_set_object (value, vala_member_access_get_inner (self));
		break;
		case VALA_MEMBER_ACCESS_MEMBER_NAME:
		g_value_set_string (value, vala_member_access_get_member_name (self));
		break;
	}
}


void vala_member_access_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaMemberAccess * self = VALA_MEMBER_ACCESS (object);
	switch (property_id) {
		case VALA_MEMBER_ACCESS_INNER:
		vala_member_access_set_inner (self, g_value_get_object (value));
		break;
		case VALA_MEMBER_ACCESS_MEMBER_NAME:
		vala_member_access_set_member_name (self, g_value_get_string (value));
		break;
	}
}


static void vala_member_access_class_init (ValaMemberAccessClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaMemberAccessPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_member_access_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_member_access_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_member_access_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_member_access_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_MEMBER_ACCESS_INNER, g_param_spec_object ("inner", "foo", "bar", VALA_TYPE_EXPRESSION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_MEMBER_ACCESS_MEMBER_NAME, g_param_spec_string ("member-name", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
}


static void vala_member_access_init (ValaMemberAccess * self)
{
	self->priv = VALA_MEMBER_ACCESS_GET_PRIVATE (self);
}


static void vala_member_access_dispose (GObject * obj)
{
	ValaMemberAccess * self = VALA_MEMBER_ACCESS (obj);
	(self->priv->_inner == NULL ? NULL : (self->priv->_inner = (g_object_unref (self->priv->_inner), NULL)));
	(self->priv->_member_name == NULL ? NULL : (self->priv->_member_name = (g_free (self->priv->_member_name), NULL)));
	ValaMemberAccessClass * klass;
	GObjectClass * parent_class;
	klass = VALA_MEMBER_ACCESS_CLASS (g_type_class_peek (VALA_TYPE_MEMBER_ACCESS));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_member_access_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaMemberAccessClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_member_access_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaMemberAccess), 0, (GInstanceInitFunc) vala_member_access_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_EXPRESSION, "ValaMemberAccess", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



