/* valamemorymanager.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valamemorymanager.h>
#include <valasymbol.h>
#include <valatypereference.h>
#include <valadatatype.h>
#include <valacodenode.h>
#include <valacallback.h>
#include <valaformalparameter.h>
#include <valasignal.h>

struct _ValaMemoryManagerPrivate {
	ValaSymbol* current_symbol;
};
#define VALA_MEMORY_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_MEMORY_MANAGER, ValaMemoryManagerPrivate))
enum  {
	VALA_MEMORY_MANAGER_DUMMY_PROPERTY,
};
static void vala_memory_manager_visit_possibly_leaked_expression (ValaMemoryManager* self, ValaExpression* expr);
static void vala_memory_manager_visit_possibly_missing_copy_expression (ValaMemoryManager* self, ValaExpression* expr);
static void vala_memory_manager_real_visit_field (ValaCodeVisitor* base, ValaField* f);
static void vala_memory_manager_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m);
static void vala_memory_manager_real_visit_begin_property (ValaCodeVisitor* base, ValaProperty* prop);
static void vala_memory_manager_real_visit_named_argument (ValaCodeVisitor* base, ValaNamedArgument* n);
static void vala_memory_manager_real_visit_variable_declarator (ValaCodeVisitor* base, ValaVariableDeclarator* decl);
static void vala_memory_manager_real_visit_expression_statement (ValaCodeVisitor* base, ValaExpressionStatement* stmt);
static void vala_memory_manager_real_visit_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt);
static void vala_memory_manager_real_visit_member_access (ValaCodeVisitor* base, ValaMemberAccess* expr);
static void vala_memory_manager_real_visit_end_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr);
static void vala_memory_manager_real_visit_binary_expression (ValaCodeVisitor* base, ValaBinaryExpression* expr);
static void vala_memory_manager_real_visit_end_assignment (ValaCodeVisitor* base, ValaAssignment* a);
static void vala_memory_manager_dispose (GObject * obj);

/**
 * Analyze memory usage in the specified code context.
 *
 * @param context a code context
 */
void vala_memory_manager_analyze (ValaMemoryManager* self, ValaCodeContext* context)
{
	g_return_if_fail (VALA_IS_MEMORY_MANAGER (self));
	g_return_if_fail (VALA_IS_CODE_CONTEXT (context));
	vala_code_context_accept (context, VALA_CODE_VISITOR (self));
}


static void vala_memory_manager_visit_possibly_leaked_expression (ValaMemoryManager* self, ValaExpression* expr)
{
	g_return_if_fail (VALA_IS_MEMORY_MANAGER (self));
	g_return_if_fail (VALA_IS_EXPRESSION (expr));
	if (vala_expression_get_static_type (expr) != NULL && (vala_expression_get_static_type (expr)->type != NULL && vala_data_type_is_reference_type (vala_expression_get_static_type (expr)->type)) && vala_type_reference_get_is_ref (vala_expression_get_static_type (expr))) {
		/* mark reference as leaked */
		vala_expression_set_ref_leaked (expr, TRUE);
	}
}


static void vala_memory_manager_visit_possibly_missing_copy_expression (ValaMemoryManager* self, ValaExpression* expr)
{
	g_return_if_fail (VALA_IS_MEMORY_MANAGER (self));
	g_return_if_fail (VALA_IS_EXPRESSION (expr));
	if (vala_expression_get_static_type (expr) != NULL && (vala_expression_get_static_type (expr)->type != NULL && vala_data_type_is_reference_type (vala_expression_get_static_type (expr)->type)) && !vala_type_reference_get_is_ref (vala_expression_get_static_type (expr))) {
		/* mark reference as missing */
		vala_expression_set_ref_missing (expr, TRUE);
	}
}


static void vala_memory_manager_real_visit_field (ValaCodeVisitor* base, ValaField* f)
{
	ValaMemoryManager * self = VALA_MEMORY_MANAGER (base);
	g_return_if_fail (VALA_IS_FIELD (f));
	if (vala_field_get_initializer (f) != NULL) {
		if (vala_type_reference_get_is_lvalue_ref (vala_field_get_type_reference (f))) {
			vala_memory_manager_visit_possibly_missing_copy_expression (self, vala_field_get_initializer (f));
		} else {
			vala_memory_manager_visit_possibly_leaked_expression (self, vala_field_get_initializer (f));
		}
	}
}


static void vala_memory_manager_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m)
{
	ValaMemoryManager * self = VALA_MEMORY_MANAGER (base);
	g_return_if_fail (VALA_IS_METHOD (m));
	ValaSymbol* __temp1 = NULL;
	ValaSymbol* __temp0 = NULL;
	self->priv->current_symbol = (__temp1 = (__temp0 = vala_code_node_get_symbol (VALA_CODE_NODE (m)), (__temp0 == NULL ? __temp0 : g_object_ref (__temp0))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp1);
}


static void vala_memory_manager_real_visit_begin_property (ValaCodeVisitor* base, ValaProperty* prop)
{
	ValaMemoryManager * self = VALA_MEMORY_MANAGER (base);
	g_return_if_fail (VALA_IS_PROPERTY (prop));
	ValaSymbol* __temp3 = NULL;
	ValaSymbol* __temp2 = NULL;
	self->priv->current_symbol = (__temp3 = (__temp2 = vala_code_node_get_symbol (VALA_CODE_NODE (prop)), (__temp2 == NULL ? __temp2 : g_object_ref (__temp2))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp3);
}


static void vala_memory_manager_real_visit_named_argument (ValaCodeVisitor* base, ValaNamedArgument* n)
{
	ValaMemoryManager * self = VALA_MEMORY_MANAGER (base);
	g_return_if_fail (VALA_IS_NAMED_ARGUMENT (n));
	vala_memory_manager_visit_possibly_leaked_expression (self, vala_named_argument_get_argument (n));
}


static void vala_memory_manager_real_visit_variable_declarator (ValaCodeVisitor* base, ValaVariableDeclarator* decl)
{
	ValaMemoryManager * self = VALA_MEMORY_MANAGER (base);
	g_return_if_fail (VALA_IS_VARIABLE_DECLARATOR (decl));
	if (vala_variable_declarator_get_initializer (decl) != NULL) {
		if (vala_type_reference_get_is_lvalue_ref (decl->type_reference)) {
			vala_memory_manager_visit_possibly_missing_copy_expression (self, vala_variable_declarator_get_initializer (decl));
		} else {
			vala_memory_manager_visit_possibly_leaked_expression (self, vala_variable_declarator_get_initializer (decl));
		}
	}
}


static void vala_memory_manager_real_visit_expression_statement (ValaCodeVisitor* base, ValaExpressionStatement* stmt)
{
	ValaMemoryManager * self = VALA_MEMORY_MANAGER (base);
	g_return_if_fail (VALA_IS_EXPRESSION_STATEMENT (stmt));
	vala_memory_manager_visit_possibly_leaked_expression (self, vala_expression_statement_get_expression (stmt));
}


static void vala_memory_manager_real_visit_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt)
{
	ValaMemoryManager * self = VALA_MEMORY_MANAGER (base);
	g_return_if_fail (VALA_IS_RETURN_STATEMENT (stmt));
	if (vala_return_statement_get_return_expression (stmt) != NULL) {
		if (VALA_IS_METHOD (vala_symbol_get_node (self->priv->current_symbol))) {
			ValaMethod* m = VALA_METHOD (vala_symbol_get_node (self->priv->current_symbol));
			if (vala_type_reference_get_is_ref (vala_method_get_return_type (m))) {
				vala_memory_manager_visit_possibly_missing_copy_expression (self, vala_return_statement_get_return_expression (stmt));
			} else {
				vala_memory_manager_visit_possibly_leaked_expression (self, vala_return_statement_get_return_expression (stmt));
			}
		} else {
			/* property get accessor */
			vala_memory_manager_visit_possibly_leaked_expression (self, vala_return_statement_get_return_expression (stmt));
		}
	}
}


static void vala_memory_manager_real_visit_member_access (ValaCodeVisitor* base, ValaMemberAccess* expr)
{
	ValaMemoryManager * self = VALA_MEMORY_MANAGER (base);
	g_return_if_fail (VALA_IS_MEMBER_ACCESS (expr));
	if (vala_member_access_get_inner (expr) != NULL) {
		vala_memory_manager_visit_possibly_leaked_expression (self, vala_member_access_get_inner (expr));
	}
}


static void vala_memory_manager_real_visit_end_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr)
{
	ValaMemoryManager * self = VALA_MEMORY_MANAGER (base);
	g_return_if_fail (VALA_IS_INVOCATION_EXPRESSION (expr));
	GList* params = NULL;
	ValaSymbol* msym = vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr));
	if (VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (msym))) {
		ValaVariableDeclarator* decl = VALA_VARIABLE_DECLARATOR (vala_symbol_get_node (msym));
		ValaCallback* cb = VALA_CALLBACK (decl->type_reference->type);
		GList* __temp4 = NULL;
		params = (__temp4 = vala_callback_get_parameters (cb), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp4);
	} else if (VALA_IS_FORMAL_PARAMETER (vala_symbol_get_node (msym))) {
		ValaFormalParameter* param = VALA_FORMAL_PARAMETER (vala_symbol_get_node (msym));
		ValaCallback* cb = VALA_CALLBACK (vala_formal_parameter_get_type_reference (param)->type);
		GList* __temp5 = NULL;
		params = (__temp5 = vala_callback_get_parameters (cb), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp5);
	} else if (VALA_IS_METHOD (vala_symbol_get_node (msym))) {
		ValaMethod* m = VALA_METHOD (vala_symbol_get_node (msym));
		GList* __temp6 = NULL;
		params = (__temp6 = vala_method_get_parameters (m), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp6);
	} else {
		ValaSignal* sig = VALA_SIGNAL (vala_symbol_get_node (msym));
		GList* __temp7 = NULL;
		params = (__temp7 = vala_signal_get_parameters (sig), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp7);
	}
	{
		GList * arg_it;
		for (arg_it = vala_invocation_expression_get_argument_list (expr); arg_it != NULL; arg_it = arg_it->next) {
			ValaExpression* arg = arg_it->data;
			{
				if (params != NULL) {
					ValaFormalParameter* param = VALA_FORMAL_PARAMETER (params->data);
					if (!vala_formal_parameter_get_ellipsis (param) && ((vala_formal_parameter_get_type_reference (param)->type != NULL && vala_data_type_is_reference_type (vala_formal_parameter_get_type_reference (param)->type)) || vala_formal_parameter_get_type_reference (param)->type_parameter != NULL)) {
						gboolean is_ref = vala_type_reference_get_is_ref (vala_formal_parameter_get_type_reference (param));
						if (is_ref && vala_formal_parameter_get_type_reference (param)->type_parameter != NULL) {
							if (VALA_IS_MEMBER_ACCESS (vala_invocation_expression_get_call (expr))) {
								ValaTypeReference* instance_type = vala_expression_get_static_type (vala_member_access_get_inner ((VALA_MEMBER_ACCESS (vala_invocation_expression_get_call (expr)))));
								{
									GList * type_arg_it;
									for (type_arg_it = vala_type_reference_get_type_arguments (instance_type); type_arg_it != NULL; type_arg_it = type_arg_it->next) {
										ValaTypeReference* type_arg = type_arg_it->data;
										{
											/* generic method parameters may only be strong references if the type argument is strong, too */
											is_ref = vala_type_reference_get_is_ref (type_arg);
										}
									}
								}
							}
						}
						if (is_ref) {
							vala_memory_manager_visit_possibly_missing_copy_expression (self, arg);
						} else {
							vala_memory_manager_visit_possibly_leaked_expression (self, arg);
						}
					} else {
						vala_memory_manager_visit_possibly_leaked_expression (self, arg);
					}
					GList* __temp9 = NULL;
					GList* __temp8 = NULL;
					params = (__temp9 = (__temp8 = params->next, (__temp8 == NULL ? __temp8 : g_list_copy (__temp8))), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp9);
				} else {
					vala_memory_manager_visit_possibly_leaked_expression (self, arg);
				}
			}
		}
	}
	(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
}


static void vala_memory_manager_real_visit_binary_expression (ValaCodeVisitor* base, ValaBinaryExpression* expr)
{
	ValaMemoryManager * self = VALA_MEMORY_MANAGER (base);
	g_return_if_fail (VALA_IS_BINARY_EXPRESSION (expr));
	vala_memory_manager_visit_possibly_leaked_expression (self, vala_binary_expression_get_left (expr));
	vala_memory_manager_visit_possibly_leaked_expression (self, vala_binary_expression_get_right (expr));
}


static void vala_memory_manager_real_visit_end_assignment (ValaCodeVisitor* base, ValaAssignment* a)
{
	ValaMemoryManager * self = VALA_MEMORY_MANAGER (base);
	g_return_if_fail (VALA_IS_ASSIGNMENT (a));
	if (VALA_IS_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_assignment_get_left (a))))) {
	} else {
		if (vala_type_reference_get_is_lvalue_ref (vala_expression_get_static_type (vala_assignment_get_left (a)))) {
			vala_memory_manager_visit_possibly_missing_copy_expression (self, vala_assignment_get_right (a));
		} else {
			vala_memory_manager_visit_possibly_leaked_expression (self, vala_assignment_get_right (a));
		}
	}
}


void vala_memory_manager_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaMemoryManager * self = VALA_MEMORY_MANAGER (object);
	switch (property_id) {
	}
}


void vala_memory_manager_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaMemoryManager * self = VALA_MEMORY_MANAGER (object);
	switch (property_id) {
	}
}


static void vala_memory_manager_class_init (ValaMemoryManagerClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaMemoryManagerPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_memory_manager_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_memory_manager_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_memory_manager_dispose;
	VALA_CODE_VISITOR_CLASS (klass)->visit_field = vala_memory_manager_real_visit_field;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_method = vala_memory_manager_real_visit_begin_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_property = vala_memory_manager_real_visit_begin_property;
	VALA_CODE_VISITOR_CLASS (klass)->visit_named_argument = vala_memory_manager_real_visit_named_argument;
	VALA_CODE_VISITOR_CLASS (klass)->visit_variable_declarator = vala_memory_manager_real_visit_variable_declarator;
	VALA_CODE_VISITOR_CLASS (klass)->visit_expression_statement = vala_memory_manager_real_visit_expression_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_return_statement = vala_memory_manager_real_visit_return_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_member_access = vala_memory_manager_real_visit_member_access;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_invocation_expression = vala_memory_manager_real_visit_end_invocation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_binary_expression = vala_memory_manager_real_visit_binary_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_assignment = vala_memory_manager_real_visit_end_assignment;
}


static void vala_memory_manager_init (ValaMemoryManager * self)
{
	self->priv = VALA_MEMORY_MANAGER_GET_PRIVATE (self);
}


static void vala_memory_manager_dispose (GObject * obj)
{
	ValaMemoryManager * self = VALA_MEMORY_MANAGER (obj);
	(self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL)));
	ValaMemoryManagerClass * klass;
	GObjectClass * parent_class;
	klass = VALA_MEMORY_MANAGER_CLASS (g_type_class_peek (VALA_TYPE_MEMORY_MANAGER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_memory_manager_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaMemoryManagerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_memory_manager_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaMemoryManager), 0, (GInstanceInitFunc) vala_memory_manager_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "ValaMemoryManager", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



