/* valanamespace.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valanamespace.h>
#include <valasourcereference.h>
#include <valaclass.h>
#include <valadatatype.h>
#include <valainterface.h>
#include <valastruct.h>
#include <valaenum.h>
#include <valaflags.h>
#include <valacallback.h>
#include <valafield.h>
#include <valamethod.h>
#include <valacodevisitor.h>
#include <valasourcefile.h>
#include <valaattribute.h>
#include <valanamedargument.h>
#include <valaliteralexpression.h>
#include <valaliteral.h>
#include <valastringliteral.h>

struct _ValaNamespacePrivate {
	char* _name;
	GList* classes;
	GList* interfaces;
	GList* structs;
	GList* enums;
	GList* flags_;
	GList* callbacks;
	GList* fields;
	GList* methods;
	GList* cheader_filenames;
};
#define VALA_NAMESPACE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_NAMESPACE, ValaNamespacePrivate))
enum  {
	VALA_NAMESPACE_DUMMY_PROPERTY,
	VALA_NAMESPACE_NAME,
};
static void vala_namespace_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_namespace_process_ccode_attribute (ValaNamespace* self, ValaAttribute* a);
static void vala_namespace_dispose (GObject * obj);

ValaNamespace* vala_namespace_new (char* name, ValaSourceReference* source)
{
	g_return_val_if_fail (name != NULL, NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_NAMESPACE, "name", name, "source_reference", source, NULL));
}


void vala_namespace_add_class (ValaNamespace* self, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_NAMESPACE (self));
	g_return_if_fail (VALA_IS_CLASS (cl));
	self->priv->classes = g_list_append (self->priv->classes, g_object_ref (cl));
	VALA_DATA_TYPE (cl)->namespace = self;
}


void vala_namespace_remove_class (ValaNamespace* self, ValaClass* cl)
{
	g_return_if_fail (VALA_IS_NAMESPACE (self));
	g_return_if_fail (VALA_IS_CLASS (cl));
	self->priv->classes = g_list_remove (self->priv->classes, cl);
	VALA_DATA_TYPE (cl)->namespace = NULL;
}


void vala_namespace_add_interface (ValaNamespace* self, ValaInterface* iface)
{
	g_return_if_fail (VALA_IS_NAMESPACE (self));
	g_return_if_fail (VALA_IS_INTERFACE (iface));
	self->priv->interfaces = g_list_append (self->priv->interfaces, g_object_ref (iface));
	VALA_DATA_TYPE (iface)->namespace = self;
}


void vala_namespace_add_struct (ValaNamespace* self, ValaStruct* st)
{
	g_return_if_fail (VALA_IS_NAMESPACE (self));
	g_return_if_fail (VALA_IS_STRUCT (st));
	self->priv->structs = g_list_append (self->priv->structs, g_object_ref (st));
	VALA_DATA_TYPE (st)->namespace = self;
}


void vala_namespace_remove_struct (ValaNamespace* self, ValaStruct* st)
{
	g_return_if_fail (VALA_IS_NAMESPACE (self));
	g_return_if_fail (VALA_IS_STRUCT (st));
	self->priv->structs = g_list_remove (self->priv->structs, st);
	VALA_DATA_TYPE (st)->namespace = NULL;
}


void vala_namespace_add_enum (ValaNamespace* self, ValaEnum* en)
{
	g_return_if_fail (VALA_IS_NAMESPACE (self));
	g_return_if_fail (VALA_IS_ENUM (en));
	self->priv->enums = g_list_append (self->priv->enums, g_object_ref (en));
	VALA_DATA_TYPE (en)->namespace = self;
}


void vala_namespace_add_flags (ValaNamespace* self, ValaFlags* fl)
{
	g_return_if_fail (VALA_IS_NAMESPACE (self));
	g_return_if_fail (VALA_IS_FLAGS (fl));
	self->priv->flags_ = g_list_append (self->priv->flags_, g_object_ref (fl));
	VALA_DATA_TYPE (fl)->namespace = self;
}


void vala_namespace_add_callback (ValaNamespace* self, ValaCallback* cb)
{
	g_return_if_fail (VALA_IS_NAMESPACE (self));
	g_return_if_fail (VALA_IS_CALLBACK (cb));
	self->priv->callbacks = g_list_append (self->priv->callbacks, g_object_ref (cb));
	VALA_DATA_TYPE (cb)->namespace = self;
}


GList* vala_namespace_get_structs (ValaNamespace* self)
{
	g_return_val_if_fail (VALA_IS_NAMESPACE (self), NULL);
	return g_list_copy (self->priv->structs);
}


GList* vala_namespace_get_classes (ValaNamespace* self)
{
	g_return_val_if_fail (VALA_IS_NAMESPACE (self), NULL);
	return g_list_copy (self->priv->classes);
}


void vala_namespace_add_field (ValaNamespace* self, ValaField* f)
{
	g_return_if_fail (VALA_IS_NAMESPACE (self));
	g_return_if_fail (VALA_IS_FIELD (f));
	self->priv->fields = g_list_append (self->priv->fields, g_object_ref (f));
}


void vala_namespace_add_method (ValaNamespace* self, ValaMethod* m)
{
	g_return_if_fail (VALA_IS_NAMESPACE (self));
	g_return_if_fail (VALA_IS_METHOD (m));
	self->priv->methods = g_list_append (self->priv->methods, g_object_ref (m));
}


static void vala_namespace_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaNamespace * self = VALA_NAMESPACE (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_begin_namespace (visitor, self);
	{
		GList * cl_it;
		for (cl_it = self->priv->classes; cl_it != NULL; cl_it = cl_it->next) {
			ValaClass* cl = cl_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (cl), visitor);
			}
		}
	}
	{
		GList * iface_it;
		for (iface_it = self->priv->interfaces; iface_it != NULL; iface_it = iface_it->next) {
			ValaInterface* iface = iface_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (iface), visitor);
			}
		}
	}
	{
		GList * st_it;
		for (st_it = self->priv->structs; st_it != NULL; st_it = st_it->next) {
			ValaStruct* st = st_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (st), visitor);
			}
		}
	}
	{
		GList * en_it;
		for (en_it = self->priv->enums; en_it != NULL; en_it = en_it->next) {
			ValaEnum* en = en_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (en), visitor);
			}
		}
	}
	{
		GList * fl_it;
		for (fl_it = self->priv->flags_; fl_it != NULL; fl_it = fl_it->next) {
			ValaFlags* fl = fl_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (fl), visitor);
			}
		}
	}
	{
		GList * cb_it;
		for (cb_it = self->priv->callbacks; cb_it != NULL; cb_it = cb_it->next) {
			ValaCallback* cb = cb_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (cb), visitor);
			}
		}
	}
	{
		GList * f_it;
		for (f_it = self->priv->fields; f_it != NULL; f_it = f_it->next) {
			ValaField* f = f_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (f), visitor);
			}
		}
	}
	{
		GList * m_it;
		for (m_it = self->priv->methods; m_it != NULL; m_it = m_it->next) {
			ValaMethod* m = m_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (m), visitor);
			}
		}
	}
	vala_code_visitor_visit_end_namespace (visitor, self);
}


char* vala_namespace_camel_case_to_lower_case (char* camel_case)
{
	GString* result = g_string_new ("");
	char* i = camel_case;
	gboolean first = TRUE;
	while (g_utf8_strlen (i, -1) > 0) {
		gunichar c = g_utf8_get_char (i);
		if (g_unichar_isupper (c) && !first) {
			/* current character is upper case and
			 * we're not at the beginning */
			char* t = g_utf8_prev_char (i);
			gboolean prev_upper = g_unichar_isupper (g_utf8_get_char (t));
			t = g_utf8_next_char (i);
			gboolean next_upper = g_unichar_isupper (g_utf8_get_char (t));
			if (!prev_upper || (g_utf8_strlen (i, -1) >= 2 && !next_upper)) {
				/* previous character wasn't upper case or
				 * next character isn't upper case*/
				int len = g_utf8_strlen (result->str, -1);
				if (len != 1 && g_utf8_get_char (g_utf8_offset_to_pointer (result->str, len - 2)) != '_') {
					/* we're not creating 1 character words */
					g_string_append_c (result, '_');
				}
			}
		}
		g_string_append_unichar (result, g_unichar_tolower (c));
		first = FALSE;
		i = g_utf8_next_char (i);
	}
	char* __temp3 = NULL;
	char* __temp4 = NULL;
	return (__temp4 = (__temp3 = result->str, (__temp3 == NULL ? __temp3 : g_strdup (__temp3))), (result == NULL ? NULL : (result = (g_string_free (result, TRUE), NULL))), __temp4);
	(result == NULL ? NULL : (result = (g_string_free (result, TRUE), NULL)));
}


char* vala_namespace_get_cprefix (ValaNamespace* self)
{
	g_return_val_if_fail (VALA_IS_NAMESPACE (self), NULL);
	if (self->cprefix == NULL) {
		if (vala_namespace_get_name (self) == NULL) {
			char* __temp5 = NULL;
			self->cprefix = (__temp5 = g_strdup (""), (self->cprefix == NULL ? NULL : (self->cprefix = (g_free (self->cprefix), NULL))), __temp5);
		} else {
			char* __temp7 = NULL;
			char* __temp6 = NULL;
			self->cprefix = (__temp7 = (__temp6 = vala_namespace_get_name (self), (__temp6 == NULL ? __temp6 : g_strdup (__temp6))), (self->cprefix == NULL ? NULL : (self->cprefix = (g_free (self->cprefix), NULL))), __temp7);
		}
	}
	return self->cprefix;
}


void vala_namespace_set_cprefix (ValaNamespace* self, char* cprefix)
{
	g_return_if_fail (VALA_IS_NAMESPACE (self));
	char* __temp10 = NULL;
	char* __temp9 = NULL;
	self->cprefix = (__temp10 = (__temp9 = cprefix, (__temp9 == NULL ? __temp9 : g_strdup (__temp9))), (self->cprefix == NULL ? NULL : (self->cprefix = (g_free (self->cprefix), NULL))), __temp10);
}


char* vala_namespace_get_lower_case_cprefix (ValaNamespace* self)
{
	g_return_val_if_fail (VALA_IS_NAMESPACE (self), NULL);
	if (self->lower_case_cprefix == NULL) {
		if (vala_namespace_get_name (self) == NULL) {
			char* __temp11 = NULL;
			self->lower_case_cprefix = (__temp11 = g_strdup (""), (self->lower_case_cprefix == NULL ? NULL : (self->lower_case_cprefix = (g_free (self->lower_case_cprefix), NULL))), __temp11);
		} else {
			char* __temp13 = NULL;
			char* __temp12 = NULL;
			self->lower_case_cprefix = (__temp13 = g_strdup_printf ("%s_", (__temp12 = vala_namespace_camel_case_to_lower_case (vala_namespace_get_name (self)))), (self->lower_case_cprefix == NULL ? NULL : (self->lower_case_cprefix = (g_free (self->lower_case_cprefix), NULL))), __temp13);
			(__temp12 == NULL ? NULL : (__temp12 = (g_free (__temp12), NULL)));
		}
	}
	return self->lower_case_cprefix;
}


void vala_namespace_set_lower_case_cprefix (ValaNamespace* self, char* cprefix)
{
	g_return_if_fail (VALA_IS_NAMESPACE (self));
	char* __temp16 = NULL;
	char* __temp15 = NULL;
	self->lower_case_cprefix = (__temp16 = (__temp15 = cprefix, (__temp15 == NULL ? __temp15 : g_strdup (__temp15))), (self->lower_case_cprefix == NULL ? NULL : (self->lower_case_cprefix = (g_free (self->lower_case_cprefix), NULL))), __temp16);
}


GList* vala_namespace_get_cheader_filenames (ValaNamespace* self)
{
	g_return_val_if_fail (VALA_IS_NAMESPACE (self), NULL);
	if (self->priv->cheader_filenames == NULL) {
		char* __temp17 = NULL;
		self->priv->cheader_filenames = g_list_append (self->priv->cheader_filenames, (__temp17 = vala_source_file_get_cheader_filename (vala_source_reference_get_file (vala_code_node_get_source_reference (VALA_CODE_NODE (self)))), (__temp17 == NULL ? __temp17 : g_strdup (__temp17))));
	}
	return g_list_copy (self->priv->cheader_filenames);
}


static void vala_namespace_process_ccode_attribute (ValaNamespace* self, ValaAttribute* a)
{
	g_return_if_fail (VALA_IS_NAMESPACE (self));
	g_return_if_fail (VALA_IS_ATTRIBUTE (a));
	{
		GList * arg_it;
		for (arg_it = a->args; arg_it != NULL; arg_it = arg_it->next) {
			ValaNamedArgument* arg = arg_it->data;
			{
				if (g_utf8_collate (vala_named_argument_get_name (arg), "cprefix") == 0) {
					/* this will already be checked during semantic analysis */
					if (VALA_IS_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))) {
						ValaLiteral* lit = vala_literal_expression_get_literal ((VALA_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))));
						if (VALA_IS_STRING_LITERAL (lit)) {
							char* __temp19 = NULL;
							vala_namespace_set_cprefix (self, (__temp19 = vala_string_literal_eval ((VALA_STRING_LITERAL (lit)))));
							(__temp19 == NULL ? NULL : (__temp19 = (g_free (__temp19), NULL)));
						}
					}
				} else if (g_utf8_collate (vala_named_argument_get_name (arg), "lower_case_cprefix") == 0) {
					/* this will already be checked during semantic analysis */
					if (VALA_IS_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))) {
						ValaLiteral* lit = vala_literal_expression_get_literal ((VALA_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))));
						if (VALA_IS_STRING_LITERAL (lit)) {
							char* __temp20 = NULL;
							vala_namespace_set_lower_case_cprefix (self, (__temp20 = vala_string_literal_eval ((VALA_STRING_LITERAL (lit)))));
							(__temp20 == NULL ? NULL : (__temp20 = (g_free (__temp20), NULL)));
						}
					}
				} else if (g_utf8_collate (vala_named_argument_get_name (arg), "cheader_filename") == 0) {
					/* this will already be checked during semantic analysis */
					if (VALA_IS_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))) {
						ValaLiteral* lit = vala_literal_expression_get_literal ((VALA_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))));
						if (VALA_IS_STRING_LITERAL (lit)) {
							char* val = vala_string_literal_eval ((VALA_STRING_LITERAL (lit)));
							{
								char** filename_it;
								for (filename_it = g_strsplit (val, ",", 0); *filename_it != NULL; filename_it = filename_it + 1) {
									char* filename = *filename_it;
									{
										char* __temp21 = NULL;
										self->priv->cheader_filenames = g_list_append (self->priv->cheader_filenames, (__temp21 = filename, (__temp21 == NULL ? __temp21 : g_strdup (__temp21))));
									}
								}
							}
							(val == NULL ? NULL : (val = (g_free (val), NULL)));
						}
					}
				}
			}
		}
	}
}


void vala_namespace_process_attributes (ValaNamespace* self)
{
	g_return_if_fail (VALA_IS_NAMESPACE (self));
	{
		GList * a_it;
		for (a_it = VALA_CODE_NODE (self)->attributes; a_it != NULL; a_it = a_it->next) {
			ValaAttribute* a = a_it->data;
			{
				if (g_utf8_collate (vala_attribute_get_name (a), "CCode") == 0) {
					vala_namespace_process_ccode_attribute (self, a);
				}
			}
		}
	}
}


char* vala_namespace_get_name (ValaNamespace* self)
{
	g_return_val_if_fail (VALA_IS_NAMESPACE (self), NULL);
	return self->priv->_name;
}


void vala_namespace_set_name (ValaNamespace* self, char* value)
{
	g_return_if_fail (VALA_IS_NAMESPACE (self));
	char* __temp24 = NULL;
	char* __temp23 = NULL;
	self->priv->_name = (__temp24 = (__temp23 = value, (__temp23 == NULL ? __temp23 : g_strdup (__temp23))), (self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL))), __temp24);
}


void vala_namespace_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaNamespace * self = VALA_NAMESPACE (object);
	switch (property_id) {
		case VALA_NAMESPACE_NAME:
		g_value_set_string (value, vala_namespace_get_name (self));
		break;
	}
}


void vala_namespace_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaNamespace * self = VALA_NAMESPACE (object);
	switch (property_id) {
		case VALA_NAMESPACE_NAME:
		vala_namespace_set_name (self, g_value_get_string (value));
		break;
	}
}


static void vala_namespace_class_init (ValaNamespaceClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaNamespacePrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_namespace_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_namespace_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_namespace_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_namespace_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_NAMESPACE_NAME, g_param_spec_string ("name", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vala_namespace_init (ValaNamespace * self)
{
	self->priv = VALA_NAMESPACE_GET_PRIVATE (self);
}


static void vala_namespace_dispose (GObject * obj)
{
	ValaNamespace * self = VALA_NAMESPACE (obj);
	(self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL)));
	(self->priv->classes == NULL ? NULL : (self->priv->classes = (g_list_foreach (self->priv->classes, (GFunc) g_object_unref, NULL), g_list_free (self->priv->classes), NULL)));
	(self->priv->interfaces == NULL ? NULL : (self->priv->interfaces = (g_list_foreach (self->priv->interfaces, (GFunc) g_object_unref, NULL), g_list_free (self->priv->interfaces), NULL)));
	(self->priv->structs == NULL ? NULL : (self->priv->structs = (g_list_foreach (self->priv->structs, (GFunc) g_object_unref, NULL), g_list_free (self->priv->structs), NULL)));
	(self->priv->enums == NULL ? NULL : (self->priv->enums = (g_list_foreach (self->priv->enums, (GFunc) g_object_unref, NULL), g_list_free (self->priv->enums), NULL)));
	(self->priv->flags_ == NULL ? NULL : (self->priv->flags_ = (g_list_foreach (self->priv->flags_, (GFunc) g_object_unref, NULL), g_list_free (self->priv->flags_), NULL)));
	(self->priv->callbacks == NULL ? NULL : (self->priv->callbacks = (g_list_foreach (self->priv->callbacks, (GFunc) g_object_unref, NULL), g_list_free (self->priv->callbacks), NULL)));
	(self->priv->fields == NULL ? NULL : (self->priv->fields = (g_list_foreach (self->priv->fields, (GFunc) g_object_unref, NULL), g_list_free (self->priv->fields), NULL)));
	(self->priv->methods == NULL ? NULL : (self->priv->methods = (g_list_foreach (self->priv->methods, (GFunc) g_object_unref, NULL), g_list_free (self->priv->methods), NULL)));
	(self->cprefix == NULL ? NULL : (self->cprefix = (g_free (self->cprefix), NULL)));
	(self->lower_case_cprefix == NULL ? NULL : (self->lower_case_cprefix = (g_free (self->lower_case_cprefix), NULL)));
	(self->priv->cheader_filenames == NULL ? NULL : (self->priv->cheader_filenames = (g_list_foreach (self->priv->cheader_filenames, (GFunc) g_free, NULL), g_list_free (self->priv->cheader_filenames), NULL)));
	ValaNamespaceClass * klass;
	GObjectClass * parent_class;
	klass = VALA_NAMESPACE_CLASS (g_type_class_peek (VALA_TYPE_NAMESPACE));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_namespace_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaNamespaceClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_namespace_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaNamespace), 0, (GInstanceInitFunc) vala_namespace_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_NODE, "ValaNamespace", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



