/* valaproperty.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valaproperty.h>
#include <valatypereference.h>
#include <valapropertyaccessor.h>
#include <valasourcereference.h>
#include <valacodevisitor.h>
#include <valasymbol.h>
#include <valaclass.h>
#include <valanamespace.h>
#include <valaattribute.h>

struct _ValaPropertyPrivate {
	char* _name;
	ValaTypeReference* _type_reference;
	ValaPropertyAccessor* _get_accessor;
	ValaPropertyAccessor* _set_accessor;
};
#define VALA_PROPERTY_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_PROPERTY, ValaPropertyPrivate))
enum  {
	VALA_PROPERTY_DUMMY_PROPERTY,
	VALA_PROPERTY_NAME,
	VALA_PROPERTY_TYPE_REFERENCE,
	VALA_PROPERTY_GET_ACCESSOR,
	VALA_PROPERTY_SET_ACCESSOR,
};
static void vala_property_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_property_dispose (GObject * obj);

ValaProperty* vala_property_new (char* name, ValaTypeReference* type, ValaPropertyAccessor* get_accessor, ValaPropertyAccessor* set_accessor, ValaSourceReference* source)
{
	g_return_val_if_fail (type == NULL || VALA_IS_TYPE_REFERENCE (type), NULL);
	g_return_val_if_fail (get_accessor == NULL || VALA_IS_PROPERTY_ACCESSOR (get_accessor), NULL);
	g_return_val_if_fail (set_accessor == NULL || VALA_IS_PROPERTY_ACCESSOR (set_accessor), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_PROPERTY, "name", name, "type_reference", type, "get_accessor", get_accessor, "set_accessor", set_accessor, "source_reference", source, NULL));
}


static void vala_property_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaProperty * self = VALA_PROPERTY (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_begin_property (visitor, self);
	vala_code_node_accept (VALA_CODE_NODE (vala_property_get_type_reference (self)), visitor);
	if (vala_property_get_get_accessor (self) != NULL) {
		vala_code_node_accept (VALA_CODE_NODE (vala_property_get_get_accessor (self)), visitor);
	}
	if (vala_property_get_set_accessor (self) != NULL) {
		vala_code_node_accept (VALA_CODE_NODE (vala_property_get_set_accessor (self)), visitor);
	}
	vala_code_visitor_visit_end_property (visitor, self);
}


char* vala_property_get_upper_case_cname (ValaProperty* self)
{
	g_return_val_if_fail (VALA_IS_PROPERTY (self), NULL);
	char* __temp3 = NULL;
	char* __temp2 = NULL;
	char* __temp1 = NULL;
	char* __temp4 = NULL;
	return (__temp4 = g_utf8_strup ((__temp3 = g_strdup_printf ("%s_%s", (__temp1 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE ((VALA_CLASS (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (self))->parent_symbol)))), NULL)), (__temp2 = vala_namespace_camel_case_to_lower_case (vala_property_get_name (self))))), -1), (__temp3 == NULL ? NULL : (__temp3 = (g_free (__temp3), NULL))), (__temp2 == NULL ? NULL : (__temp2 = (g_free (__temp2), NULL))), (__temp1 == NULL ? NULL : (__temp1 = (g_free (__temp1), NULL))), __temp4);
}


ValaCCodeConstant* vala_property_get_canonical_cconstant (ValaProperty* self)
{
	g_return_val_if_fail (VALA_IS_PROPERTY (self), NULL);
	GString* str = g_string_new ("\"");
	char* i = vala_property_get_name (self);
	while (g_utf8_strlen (i, -1) > 0) {
		gunichar c = g_utf8_get_char (i);
		if (c == '_') {
			g_string_append_c (str, '-');
		} else {
			g_string_append_unichar (str, c);
		}
		i = g_utf8_next_char (i);
	}
	g_string_append_c (str, '"');
	ValaCCodeConstant* __temp6 = NULL;
	return (__temp6 = g_object_new (VALA_TYPE_CCODE_CONSTANT, "name", str->str, NULL), (str == NULL ? NULL : (str = (g_string_free (str, TRUE), NULL))), __temp6);
	(str == NULL ? NULL : (str = (g_string_free (str, TRUE), NULL)));
}


void vala_property_process_attributes (ValaProperty* self)
{
	g_return_if_fail (VALA_IS_PROPERTY (self));
	{
		GList * a_it;
		for (a_it = VALA_CODE_NODE (self)->attributes; a_it != NULL; a_it = a_it->next) {
			ValaAttribute* a = a_it->data;
			{
				if (g_utf8_collate (vala_attribute_get_name (a), "NoAccessorMethod") == 0) {
					self->no_accessor_method = TRUE;
				}
			}
		}
	}
}


char* vala_property_get_name (ValaProperty* self)
{
	g_return_val_if_fail (VALA_IS_PROPERTY (self), NULL);
	return self->priv->_name;
}


void vala_property_set_name (ValaProperty* self, char* value)
{
	g_return_if_fail (VALA_IS_PROPERTY (self));
	char* __temp9 = NULL;
	char* __temp8 = NULL;
	self->priv->_name = (__temp9 = (__temp8 = value, (__temp8 == NULL ? __temp8 : g_strdup (__temp8))), (self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL))), __temp9);
}


ValaTypeReference* vala_property_get_type_reference (ValaProperty* self)
{
	g_return_val_if_fail (VALA_IS_PROPERTY (self), NULL);
	return self->priv->_type_reference;
}


void vala_property_set_type_reference (ValaProperty* self, ValaTypeReference* value)
{
	g_return_if_fail (VALA_IS_PROPERTY (self));
	ValaTypeReference* __temp12 = NULL;
	ValaTypeReference* __temp11 = NULL;
	self->priv->_type_reference = (__temp12 = (__temp11 = value, (__temp11 == NULL ? __temp11 : g_object_ref (__temp11))), (self->priv->_type_reference == NULL ? NULL : (self->priv->_type_reference = (g_object_unref (self->priv->_type_reference), NULL))), __temp12);
}


ValaPropertyAccessor* vala_property_get_get_accessor (ValaProperty* self)
{
	g_return_val_if_fail (VALA_IS_PROPERTY (self), NULL);
	return self->priv->_get_accessor;
}


void vala_property_set_get_accessor (ValaProperty* self, ValaPropertyAccessor* value)
{
	g_return_if_fail (VALA_IS_PROPERTY (self));
	ValaPropertyAccessor* __temp15 = NULL;
	ValaPropertyAccessor* __temp14 = NULL;
	self->priv->_get_accessor = (__temp15 = (__temp14 = value, (__temp14 == NULL ? __temp14 : g_object_ref (__temp14))), (self->priv->_get_accessor == NULL ? NULL : (self->priv->_get_accessor = (g_object_unref (self->priv->_get_accessor), NULL))), __temp15);
}


ValaPropertyAccessor* vala_property_get_set_accessor (ValaProperty* self)
{
	g_return_val_if_fail (VALA_IS_PROPERTY (self), NULL);
	return self->priv->_set_accessor;
}


void vala_property_set_set_accessor (ValaProperty* self, ValaPropertyAccessor* value)
{
	g_return_if_fail (VALA_IS_PROPERTY (self));
	ValaPropertyAccessor* __temp18 = NULL;
	ValaPropertyAccessor* __temp17 = NULL;
	self->priv->_set_accessor = (__temp18 = (__temp17 = value, (__temp17 == NULL ? __temp17 : g_object_ref (__temp17))), (self->priv->_set_accessor == NULL ? NULL : (self->priv->_set_accessor = (g_object_unref (self->priv->_set_accessor), NULL))), __temp18);
}


void vala_property_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaProperty * self = VALA_PROPERTY (object);
	switch (property_id) {
		case VALA_PROPERTY_NAME:
		g_value_set_string (value, vala_property_get_name (self));
		break;
		case VALA_PROPERTY_TYPE_REFERENCE:
		g_value_set_object (value, vala_property_get_type_reference (self));
		break;
		case VALA_PROPERTY_GET_ACCESSOR:
		g_value_set_object (value, vala_property_get_get_accessor (self));
		break;
		case VALA_PROPERTY_SET_ACCESSOR:
		g_value_set_object (value, vala_property_get_set_accessor (self));
		break;
	}
}


void vala_property_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaProperty * self = VALA_PROPERTY (object);
	switch (property_id) {
		case VALA_PROPERTY_NAME:
		vala_property_set_name (self, g_value_get_string (value));
		break;
		case VALA_PROPERTY_TYPE_REFERENCE:
		vala_property_set_type_reference (self, g_value_get_object (value));
		break;
		case VALA_PROPERTY_GET_ACCESSOR:
		vala_property_set_get_accessor (self, g_value_get_object (value));
		break;
		case VALA_PROPERTY_SET_ACCESSOR:
		vala_property_set_set_accessor (self, g_value_get_object (value));
		break;
	}
}


static void vala_property_class_init (ValaPropertyClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaPropertyPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_property_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_property_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_property_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_property_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_PROPERTY_NAME, g_param_spec_string ("name", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_PROPERTY_TYPE_REFERENCE, g_param_spec_object ("type-reference", "foo", "bar", VALA_TYPE_TYPE_REFERENCE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_PROPERTY_GET_ACCESSOR, g_param_spec_object ("get-accessor", "foo", "bar", VALA_TYPE_PROPERTY_ACCESSOR, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_PROPERTY_SET_ACCESSOR, g_param_spec_object ("set-accessor", "foo", "bar", VALA_TYPE_PROPERTY_ACCESSOR, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vala_property_init (ValaProperty * self)
{
	self->priv = VALA_PROPERTY_GET_PRIVATE (self);
}


static void vala_property_dispose (GObject * obj)
{
	ValaProperty * self = VALA_PROPERTY (obj);
	(self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL)));
	(self->priv->_type_reference == NULL ? NULL : (self->priv->_type_reference = (g_object_unref (self->priv->_type_reference), NULL)));
	(self->priv->_get_accessor == NULL ? NULL : (self->priv->_get_accessor = (g_object_unref (self->priv->_get_accessor), NULL)));
	(self->priv->_set_accessor == NULL ? NULL : (self->priv->_set_accessor = (g_object_unref (self->priv->_set_accessor), NULL)));
	(self->this_parameter == NULL ? NULL : (self->this_parameter = (g_object_unref (self->this_parameter), NULL)));
	ValaPropertyClass * klass;
	GObjectClass * parent_class;
	klass = VALA_PROPERTY_CLASS (g_type_class_peek (VALA_TYPE_PROPERTY));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_property_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaPropertyClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_property_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaProperty), 0, (GInstanceInitFunc) vala_property_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_NODE, "ValaProperty", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



