/* valasemanticanalyzer.vala
 *
 * Copyright (C) 2006  Jürg Billeter, Raffaele Sandrini
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 *	Raffaele Sandrini <rasa@gmx.ch>
 */

#include <valasemanticanalyzer.h>
#include <valamemberaccessibility.h>
#include <valareport.h>
#include <valasourcereference.h>
#include <valainterface.h>
#include <valaexpression.h>
#include <valacallback.h>
#include <valaliteral.h>
#include <valaconstant.h>
#include <valaenumvalue.h>
#include <valanamespacereference.h>
#include <valaarray.h>
#include <valasignal.h>
#include <valastatement.h>

struct _ValaSemanticAnalyzerPrivate {
	gboolean _memory_management;
	ValaSymbol* root_symbol;
	ValaSymbol* current_symbol;
	ValaSourceFile* current_source_file;
	ValaTypeReference* current_return_type;
	GList* current_using_directives;
	ValaTypeReference* bool_type;
	ValaTypeReference* string_type;
	ValaTypeReference* int_type;
	ValaDataType* pointer_type;
	ValaDataType* initially_unowned_type;
	int next_lambda_id;
};
#define VALA_SEMANTIC_ANALYZER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_SEMANTIC_ANALYZER, ValaSemanticAnalyzerPrivate))
enum  {
	VALA_SEMANTIC_ANALYZER_DUMMY_PROPERTY,
	VALA_SEMANTIC_ANALYZER_MEMORY_MANAGEMENT,
};
static void vala_semantic_analyzer_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* file);
static void vala_semantic_analyzer_real_visit_end_source_file (ValaCodeVisitor* base, ValaSourceFile* file);
static void vala_semantic_analyzer_real_visit_begin_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
static void vala_semantic_analyzer_real_visit_end_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
static void vala_semantic_analyzer_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_semantic_analyzer_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_semantic_analyzer_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vala_semantic_analyzer_real_visit_end_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vala_semantic_analyzer_real_visit_field (ValaCodeVisitor* base, ValaField* f);
static void vala_semantic_analyzer_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m);
static void vala_semantic_analyzer_real_visit_end_method (ValaCodeVisitor* base, ValaMethod* m);
static void vala_semantic_analyzer_real_visit_formal_parameter (ValaCodeVisitor* base, ValaFormalParameter* p);
static void vala_semantic_analyzer_real_visit_end_property (ValaCodeVisitor* base, ValaProperty* prop);
static void vala_semantic_analyzer_real_visit_begin_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc);
static void vala_semantic_analyzer_real_visit_end_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc);
static void vala_semantic_analyzer_real_visit_begin_constructor (ValaCodeVisitor* base, ValaConstructor* c);
static void vala_semantic_analyzer_real_visit_end_constructor (ValaCodeVisitor* base, ValaConstructor* c);
static void vala_semantic_analyzer_real_visit_begin_destructor (ValaCodeVisitor* base, ValaDestructor* d);
static void vala_semantic_analyzer_real_visit_end_destructor (ValaCodeVisitor* base, ValaDestructor* d);
static void vala_semantic_analyzer_real_visit_named_argument (ValaCodeVisitor* base, ValaNamedArgument* n);
static void vala_semantic_analyzer_real_visit_begin_block (ValaCodeVisitor* base, ValaBlock* b);
static void vala_semantic_analyzer_real_visit_end_block (ValaCodeVisitor* base, ValaBlock* b);
static void vala_semantic_analyzer_real_visit_variable_declarator (ValaCodeVisitor* base, ValaVariableDeclarator* decl);
static void vala_semantic_analyzer_real_visit_expression_statement (ValaCodeVisitor* base, ValaExpressionStatement* stmt);
static void vala_semantic_analyzer_real_visit_if_statement (ValaCodeVisitor* base, ValaIfStatement* stmt);
static void vala_semantic_analyzer_real_visit_while_statement (ValaCodeVisitor* base, ValaWhileStatement* stmt);
static void vala_semantic_analyzer_real_visit_for_statement (ValaCodeVisitor* base, ValaForStatement* stmt);
static void vala_semantic_analyzer_real_visit_begin_foreach_statement (ValaCodeVisitor* base, ValaForeachStatement* stmt);
static void vala_semantic_analyzer_real_visit_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt);
static void vala_semantic_analyzer_real_visit_boolean_literal (ValaCodeVisitor* base, ValaBooleanLiteral* expr);
static void vala_semantic_analyzer_real_visit_character_literal (ValaCodeVisitor* base, ValaCharacterLiteral* expr);
static void vala_semantic_analyzer_real_visit_integer_literal (ValaCodeVisitor* base, ValaIntegerLiteral* expr);
static void vala_semantic_analyzer_real_visit_real_literal (ValaCodeVisitor* base, ValaRealLiteral* expr);
static void vala_semantic_analyzer_real_visit_string_literal (ValaCodeVisitor* base, ValaStringLiteral* expr);
static void vala_semantic_analyzer_real_visit_null_literal (ValaCodeVisitor* base, ValaNullLiteral* expr);
static void vala_semantic_analyzer_real_visit_literal_expression (ValaCodeVisitor* base, ValaLiteralExpression* expr);
static ValaTypeReference* vala_semantic_analyzer_get_static_type_for_node (ValaSemanticAnalyzer* self, ValaCodeNode* node);
static ValaSymbol* vala_semantic_analyzer_symbol_lookup_inherited (ValaSemanticAnalyzer* self, ValaSymbol* sym, char* name);
static void vala_semantic_analyzer_real_visit_parenthesized_expression (ValaCodeVisitor* base, ValaParenthesizedExpression* expr);
static ValaDataType* vala_semantic_analyzer_find_parent_type (ValaSemanticAnalyzer* self, ValaSymbol* sym);
static void vala_semantic_analyzer_real_visit_member_access (ValaCodeVisitor* base, ValaMemberAccess* expr);
static gboolean vala_semantic_analyzer_is_type_compatible (ValaSemanticAnalyzer* self, ValaTypeReference* expression_type, ValaTypeReference* expected_type);
static void vala_semantic_analyzer_real_visit_begin_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr);
static void vala_semantic_analyzer_real_visit_end_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr);
static void vala_semantic_analyzer_real_visit_element_access (ValaCodeVisitor* base, ValaElementAccess* expr);
static void vala_semantic_analyzer_real_visit_postfix_expression (ValaCodeVisitor* base, ValaPostfixExpression* expr);
static void vala_semantic_analyzer_real_visit_object_creation_expression (ValaCodeVisitor* base, ValaObjectCreationExpression* expr);
static void vala_semantic_analyzer_real_visit_unary_expression (ValaCodeVisitor* base, ValaUnaryExpression* expr);
static void vala_semantic_analyzer_real_visit_cast_expression (ValaCodeVisitor* base, ValaCastExpression* expr);
static gboolean vala_semantic_analyzer_check_binary_type (ValaSemanticAnalyzer* self, ValaBinaryExpression* expr, char* operation);
static void vala_semantic_analyzer_real_visit_binary_expression (ValaCodeVisitor* base, ValaBinaryExpression* expr);
static void vala_semantic_analyzer_real_visit_type_check (ValaCodeVisitor* base, ValaTypeCheck* expr);
static void vala_semantic_analyzer_real_visit_conditional_expression (ValaCodeVisitor* base, ValaConditionalExpression* expr);
static char* vala_semantic_analyzer_get_lambda_name (ValaSemanticAnalyzer* self);
static ValaMethod* vala_semantic_analyzer_find_current_method (ValaSemanticAnalyzer* self);
static void vala_semantic_analyzer_real_visit_begin_lambda_expression (ValaCodeVisitor* base, ValaLambdaExpression* l);
static void vala_semantic_analyzer_real_visit_begin_assignment (ValaCodeVisitor* base, ValaAssignment* a);
static void vala_semantic_analyzer_real_visit_end_assignment (ValaCodeVisitor* base, ValaAssignment* a);
static void vala_semantic_analyzer_dispose (GObject * obj);

/**
 * Analyze and check code in the specified context.
 *
 * @param context a code context
 */
void vala_semantic_analyzer_analyze (ValaSemanticAnalyzer* self, ValaCodeContext* context)
{
	g_return_if_fail (VALA_IS_SEMANTIC_ANALYZER (self));
	g_return_if_fail (VALA_IS_CODE_CONTEXT (context));
	ValaSymbol* __temp0 = NULL;
	self->priv->root_symbol = (__temp0 = g_object_ref (vala_code_context_get_root (context)), (self->priv->root_symbol == NULL ? NULL : (self->priv->root_symbol = (g_object_unref (self->priv->root_symbol), NULL))), __temp0);
	ValaTypeReference* __temp1 = NULL;
	self->priv->bool_type = (__temp1 = g_object_new (VALA_TYPE_TYPE_REFERENCE, NULL), (self->priv->bool_type == NULL ? NULL : (self->priv->bool_type = (g_object_unref (self->priv->bool_type), NULL))), __temp1);
	self->priv->bool_type->type = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "bool")));
	ValaTypeReference* __temp2 = NULL;
	self->priv->string_type = (__temp2 = g_object_new (VALA_TYPE_TYPE_REFERENCE, NULL), (self->priv->string_type == NULL ? NULL : (self->priv->string_type = (g_object_unref (self->priv->string_type), NULL))), __temp2);
	self->priv->string_type->type = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "string")));
	ValaDataType* __temp4 = NULL;
	ValaDataType* __temp3 = NULL;
	self->priv->pointer_type = (__temp4 = (__temp3 = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "pointer"))), (__temp3 == NULL ? __temp3 : g_object_ref (__temp3))), (self->priv->pointer_type == NULL ? NULL : (self->priv->pointer_type = (g_object_unref (self->priv->pointer_type), NULL))), __temp4);
	ValaTypeReference* __temp5 = NULL;
	self->priv->int_type = (__temp5 = g_object_new (VALA_TYPE_TYPE_REFERENCE, NULL), (self->priv->int_type == NULL ? NULL : (self->priv->int_type = (g_object_unref (self->priv->int_type), NULL))), __temp5);
	self->priv->int_type->type = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "int")));
	ValaSymbol* glib_ns = vala_symbol_lookup (self->priv->root_symbol, "GLib");
	ValaDataType* __temp7 = NULL;
	ValaDataType* __temp6 = NULL;
	self->priv->initially_unowned_type = (__temp7 = (__temp6 = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (glib_ns, "InitiallyUnowned"))), (__temp6 == NULL ? __temp6 : g_object_ref (__temp6))), (self->priv->initially_unowned_type == NULL ? NULL : (self->priv->initially_unowned_type = (g_object_unref (self->priv->initially_unowned_type), NULL))), __temp7);
	ValaSymbol* __temp9 = NULL;
	ValaSymbol* __temp8 = NULL;
	self->priv->current_symbol = (__temp9 = (__temp8 = self->priv->root_symbol, (__temp8 == NULL ? __temp8 : g_object_ref (__temp8))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp9);
	vala_code_context_accept (context, VALA_CODE_VISITOR (self));
}


static void vala_semantic_analyzer_real_visit_begin_source_file (ValaCodeVisitor* base, ValaSourceFile* file)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_SOURCE_FILE (file));
	ValaSourceFile* __temp10 = NULL;
	self->priv->current_source_file = (__temp10 = g_object_ref (file), (self->priv->current_source_file == NULL ? NULL : (self->priv->current_source_file = (g_object_unref (self->priv->current_source_file), NULL))), __temp10);
	GList* __temp12 = NULL;
	GList* __temp11 = NULL;
	self->priv->current_using_directives = (__temp12 = (__temp11 = vala_source_file_get_using_directives (file), (__temp11 == NULL ? __temp11 : g_list_copy (__temp11))), (self->priv->current_using_directives == NULL ? NULL : (self->priv->current_using_directives = (g_list_free (self->priv->current_using_directives), NULL))), __temp12);
	self->priv->next_lambda_id = 0;
}


static void vala_semantic_analyzer_real_visit_end_source_file (ValaCodeVisitor* base, ValaSourceFile* file)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_SOURCE_FILE (file));
	GList* __temp13 = NULL;
	self->priv->current_using_directives = (__temp13 = NULL, (self->priv->current_using_directives == NULL ? NULL : (self->priv->current_using_directives = (g_list_free (self->priv->current_using_directives), NULL))), __temp13);
}


static void vala_semantic_analyzer_real_visit_begin_namespace (ValaCodeVisitor* base, ValaNamespace* ns)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_NAMESPACE (ns));
	ValaSymbol* __temp15 = NULL;
	ValaSymbol* __temp14 = NULL;
	self->priv->current_symbol = (__temp15 = (__temp14 = vala_code_node_get_symbol (VALA_CODE_NODE (ns)), (__temp14 == NULL ? __temp14 : g_object_ref (__temp14))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp15);
}


static void vala_semantic_analyzer_real_visit_end_namespace (ValaCodeVisitor* base, ValaNamespace* ns)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_NAMESPACE (ns));
	ValaSymbol* __temp17 = NULL;
	ValaSymbol* __temp16 = NULL;
	self->priv->current_symbol = (__temp17 = (__temp16 = self->priv->current_symbol->parent_symbol, (__temp16 == NULL ? __temp16 : g_object_ref (__temp16))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp17);
}


static void vala_semantic_analyzer_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	ValaSymbol* __temp19 = NULL;
	ValaSymbol* __temp18 = NULL;
	self->priv->current_symbol = (__temp19 = (__temp18 = vala_code_node_get_symbol (VALA_CODE_NODE (cl)), (__temp18 == NULL ? __temp18 : g_object_ref (__temp18))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp19);
	if (vala_class_get_base_class (cl) != NULL) {
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_class_get_base_class (cl))), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_FULL);
	}
}


static void vala_semantic_analyzer_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	ValaSymbol* __temp21 = NULL;
	ValaSymbol* __temp20 = NULL;
	self->priv->current_symbol = (__temp21 = (__temp20 = self->priv->current_symbol->parent_symbol, (__temp20 == NULL ? __temp20 : g_object_ref (__temp20))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp21);
}


static void vala_semantic_analyzer_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_STRUCT (st));
	ValaSymbol* __temp23 = NULL;
	ValaSymbol* __temp22 = NULL;
	self->priv->current_symbol = (__temp23 = (__temp22 = vala_code_node_get_symbol (VALA_CODE_NODE (st)), (__temp22 == NULL ? __temp22 : g_object_ref (__temp22))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp23);
}


static void vala_semantic_analyzer_real_visit_end_struct (ValaCodeVisitor* base, ValaStruct* st)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_STRUCT (st));
	ValaSymbol* __temp25 = NULL;
	ValaSymbol* __temp24 = NULL;
	self->priv->current_symbol = (__temp25 = (__temp24 = self->priv->current_symbol->parent_symbol, (__temp24 == NULL ? __temp24 : g_object_ref (__temp24))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp25);
}


static void vala_semantic_analyzer_real_visit_field (ValaCodeVisitor* base, ValaField* f)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_FIELD (f));
	if (f->access == VALA_MEMBER_ACCESSIBILITY_PUBLIC) {
		if (vala_field_get_type_reference (f)->type != NULL) {
			/* is null if it references a type parameter */
			vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_field_get_type_reference (f)->type)), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_SHALLOW);
		}
	} else {
		if (vala_field_get_type_reference (f)->type != NULL) {
			/* is null if it references a type parameter */
			vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_field_get_type_reference (f)->type)), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
		}
	}
}


static void vala_semantic_analyzer_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_METHOD (m));
	ValaSymbol* __temp27 = NULL;
	ValaSymbol* __temp26 = NULL;
	self->priv->current_symbol = (__temp27 = (__temp26 = vala_code_node_get_symbol (VALA_CODE_NODE (m)), (__temp26 == NULL ? __temp26 : g_object_ref (__temp26))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp27);
	ValaTypeReference* __temp28 = NULL;
	self->priv->current_return_type = (__temp28 = g_object_ref (vala_method_get_return_type (m)), (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp28);
	if (vala_method_get_return_type (m)->type != NULL) {
		/* is null if it is void or a reference to a type parameter */
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_method_get_return_type (m)->type)), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_SHALLOW);
	}
}


static void vala_semantic_analyzer_real_visit_end_method (ValaCodeVisitor* base, ValaMethod* m)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_METHOD (m));
	ValaSymbol* __temp30 = NULL;
	ValaSymbol* __temp29 = NULL;
	self->priv->current_symbol = (__temp30 = (__temp29 = self->priv->current_symbol->parent_symbol, (__temp29 == NULL ? __temp29 : g_object_ref (__temp29))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp30);
	ValaTypeReference* __temp31 = NULL;
	self->priv->current_return_type = (__temp31 = NULL, (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp31);
	if (VALA_IS_METHOD (vala_symbol_get_node (self->priv->current_symbol->parent_symbol))) {
		/* lambda expressions produce nested methods */
		ValaMethod* up_method = VALA_METHOD (vala_symbol_get_node (self->priv->current_symbol->parent_symbol));
		ValaTypeReference* __temp32 = NULL;
		self->priv->current_return_type = (__temp32 = g_object_ref (vala_method_get_return_type (up_method)), (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp32);
	}
	if (vala_method_get_is_virtual (m) || vala_method_get_overrides (m)) {
		if (VALA_IS_CLASS (vala_symbol_get_node (self->priv->current_symbol))) {
			ValaClass* cl = VALA_CLASS (vala_symbol_get_node (self->priv->current_symbol));
			ValaClass* base_class = NULL;
			for (base_class = cl; base_class != NULL; base_class = vala_class_get_base_class (base_class)) {
				ValaSymbol* sym = vala_symbol_lookup (vala_code_node_get_symbol (VALA_CODE_NODE (base_class)), vala_method_get_name (m));
				if (sym != NULL && VALA_IS_METHOD (vala_symbol_get_node (sym))) {
					ValaMethod* base_method = VALA_METHOD (vala_symbol_get_node (sym));
					if (vala_method_get_is_abstract (base_method) || vala_method_get_is_virtual (base_method)) {
						if (!vala_method_equals (m, base_method)) {
							vala_code_node_set_error (VALA_CODE_NODE (m), TRUE);
							char* __temp35 = NULL;
							char* __temp34 = NULL;
							char* __temp33 = NULL;
							vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (m)), (__temp35 = g_strdup_printf ("Return type and or parameters of overridding method `%s' do not match overridden method `%s'.", (__temp33 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))), (__temp34 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (base_method)))))));
							(__temp35 == NULL ? NULL : (__temp35 = (g_free (__temp35), NULL)));
							(__temp34 == NULL ? NULL : (__temp34 = (g_free (__temp34), NULL)));
							(__temp33 == NULL ? NULL : (__temp33 = (g_free (__temp33), NULL)));
							return;
						}
						vala_method_set_base_method (m, base_method);
						break;
					}
				}
			}
			if (vala_method_get_base_method (m) == NULL) {
				{
					GList * type_it;
					for (type_it = vala_class_get_base_types (cl); type_it != NULL; type_it = type_it->next) {
						ValaTypeReference* type = type_it->data;
						{
							/* FIXME: also look at interfaces implemented
							 * by one of the base types
							 */
							if (VALA_IS_INTERFACE (type->type)) {
								ValaInterface* iface = VALA_INTERFACE (type->type);
								ValaSymbol* sym = vala_symbol_lookup (vala_code_node_get_symbol (VALA_CODE_NODE (iface)), vala_method_get_name (m));
								if (sym != NULL && VALA_IS_METHOD (vala_symbol_get_node (sym))) {
									ValaMethod* base_method = VALA_METHOD (vala_symbol_get_node (sym));
									if (vala_method_get_is_abstract (base_method) || vala_method_get_is_virtual (base_method)) {
										vala_method_set_base_method (m, base_method);
										break;
									}
								}
							}
						}
					}
				}
			}
			if (vala_method_get_base_method (m) == NULL) {
				char* __temp37 = NULL;
				char* __temp36 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (m)), (__temp37 = g_strdup_printf ("%s: no suitable method found to override", (__temp36 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))))));
				(__temp37 == NULL ? NULL : (__temp37 = (g_free (__temp37), NULL)));
				(__temp36 == NULL ? NULL : (__temp36 = (g_free (__temp36), NULL)));
			}
		} else if (VALA_IS_STRUCT (vala_symbol_get_node (self->priv->current_symbol))) {
			char* __temp39 = NULL;
			char* __temp38 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (m)), (__temp39 = g_strdup_printf ("A struct member `%s' cannot be marked as override, virtual, or abstract", (__temp38 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))))));
			(__temp39 == NULL ? NULL : (__temp39 = (g_free (__temp39), NULL)));
			(__temp38 == NULL ? NULL : (__temp38 = (g_free (__temp38), NULL)));
			return;
		}
	}
}


static void vala_semantic_analyzer_real_visit_formal_parameter (ValaCodeVisitor* base, ValaFormalParameter* p)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_FORMAL_PARAMETER (p));
	if (!vala_formal_parameter_get_ellipsis (p)) {
		if (vala_formal_parameter_get_type_reference (p)->type != NULL) {
			/* is null if it references a type parameter */
			vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_formal_parameter_get_type_reference (p)->type)), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_SHALLOW);
			vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_formal_parameter_get_type_reference (p)->type)), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
		}
	}
}


static void vala_semantic_analyzer_real_visit_end_property (ValaCodeVisitor* base, ValaProperty* prop)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_PROPERTY (prop));
	if (vala_property_get_type_reference (prop)->type != NULL) {
		/* is null if it references a type parameter */
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_property_get_type_reference (prop)->type)), VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_SHALLOW);
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_property_get_type_reference (prop)->type)), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	}
}


static void vala_semantic_analyzer_real_visit_begin_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_PROPERTY_ACCESSOR (acc));
	ValaProperty* prop = VALA_PROPERTY (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (acc))->parent_symbol));
	if (vala_property_accessor_get_readable (acc)) {
		ValaTypeReference* __temp41 = NULL;
		ValaTypeReference* __temp40 = NULL;
		self->priv->current_return_type = (__temp41 = (__temp40 = vala_property_get_type_reference (prop), (__temp40 == NULL ? __temp40 : g_object_ref (__temp40))), (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp41);
	}
}


static void vala_semantic_analyzer_real_visit_end_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_PROPERTY_ACCESSOR (acc));
	ValaTypeReference* __temp42 = NULL;
	self->priv->current_return_type = (__temp42 = NULL, (self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL))), __temp42);
}


static void vala_semantic_analyzer_real_visit_begin_constructor (ValaCodeVisitor* base, ValaConstructor* c)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CONSTRUCTOR (c));
	ValaSymbol* __temp44 = NULL;
	ValaSymbol* __temp43 = NULL;
	self->priv->current_symbol = (__temp44 = (__temp43 = vala_code_node_get_symbol (VALA_CODE_NODE (c)), (__temp43 == NULL ? __temp43 : g_object_ref (__temp43))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp44);
}


static void vala_semantic_analyzer_real_visit_end_constructor (ValaCodeVisitor* base, ValaConstructor* c)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CONSTRUCTOR (c));
	ValaSymbol* __temp46 = NULL;
	ValaSymbol* __temp45 = NULL;
	self->priv->current_symbol = (__temp46 = (__temp45 = self->priv->current_symbol->parent_symbol, (__temp45 == NULL ? __temp45 : g_object_ref (__temp45))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp46);
}


static void vala_semantic_analyzer_real_visit_begin_destructor (ValaCodeVisitor* base, ValaDestructor* d)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_DESTRUCTOR (d));
	ValaSymbol* __temp48 = NULL;
	ValaSymbol* __temp47 = NULL;
	self->priv->current_symbol = (__temp48 = (__temp47 = vala_code_node_get_symbol (VALA_CODE_NODE (d)), (__temp47 == NULL ? __temp47 : g_object_ref (__temp47))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp48);
}


static void vala_semantic_analyzer_real_visit_end_destructor (ValaCodeVisitor* base, ValaDestructor* d)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_DESTRUCTOR (d));
	ValaSymbol* __temp50 = NULL;
	ValaSymbol* __temp49 = NULL;
	self->priv->current_symbol = (__temp50 = (__temp49 = self->priv->current_symbol->parent_symbol, (__temp49 == NULL ? __temp49 : g_object_ref (__temp49))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp50);
}


static void vala_semantic_analyzer_real_visit_named_argument (ValaCodeVisitor* base, ValaNamedArgument* n)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_NAMED_ARGUMENT (n));
}


static void vala_semantic_analyzer_real_visit_begin_block (ValaCodeVisitor* base, ValaBlock* b)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_BLOCK (b));
	ValaSymbol* __temp52 = NULL;
	ValaSymbol* __temp51 = NULL;
	self->priv->current_symbol = (__temp52 = (__temp51 = vala_code_node_get_symbol (VALA_CODE_NODE (b)), (__temp51 == NULL ? __temp51 : g_object_ref (__temp51))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp52);
}


static void vala_semantic_analyzer_real_visit_end_block (ValaCodeVisitor* base, ValaBlock* b)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_BLOCK (b));
	{
		GList * decl_it;
		for (decl_it = vala_block_get_local_variables (b); decl_it != NULL; decl_it = decl_it->next) {
			ValaVariableDeclarator* decl = decl_it->data;
			{
				vala_code_node_get_symbol (VALA_CODE_NODE (decl))->active = FALSE;
			}
		}
	}
	ValaSymbol* __temp54 = NULL;
	ValaSymbol* __temp53 = NULL;
	self->priv->current_symbol = (__temp54 = (__temp53 = self->priv->current_symbol->parent_symbol, (__temp53 == NULL ? __temp53 : g_object_ref (__temp53))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp54);
}


static void vala_semantic_analyzer_real_visit_variable_declarator (ValaCodeVisitor* base, ValaVariableDeclarator* decl)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_VARIABLE_DECLARATOR (decl));
	if (decl->type_reference == NULL) {
		/* var type */
		if (vala_variable_declarator_get_initializer (decl) == NULL) {
			vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE);
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (decl)), "var declaration not allowed without initializer");
			return;
		}
		if (vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl)) == NULL) {
			vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE);
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (decl)), "var declaration not allowed with non-typed initializer");
			return;
		}
		ValaTypeReference* __temp55 = NULL;
		decl->type_reference = (__temp55 = vala_type_reference_copy (vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl))), (decl->type_reference == NULL ? NULL : (decl->type_reference = (g_object_unref (decl->type_reference), NULL))), __temp55);
		vala_type_reference_set_is_lvalue_ref (decl->type_reference, vala_type_reference_get_is_ref (decl->type_reference));
		vala_type_reference_set_is_ref (decl->type_reference, FALSE);
	}
	if (vala_variable_declarator_get_initializer (decl) != NULL) {
		if (vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl)) == NULL) {
			if (!(VALA_IS_MEMBER_ACCESS (vala_variable_declarator_get_initializer (decl)))) {
				vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE);
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (decl)), "expression type not allowed as initializer");
				return;
			}
			if (VALA_IS_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_variable_declarator_get_initializer (decl)))) && VALA_IS_CALLBACK (decl->type_reference->type)) {
				ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_variable_declarator_get_initializer (decl))));
				ValaCallback* cb = VALA_CALLBACK (decl->type_reference->type);
				/* check whether method matches callback type */
				if (!vala_callback_matches_method (cb, m)) {
					vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE);
					char* __temp58 = NULL;
					char* __temp57 = NULL;
					char* __temp56 = NULL;
					vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (decl)), (__temp58 = g_strdup_printf ("declaration of method `%s' doesn't match declaration of callback `%s'", (__temp56 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))), (__temp57 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (cb)))))));
					(__temp58 == NULL ? NULL : (__temp58 = (g_free (__temp58), NULL)));
					(__temp57 == NULL ? NULL : (__temp57 = (g_free (__temp57), NULL)));
					(__temp56 == NULL ? NULL : (__temp56 = (g_free (__temp56), NULL)));
					return;
				}
				vala_expression_set_static_type (vala_variable_declarator_get_initializer (decl), decl->type_reference);
			} else {
				vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE);
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (decl)), "expression type not allowed as initializer");
				return;
			}
		}
		if (vala_semantic_analyzer_get_memory_management (self)) {
			if (vala_type_reference_get_is_ref (vala_expression_get_static_type (vala_variable_declarator_get_initializer (decl)))) {
				/* rhs transfers ownership of the expression */
				if (!vala_type_reference_get_is_lvalue_ref (decl->type_reference)) {
					/* lhs doesn't own the value
					 * promote lhs type */
					vala_type_reference_set_is_lvalue_ref (decl->type_reference, TRUE);
				}
			}
		}
	}
	if (decl->type_reference->type != NULL) {
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (decl->type_reference->type)), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	}
	ValaSymbol* __temp59 = NULL;
	vala_code_node_set_symbol (VALA_CODE_NODE (decl), (__temp59 = g_object_new (VALA_TYPE_SYMBOL, "node", decl, NULL)));
	(__temp59 == NULL ? NULL : (__temp59 = (g_object_unref (__temp59), NULL)));
	vala_symbol_add (self->priv->current_symbol, vala_variable_declarator_get_name (decl), vala_code_node_get_symbol (VALA_CODE_NODE (decl)));
	ValaBlock* block = VALA_BLOCK (vala_symbol_get_node (self->priv->current_symbol));
	vala_block_add_local_variable (block, decl);
	vala_code_node_get_symbol (VALA_CODE_NODE (decl))->active = TRUE;
}


static void vala_semantic_analyzer_real_visit_expression_statement (ValaCodeVisitor* base, ValaExpressionStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_EXPRESSION_STATEMENT (stmt));
	if (vala_expression_get_static_type (vala_expression_statement_get_expression (stmt)) != NULL && vala_type_reference_get_is_ref (vala_expression_get_static_type (vala_expression_statement_get_expression (stmt)))) {
		vala_report_warning (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Short-living reference");
		return;
	}
}


static void vala_semantic_analyzer_real_visit_if_statement (ValaCodeVisitor* base, ValaIfStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_IF_STATEMENT (stmt));
	if (vala_expression_get_static_type (vala_if_statement_get_condition (stmt))->type != self->priv->bool_type->type) {
		vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_if_statement_get_condition (stmt))), "Condition must be boolean");
		return;
	}
}


static void vala_semantic_analyzer_real_visit_while_statement (ValaCodeVisitor* base, ValaWhileStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_WHILE_STATEMENT (stmt));
	if (vala_expression_get_static_type (vala_while_statement_get_condition (stmt))->type != self->priv->bool_type->type) {
		vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_while_statement_get_condition (stmt))), "Condition must be boolean");
		return;
	}
}


static void vala_semantic_analyzer_real_visit_for_statement (ValaCodeVisitor* base, ValaForStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_FOR_STATEMENT (stmt));
	if (vala_expression_get_static_type (vala_for_statement_get_condition (stmt))->type != self->priv->bool_type->type) {
		vala_code_node_set_error (VALA_CODE_NODE (stmt), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_for_statement_get_condition (stmt))), "Condition must be boolean");
		return;
	}
}


static void vala_semantic_analyzer_real_visit_begin_foreach_statement (ValaCodeVisitor* base, ValaForeachStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_FOREACH_STATEMENT (stmt));
	if (vala_foreach_statement_get_type_reference (stmt)->type != NULL) {
		vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_foreach_statement_get_type_reference (stmt)->type)), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	}
	ValaVariableDeclarator* __temp60 = NULL;
	vala_foreach_statement_set_variable_declarator (stmt, (__temp60 = g_object_new (VALA_TYPE_VARIABLE_DECLARATOR, "name", vala_foreach_statement_get_variable_name (stmt), NULL)));
	(__temp60 == NULL ? NULL : (__temp60 = (g_object_unref (__temp60), NULL)));
	ValaTypeReference* __temp61 = NULL;
	vala_foreach_statement_get_variable_declarator (stmt)->type_reference = (__temp61 = g_object_ref (vala_foreach_statement_get_type_reference (stmt)), (vala_foreach_statement_get_variable_declarator (stmt)->type_reference == NULL ? NULL : (vala_foreach_statement_get_variable_declarator (stmt)->type_reference = (g_object_unref (vala_foreach_statement_get_variable_declarator (stmt)->type_reference), NULL))), __temp61);
	ValaSymbol* __temp62 = NULL;
	vala_code_node_set_symbol (VALA_CODE_NODE (vala_foreach_statement_get_variable_declarator (stmt)), (__temp62 = g_object_new (VALA_TYPE_SYMBOL, "node", vala_foreach_statement_get_variable_declarator (stmt), NULL)));
	(__temp62 == NULL ? NULL : (__temp62 = (g_object_unref (__temp62), NULL)));
	vala_symbol_add (self->priv->current_symbol, vala_foreach_statement_get_variable_name (stmt), vala_code_node_get_symbol (VALA_CODE_NODE (vala_foreach_statement_get_variable_declarator (stmt))));
}


static void vala_semantic_analyzer_real_visit_return_statement (ValaCodeVisitor* base, ValaReturnStatement* stmt)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_RETURN_STATEMENT (stmt));
	if (self->priv->current_return_type == NULL) {
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Return not allowed in this context");
		return;
	}
	if (vala_return_statement_get_return_expression (stmt) == NULL && self->priv->current_return_type->type != NULL) {
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Return with value in void function");
		return;
	}
	if (vala_return_statement_get_return_expression (stmt) != NULL && self->priv->current_return_type->type == NULL) {
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), "Return without value in non-void function");
		return;
	}
	if (vala_return_statement_get_return_expression (stmt) != NULL && !vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_return_statement_get_return_expression (stmt)), self->priv->current_return_type)) {
		char* __temp65 = NULL;
		char* __temp64 = NULL;
		char* __temp63 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (stmt)), (__temp65 = g_strdup_printf ("Return: Cannot convert from `%s' to `%s'", (__temp63 = vala_type_reference_to_string (vala_expression_get_static_type (vala_return_statement_get_return_expression (stmt)))), (__temp64 = vala_type_reference_to_string (self->priv->current_return_type)))));
		(__temp65 == NULL ? NULL : (__temp65 = (g_free (__temp65), NULL)));
		(__temp64 == NULL ? NULL : (__temp64 = (g_free (__temp64), NULL)));
		(__temp63 == NULL ? NULL : (__temp63 = (g_free (__temp63), NULL)));
		return;
	}
}


static void vala_semantic_analyzer_real_visit_boolean_literal (ValaCodeVisitor* base, ValaBooleanLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_BOOLEAN_LITERAL (expr));
	vala_literal_set_static_type (VALA_LITERAL (expr), self->priv->bool_type);
}


static void vala_semantic_analyzer_real_visit_character_literal (ValaCodeVisitor* base, ValaCharacterLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CHARACTER_LITERAL (expr));
	ValaTypeReference* __temp66 = NULL;
	vala_literal_set_static_type (VALA_LITERAL (expr), (__temp66 = g_object_new (VALA_TYPE_TYPE_REFERENCE, NULL)));
	(__temp66 == NULL ? NULL : (__temp66 = (g_object_unref (__temp66), NULL)));
	vala_literal_get_static_type (VALA_LITERAL (expr))->type = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "char")));
}


static void vala_semantic_analyzer_real_visit_integer_literal (ValaCodeVisitor* base, ValaIntegerLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_INTEGER_LITERAL (expr));
	ValaTypeReference* __temp67 = NULL;
	vala_literal_set_static_type (VALA_LITERAL (expr), (__temp67 = g_object_new (VALA_TYPE_TYPE_REFERENCE, NULL)));
	(__temp67 == NULL ? NULL : (__temp67 = (g_object_unref (__temp67), NULL)));
	vala_literal_get_static_type (VALA_LITERAL (expr))->type = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "int")));
}


static void vala_semantic_analyzer_real_visit_real_literal (ValaCodeVisitor* base, ValaRealLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_REAL_LITERAL (expr));
	ValaTypeReference* __temp68 = NULL;
	vala_literal_set_static_type (VALA_LITERAL (expr), (__temp68 = g_object_new (VALA_TYPE_TYPE_REFERENCE, NULL)));
	(__temp68 == NULL ? NULL : (__temp68 = (g_object_unref (__temp68), NULL)));
	vala_literal_get_static_type (VALA_LITERAL (expr))->type = VALA_DATA_TYPE (vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "double")));
}


static void vala_semantic_analyzer_real_visit_string_literal (ValaCodeVisitor* base, ValaStringLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_STRING_LITERAL (expr));
	ValaTypeReference* __temp69 = NULL;
	vala_literal_set_static_type (VALA_LITERAL (expr), (__temp69 = vala_type_reference_copy (self->priv->string_type)));
	(__temp69 == NULL ? NULL : (__temp69 = (g_object_unref (__temp69), NULL)));
	vala_type_reference_set_non_null (vala_literal_get_static_type (VALA_LITERAL (expr)), TRUE);
}


static void vala_semantic_analyzer_real_visit_null_literal (ValaCodeVisitor* base, ValaNullLiteral* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_NULL_LITERAL (expr));
	/* empty TypeReference represents null */
	ValaTypeReference* __temp70 = NULL;
	vala_literal_set_static_type (VALA_LITERAL (expr), (__temp70 = g_object_new (VALA_TYPE_TYPE_REFERENCE, NULL)));
	(__temp70 == NULL ? NULL : (__temp70 = (g_object_unref (__temp70), NULL)));
}


static void vala_semantic_analyzer_real_visit_literal_expression (ValaCodeVisitor* base, ValaLiteralExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_LITERAL_EXPRESSION (expr));
	vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_literal_get_static_type (vala_literal_expression_get_literal (expr)));
}


static ValaTypeReference* vala_semantic_analyzer_get_static_type_for_node (ValaSemanticAnalyzer* self, ValaCodeNode* node)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	g_return_val_if_fail (VALA_IS_CODE_NODE (node), NULL);
	if (VALA_IS_FIELD (node)) {
		ValaField* f = VALA_FIELD (node);
		return g_object_ref (vala_field_get_type_reference (f));
	} else if (VALA_IS_CONSTANT (node)) {
		ValaConstant* c = VALA_CONSTANT (node);
		return g_object_ref (vala_constant_get_type_reference (c));
	} else if (VALA_IS_PROPERTY (node)) {
		ValaProperty* prop = VALA_PROPERTY (node);
		ValaTypeReference* type = vala_type_reference_copy (vala_property_get_type_reference (prop));
		vala_type_reference_set_is_lvalue_ref (type, FALSE);
		ValaTypeReference* __temp73 = NULL;
		ValaTypeReference* __temp74 = NULL;
		return (__temp74 = (__temp73 = type, (__temp73 == NULL ? __temp73 : g_object_ref (__temp73))), (type == NULL ? NULL : (type = (g_object_unref (type), NULL))), __temp74);
		(type == NULL ? NULL : (type = (g_object_unref (type), NULL)));
	} else if (VALA_IS_FORMAL_PARAMETER (node)) {
		ValaFormalParameter* p = VALA_FORMAL_PARAMETER (node);
		ValaTypeReference* __temp75 = NULL;
		return (__temp75 = vala_formal_parameter_get_type_reference (p), (__temp75 == NULL ? __temp75 : g_object_ref (__temp75)));
	} else if (VALA_IS_TYPE_REFERENCE (node)) {
		ValaTypeReference* __temp77 = NULL;
		return (__temp77 = VALA_TYPE_REFERENCE (node), (__temp77 == NULL ? __temp77 : g_object_ref (__temp77)));
	} else if (VALA_IS_VARIABLE_DECLARATOR (node)) {
		ValaVariableDeclarator* decl = VALA_VARIABLE_DECLARATOR (node);
		ValaTypeReference* __temp79 = NULL;
		return (__temp79 = decl->type_reference, (__temp79 == NULL ? __temp79 : g_object_ref (__temp79)));
	} else if (VALA_IS_ENUM_VALUE (node)) {
		ValaTypeReference* type = g_object_new (VALA_TYPE_TYPE_REFERENCE, NULL);
		type->type = VALA_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (node)->parent_symbol));
		ValaTypeReference* __temp81 = NULL;
		ValaTypeReference* __temp82 = NULL;
		return (__temp82 = (__temp81 = type, (__temp81 == NULL ? __temp81 : g_object_ref (__temp81))), (type == NULL ? NULL : (type = (g_object_unref (type), NULL))), __temp82);
		(type == NULL ? NULL : (type = (g_object_unref (type), NULL)));
	}
	return NULL;
}


static ValaSymbol* vala_semantic_analyzer_symbol_lookup_inherited (ValaSemanticAnalyzer* self, ValaSymbol* sym, char* name)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	g_return_val_if_fail (VALA_IS_SYMBOL (sym), NULL);
	g_return_val_if_fail (name != NULL, NULL);
	ValaSymbol* result = vala_symbol_lookup (sym, name);
	if (result == NULL && VALA_IS_CLASS (vala_symbol_get_node (sym))) {
		ValaClass* cl = VALA_CLASS (vala_symbol_get_node (sym));
		for (cl = vala_class_get_base_class (cl); cl != NULL && result == NULL; cl = vala_class_get_base_class (cl)) {
			result = vala_symbol_lookup (vala_code_node_get_symbol (VALA_CODE_NODE (cl)), name);
		}
	}
	return result;
}


static void vala_semantic_analyzer_real_visit_parenthesized_expression (ValaCodeVisitor* base, ValaParenthesizedExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_PARENTHESIZED_EXPRESSION (expr));
	vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_parenthesized_expression_get_inner (expr)));
}


static ValaDataType* vala_semantic_analyzer_find_parent_type (ValaSemanticAnalyzer* self, ValaSymbol* sym)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	g_return_val_if_fail (sym == NULL || VALA_IS_SYMBOL (sym), NULL);
	while (sym != NULL) {
		if (VALA_IS_DATA_TYPE (vala_symbol_get_node (sym))) {
			return VALA_DATA_TYPE (vala_symbol_get_node (sym));
		}
		sym = sym->parent_symbol;
	}
	return NULL;
}


static void vala_semantic_analyzer_real_visit_member_access (ValaCodeVisitor* base, ValaMemberAccess* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_MEMBER_ACCESS (expr));
	ValaSymbol* base_symbol = NULL;
	if (vala_member_access_get_inner (expr) == NULL) {
		base_symbol = self->priv->current_symbol;
		ValaSymbol* sym = self->priv->current_symbol;
		while (sym != NULL && vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL) {
			vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), vala_semantic_analyzer_symbol_lookup_inherited (self, sym, vala_member_access_get_member_name (expr)));
			sym = sym->parent_symbol;
		}
		if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL) {
			{
				GList * ns_it;
				for (ns_it = self->priv->current_using_directives; ns_it != NULL; ns_it = ns_it->next) {
					ValaNamespaceReference* ns = ns_it->data;
					{
						ValaSymbol* local_sym = vala_symbol_lookup (ns->namespace_symbol, vala_member_access_get_member_name (expr));
						if (local_sym != NULL) {
							if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) != NULL) {
								char* __temp89 = NULL;
								char* __temp88 = NULL;
								char* __temp87 = NULL;
								vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp89 = g_strdup_printf ("`%s' is an ambiguous reference between `%s' and `%s'", vala_member_access_get_member_name (expr), (__temp87 = vala_symbol_get_full_name (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)))), (__temp88 = vala_symbol_get_full_name (local_sym)))));
								(__temp89 == NULL ? NULL : (__temp89 = (g_free (__temp89), NULL)));
								(__temp88 == NULL ? NULL : (__temp88 = (g_free (__temp88), NULL)));
								(__temp87 == NULL ? NULL : (__temp87 = (g_free (__temp87), NULL)));
								return;
							}
							vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), local_sym);
						}
					}
				}
			}
		}
	} else {
		if (vala_code_node_get_error (VALA_CODE_NODE (vala_member_access_get_inner (expr)))) {
			return;
		}
		/* if there was an error in the inner expression, skip this check */
		if (VALA_IS_MEMBER_ACCESS (vala_member_access_get_inner (expr))) {
			ValaMemberAccess* base = VALA_MEMBER_ACCESS (vala_member_access_get_inner (expr));
			base_symbol = vala_expression_get_symbol_reference (VALA_EXPRESSION (base));
			if (VALA_IS_NAMESPACE (vala_symbol_get_node (base_symbol)) || VALA_IS_DATA_TYPE (vala_symbol_get_node (base_symbol))) {
				vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), vala_symbol_lookup (base_symbol, vala_member_access_get_member_name (expr)));
			}
		}
		if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL && vala_expression_get_static_type (vala_member_access_get_inner (expr)) != NULL) {
			base_symbol = vala_code_node_get_symbol (VALA_CODE_NODE (vala_expression_get_static_type (vala_member_access_get_inner (expr))->type));
			vala_expression_set_symbol_reference (VALA_EXPRESSION (expr), vala_semantic_analyzer_symbol_lookup_inherited (self, base_symbol, vala_member_access_get_member_name (expr)));
		}
	}
	if (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)) == NULL) {
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		char* __temp91 = NULL;
		char* __temp90 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp91 = g_strdup_printf ("The name `%s' does not exist in the context of `%s'", vala_member_access_get_member_name (expr), (__temp90 = vala_symbol_get_full_name (base_symbol)))));
		(__temp91 == NULL ? NULL : (__temp91 = (g_free (__temp91), NULL)));
		(__temp90 == NULL ? NULL : (__temp90 = (g_free (__temp90), NULL)));
		return;
	}
	ValaCodeNode* member = vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)));
	ValaMemberAccessibility access = VALA_MEMBER_ACCESSIBILITY_PUBLIC;
	if (VALA_IS_FIELD (member)) {
		access = (VALA_FIELD (member))->access;
	} else if (VALA_IS_METHOD (member)) {
		access = (VALA_METHOD (member))->access;
	}
	if (access != VALA_MEMBER_ACCESSIBILITY_PUBLIC) {
		ValaDataType* target_type = VALA_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (member)->parent_symbol));
		ValaDataType* this_type = vala_semantic_analyzer_find_parent_type (self, self->priv->current_symbol);
		if (target_type != this_type) {
			vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
			char* __temp93 = NULL;
			char* __temp92 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp93 = g_strdup_printf ("Access to private member `%s' denied", (__temp92 = vala_symbol_get_full_name (vala_code_node_get_symbol (member))))));
			(__temp93 == NULL ? NULL : (__temp93 = (g_free (__temp93), NULL)));
			(__temp92 == NULL ? NULL : (__temp92 = (g_free (__temp92), NULL)));
			return;
		}
	}
	vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_expression_get_symbol_reference (VALA_EXPRESSION (expr)), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	ValaTypeReference* __temp94 = NULL;
	vala_expression_set_static_type (VALA_EXPRESSION (expr), (__temp94 = vala_semantic_analyzer_get_static_type_for_node (self, vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (expr))))));
	(__temp94 == NULL ? NULL : (__temp94 = (g_object_unref (__temp94), NULL)));
}


static gboolean vala_semantic_analyzer_is_type_compatible (ValaSemanticAnalyzer* self, ValaTypeReference* expression_type, ValaTypeReference* expected_type)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (expression_type), FALSE);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (expected_type), FALSE);
	/* only null is compatible to null */
	if (expected_type->type == NULL && expected_type->type_parameter == NULL) {
		return (expression_type->type == NULL && expected_type->type_parameter == NULL);
	}
	/* null can be casted to any reference or array type */
	if (expression_type->type == NULL && (expected_type->type_parameter != NULL || vala_data_type_is_reference_type (expected_type->type) || vala_type_reference_get_is_ref (expected_type) || VALA_IS_ARRAY (expected_type->type) || VALA_IS_CALLBACK (expected_type->type) || expected_type->type == self->priv->pointer_type)) {
		return TRUE;
	}
	/* temporarily ignore type parameters */
	if (expected_type->type_parameter != NULL) {
		return TRUE;
	}
	if (VALA_IS_ARRAY (expression_type->type) != VALA_IS_ARRAY (expected_type->type)) {
		return FALSE;
	}
	if (expression_type->type == expected_type->type) {
		return TRUE;
	}
	/* int may be implicitly casted to long */
	if (expression_type->type == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "int")) && expected_type->type == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "long"))) {
		return TRUE;
	}
	/* int may be implicitly casted to ulong */
	if (expression_type->type == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "int")) && expected_type->type == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "ulong"))) {
		return TRUE;
	}
	/* uint may be implicitly casted to long */
	if (expression_type->type == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "uint")) && expected_type->type == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "long"))) {
		return TRUE;
	}
	/* uint may be implicitly casted to ulong */
	if (expression_type->type == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "uint")) && expected_type->type == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "ulong"))) {
		return TRUE;
	}
	/* int may be implicitly casted to uint */
	if (expression_type->type == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "int")) && expected_type->type == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "uint"))) {
		return TRUE;
	}
	/* uint may be implicitly casted to int */
	if (expression_type->type == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "uint")) && expected_type->type == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "int"))) {
		return TRUE;
	}
	/* long may be implicitly casted to ulong */
	if (expression_type->type == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "long")) && expected_type->type == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "ulong"))) {
		return TRUE;
	}
	/* ulong may be implicitly casted to long */
	if (expression_type->type == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "ulong")) && expected_type->type == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "long"))) {
		return TRUE;
	}
	/* int may be implicitly casted to double */
	if (expression_type->type == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "int")) && expected_type->type == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "double"))) {
		return TRUE;
	}
	/* char may be implicitly casted to unichar */
	if (expression_type->type == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "char")) && expected_type->type == vala_symbol_get_node (vala_symbol_lookup (self->priv->root_symbol, "unichar"))) {
		return TRUE;
	}
	/* non-class types must match exactly */
	if (!(VALA_IS_CLASS (expression_type->type))) {
		return FALSE;
	}
	ValaClass* cl = VALA_CLASS (expression_type->type);
	ValaClass* base_class = vala_class_get_base_class (cl);
	for (; base_class != NULL; base_class = vala_class_get_base_class (base_class)) {
		if (base_class == expected_type->type) {
			return TRUE;
		}
	}
	return FALSE;
}


static void vala_semantic_analyzer_real_visit_begin_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_INVOCATION_EXPRESSION (expr));
	if (vala_code_node_get_error (VALA_CODE_NODE (vala_invocation_expression_get_call (expr)))) {
		/* if method resolving didn't succeed, skip this check */
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		return;
	}
	ValaSymbol* msym = vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr));
	GList* params = NULL;
	if (VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (msym))) {
		ValaVariableDeclarator* decl = VALA_VARIABLE_DECLARATOR (vala_symbol_get_node (msym));
		if (VALA_IS_CALLBACK (decl->type_reference->type)) {
			ValaCallback* cb = VALA_CALLBACK (decl->type_reference->type);
			GList* __temp113 = NULL;
			params = (__temp113 = vala_callback_get_parameters (cb), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp113);
		} else {
			vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "invocation not supported in this context");
			(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
			return;
		}
	} else if (VALA_IS_FORMAL_PARAMETER (vala_symbol_get_node (msym))) {
		ValaFormalParameter* param = VALA_FORMAL_PARAMETER (vala_symbol_get_node (msym));
		if (VALA_IS_CALLBACK (vala_formal_parameter_get_type_reference (param)->type)) {
			ValaCallback* cb = VALA_CALLBACK (vala_formal_parameter_get_type_reference (param)->type);
			GList* __temp114 = NULL;
			params = (__temp114 = vala_callback_get_parameters (cb), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp114);
		} else {
			vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "invocation not supported in this context");
			(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
			return;
		}
	} else if (VALA_IS_METHOD (vala_symbol_get_node (msym))) {
		ValaMethod* m = VALA_METHOD (vala_symbol_get_node (msym));
		GList* __temp115 = NULL;
		params = (__temp115 = vala_method_get_parameters (m), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp115);
	} else if (VALA_IS_SIGNAL (vala_symbol_get_node (msym))) {
		ValaSignal* sig = VALA_SIGNAL (vala_symbol_get_node (msym));
		GList* __temp116 = NULL;
		params = (__temp116 = vala_signal_get_parameters (sig), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp116);
	} else {
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "invocation not supported in this context");
		(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
		return;
	}
	GList* args = vala_invocation_expression_get_argument_list (expr);
	GList* arg_it = args;
	{
		GList * param_it;
		for (param_it = params; param_it != NULL; param_it = param_it->next) {
			ValaFormalParameter* param = param_it->data;
			{
				if (vala_formal_parameter_get_ellipsis (param)) {
					break;
				}
				if (arg_it != NULL) {
					ValaExpression* arg = VALA_EXPRESSION (arg_it->data);
					/* store expected type for callback parameters */
					vala_expression_set_expected_type (arg, vala_formal_parameter_get_type_reference (param));
					arg_it = arg_it->next;
				}
			}
		}
	}
	(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
	(args == NULL ? NULL : (args = (g_list_free (args), NULL)));
}


static void vala_semantic_analyzer_real_visit_end_invocation_expression (ValaCodeVisitor* base, ValaInvocationExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_INVOCATION_EXPRESSION (expr));
	if (vala_code_node_get_error (VALA_CODE_NODE (expr))) {
		return;
	}
	ValaSymbol* msym = vala_expression_get_symbol_reference (vala_invocation_expression_get_call (expr));
	ValaTypeReference* ret_type = NULL;
	GList* params = NULL;
	if (VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (msym))) {
		ValaVariableDeclarator* decl = VALA_VARIABLE_DECLARATOR (vala_symbol_get_node (msym));
		ValaCallback* cb = VALA_CALLBACK (decl->type_reference->type);
		ret_type = vala_callback_get_return_type (cb);
		GList* __temp117 = NULL;
		params = (__temp117 = vala_callback_get_parameters (cb), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp117);
	} else if (VALA_IS_FORMAL_PARAMETER (vala_symbol_get_node (msym))) {
		ValaFormalParameter* param = VALA_FORMAL_PARAMETER (vala_symbol_get_node (msym));
		ValaCallback* cb = VALA_CALLBACK (vala_formal_parameter_get_type_reference (param)->type);
		ret_type = vala_callback_get_return_type (cb);
		GList* __temp118 = NULL;
		params = (__temp118 = vala_callback_get_parameters (cb), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp118);
	} else if (VALA_IS_METHOD (vala_symbol_get_node (msym))) {
		ValaMethod* m = VALA_METHOD (vala_symbol_get_node (msym));
		ret_type = vala_method_get_return_type (m);
		GList* __temp119 = NULL;
		params = (__temp119 = vala_method_get_parameters (m), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp119);
	} else if (VALA_IS_SIGNAL (vala_symbol_get_node (msym))) {
		ValaSignal* sig = VALA_SIGNAL (vala_symbol_get_node (msym));
		ret_type = vala_signal_get_return_type (sig);
		GList* __temp120 = NULL;
		params = (__temp120 = vala_signal_get_parameters (sig), (params == NULL ? NULL : (params = (g_list_free (params), NULL))), __temp120);
	}
	vala_expression_set_static_type (VALA_EXPRESSION (expr), ret_type);
	GList* args = vala_invocation_expression_get_argument_list (expr);
	GList* arg_it = args;
	gboolean ellipsis = FALSE;
	int i = 0;
	{
		GList * param_it;
		for (param_it = params; param_it != NULL; param_it = param_it->next) {
			ValaFormalParameter* param = param_it->data;
			{
				if (vala_formal_parameter_get_ellipsis (param)) {
					ellipsis = TRUE;
					break;
				}
				/* header file necessary if we need to cast argument */
				if (vala_formal_parameter_get_type_reference (param)->type != NULL) {
					vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_formal_parameter_get_type_reference (param)->type)), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
				}
				if (arg_it == NULL) {
					if (vala_formal_parameter_get_default_expression (param) == NULL) {
						char* __temp122 = NULL;
						char* __temp121 = NULL;
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp122 = g_strdup_printf ("Too few arguments, method `%s' does not take %d arguments", (__temp121 = vala_symbol_get_full_name (msym)), g_list_length (args))));
						(__temp122 == NULL ? NULL : (__temp122 = (g_free (__temp122), NULL)));
						(__temp121 == NULL ? NULL : (__temp121 = (g_free (__temp121), NULL)));
						(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
						(args == NULL ? NULL : (args = (g_list_free (args), NULL)));
						return;
					}
				} else {
					ValaExpression* arg = VALA_EXPRESSION (arg_it->data);
					if (vala_expression_get_static_type (arg) != NULL && !vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (arg), vala_formal_parameter_get_type_reference (param))) {
						/* if there was an error in the argument,
						 * i.e. arg.static_type == null, skip type check */
						char* __temp125 = NULL;
						char* __temp124 = NULL;
						char* __temp123 = NULL;
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp125 = g_strdup_printf ("Argument %d: Cannot convert from `%s' to `%s'", i + 1, (__temp123 = vala_type_reference_to_string (vala_expression_get_static_type (arg))), (__temp124 = vala_type_reference_to_string (vala_formal_parameter_get_type_reference (param))))));
						(__temp125 == NULL ? NULL : (__temp125 = (g_free (__temp125), NULL)));
						(__temp124 == NULL ? NULL : (__temp124 = (g_free (__temp124), NULL)));
						(__temp123 == NULL ? NULL : (__temp123 = (g_free (__temp123), NULL)));
						(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
						(args == NULL ? NULL : (args = (g_list_free (args), NULL)));
						return;
					}
					arg_it = arg_it->next;
					i++;
				}
			}
		}
	}
	if (!ellipsis && arg_it != NULL) {
		char* __temp127 = NULL;
		char* __temp126 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp127 = g_strdup_printf ("Too many arguments, method `%s' does not take %d arguments", (__temp126 = vala_symbol_get_full_name (msym)), g_list_length (args))));
		(__temp127 == NULL ? NULL : (__temp127 = (g_free (__temp127), NULL)));
		(__temp126 == NULL ? NULL : (__temp126 = (g_free (__temp126), NULL)));
		(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
		(args == NULL ? NULL : (args = (g_list_free (args), NULL)));
		return;
	}
	(params == NULL ? NULL : (params = (g_list_free (params), NULL)));
	(args == NULL ? NULL : (args = (g_list_free (args), NULL)));
}


static void vala_semantic_analyzer_real_visit_element_access (ValaCodeVisitor* base, ValaElementAccess* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_ELEMENT_ACCESS (expr));
	if (vala_expression_get_static_type (vala_element_access_get_container (expr)) == NULL || vala_expression_get_static_type (vala_element_access_get_index (expr)) == NULL) {
		return;
	}
	/* don't proceed if a child expression failed 
	 assign a static_type when possible */
	if (VALA_IS_ARRAY (vala_expression_get_static_type (vala_element_access_get_container (expr))->type)) {
		ValaTypeReference* __temp128 = NULL;
		vala_expression_set_static_type (VALA_EXPRESSION (expr), (__temp128 = g_object_new (VALA_TYPE_TYPE_REFERENCE, NULL)));
		(__temp128 == NULL ? NULL : (__temp128 = (g_object_unref (__temp128), NULL)));
		vala_expression_get_static_type (VALA_EXPRESSION (expr))->type = vala_array_get_element_type ((VALA_ARRAY (vala_expression_get_static_type (vala_element_access_get_container (expr))->type)));
	} else {
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		char* __temp130 = NULL;
		char* __temp129 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp130 = g_strdup_printf ("The expression `%s' does not denote an Array", (__temp129 = vala_type_reference_to_string (vala_expression_get_static_type (vala_element_access_get_container (expr)))))));
		(__temp130 == NULL ? NULL : (__temp130 = (g_free (__temp130), NULL)));
		(__temp129 == NULL ? NULL : (__temp129 = (g_free (__temp129), NULL)));
	}
	/* check if the index is of type integer */
	if (vala_expression_get_static_type (vala_element_access_get_index (expr))->type != self->priv->int_type->type) {
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		char* __temp132 = NULL;
		char* __temp131 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp132 = g_strdup_printf ("The expression `%s' does not denote an `int' which is needed for element access", (__temp131 = vala_type_reference_to_string (vala_expression_get_static_type (vala_element_access_get_index (expr)))))));
		(__temp132 == NULL ? NULL : (__temp132 = (g_free (__temp132), NULL)));
		(__temp131 == NULL ? NULL : (__temp131 = (g_free (__temp131), NULL)));
	}
}


static void vala_semantic_analyzer_real_visit_postfix_expression (ValaCodeVisitor* base, ValaPostfixExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_POSTFIX_EXPRESSION (expr));
	vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_postfix_expression_get_inner (expr)));
}


static void vala_semantic_analyzer_real_visit_object_creation_expression (ValaCodeVisitor* base, ValaObjectCreationExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_OBJECT_CREATION_EXPRESSION (expr));
	if (vala_object_creation_expression_get_type_reference (expr)->type == NULL) {
		return;
	}
	/* if type resolving didn't succeed, skip this check */
	vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_object_creation_expression_get_type_reference (expr)->type)), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	ValaTypeReference* __temp133 = NULL;
	vala_expression_set_static_type (VALA_EXPRESSION (expr), (__temp133 = vala_type_reference_copy (vala_object_creation_expression_get_type_reference (expr))));
	(__temp133 == NULL ? NULL : (__temp133 = (g_object_unref (__temp133), NULL)));
	vala_type_reference_set_is_ref (vala_expression_get_static_type (VALA_EXPRESSION (expr)), TRUE);
	ValaClass* cl = VALA_CLASS (vala_object_creation_expression_get_type_reference (expr)->type);
	while (cl != NULL) {
		if (cl == self->priv->initially_unowned_type) {
			vala_type_reference_set_floating_reference (vala_expression_get_static_type (VALA_EXPRESSION (expr)), TRUE);
			break;
		}
		cl = vala_class_get_base_class (cl);
	}
}


static void vala_semantic_analyzer_real_visit_unary_expression (ValaCodeVisitor* base, ValaUnaryExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_UNARY_EXPRESSION (expr));
	if (vala_expression_get_static_type (vala_unary_expression_get_inner (expr)) == NULL) {
		return;
	}
	/* if there was an error in the inner expression, skip type check */
	if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_PLUS || vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_MINUS) {
		/* integer or floating point type*/
		vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr)));
	} else if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_LOGICAL_NEGATION) {
		/* boolean type*/
		vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr)));
	} else if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_BITWISE_COMPLEMENT) {
		/* integer type*/
		vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr)));
	} else if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_REF) {
		/* value type*/
		vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr)));
	} else if (vala_unary_expression_get_operator (expr) == VALA_UNARY_OPERATOR_OUT) {
		/* reference type*/
		vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_unary_expression_get_inner (expr)));
	} else {
		g_assert_not_reached ();
	}
}


static void vala_semantic_analyzer_real_visit_cast_expression (ValaCodeVisitor* base, ValaCastExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CAST_EXPRESSION (expr));
	if (vala_cast_expression_get_type_reference (expr)->type == NULL) {
		return;
	}
	/* if type resolving didn't succeed, skip this check */
	vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_cast_expression_get_type_reference (expr)->type)), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_cast_expression_get_type_reference (expr));
}


static gboolean vala_semantic_analyzer_check_binary_type (ValaSemanticAnalyzer* self, ValaBinaryExpression* expr, char* operation)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	g_return_val_if_fail (VALA_IS_BINARY_EXPRESSION (expr), FALSE);
	g_return_val_if_fail (operation != NULL, FALSE);
	if (!vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_binary_expression_get_right (expr)), vala_expression_get_static_type (vala_binary_expression_get_left (expr)))) {
		char* __temp136 = NULL;
		char* __temp135 = NULL;
		char* __temp134 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp136 = g_strdup_printf ("%s: Cannot convert from `%s' to `%s'", operation, (__temp134 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_right (expr)))), (__temp135 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_left (expr)))))));
		(__temp136 == NULL ? NULL : (__temp136 = (g_free (__temp136), NULL)));
		(__temp135 == NULL ? NULL : (__temp135 = (g_free (__temp135), NULL)));
		(__temp134 == NULL ? NULL : (__temp134 = (g_free (__temp134), NULL)));
		return FALSE;
	}
	return TRUE;
}


static void vala_semantic_analyzer_real_visit_binary_expression (ValaCodeVisitor* base, ValaBinaryExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_BINARY_EXPRESSION (expr));
	if (vala_expression_get_static_type (vala_binary_expression_get_left (expr)) == NULL || vala_expression_get_static_type (vala_binary_expression_get_right (expr)) == NULL) {
		return;
	}
	/* if there were any errors in inner expressions, skip type check */
	if (vala_expression_get_static_type (vala_binary_expression_get_left (expr))->type == self->priv->string_type->type && vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_PLUS) {
		if (vala_expression_get_static_type (vala_binary_expression_get_right (expr))->type != self->priv->string_type->type) {
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Operands must be strings");
		}
		vala_expression_set_static_type (VALA_EXPRESSION (expr), self->priv->string_type);
	} else if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_PLUS || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_MINUS || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_MUL || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_DIV) {
		/* TODO: check for integer or floating point type in expr.left*/
		if (!vala_semantic_analyzer_check_binary_type (self, expr, "Arithmetic operation")) {
			return;
		}
		vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_binary_expression_get_left (expr)));
	} else if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_MOD || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_SHIFT_LEFT || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_SHIFT_RIGHT || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_BITWISE_XOR) {
		/* TODO: check for integer type in expr.left*/
		if (!vala_semantic_analyzer_check_binary_type (self, expr, "Arithmetic operation")) {
			return;
		}
		vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_binary_expression_get_left (expr)));
	} else if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_LESS_THAN || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_GREATER_THAN || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_LESS_THAN_OR_EQUAL || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_GREATER_THAN_OR_EQUAL) {
		if (vala_expression_get_static_type (vala_binary_expression_get_left (expr))->type == self->priv->string_type->type && vala_expression_get_static_type (vala_binary_expression_get_right (expr))->type == self->priv->string_type->type) {
			/* string comparison: convert to a.collate (b) OP 0 */
			ValaMemberAccess* __temp139 = NULL;
			ValaInvocationExpression* __temp140 = NULL;
			ValaInvocationExpression* cmp_call = (__temp140 = g_object_new (VALA_TYPE_INVOCATION_EXPRESSION, "call", (__temp139 = g_object_new (VALA_TYPE_MEMBER_ACCESS, "inner", vala_binary_expression_get_left (expr), "member_name", "collate", NULL)), NULL), (__temp139 == NULL ? NULL : (__temp139 = (g_object_unref (__temp139), NULL))), __temp140);
			vala_invocation_expression_add_argument (cmp_call, vala_binary_expression_get_right (expr));
			vala_binary_expression_set_left (expr, VALA_EXPRESSION (cmp_call));
			ValaLiteralExpression* __temp142 = NULL;
			ValaIntegerLiteral* __temp141 = NULL;
			vala_binary_expression_set_right (expr, VALA_EXPRESSION ((__temp142 = g_object_new (VALA_TYPE_LITERAL_EXPRESSION, "literal", (__temp141 = g_object_new (VALA_TYPE_INTEGER_LITERAL, "value", "0", NULL)), NULL))));
			(__temp142 == NULL ? NULL : (__temp142 = (g_object_unref (__temp142), NULL)));
			(__temp141 == NULL ? NULL : (__temp141 = (g_object_unref (__temp141), NULL)));
			vala_code_node_accept (VALA_CODE_NODE (vala_binary_expression_get_left (expr)), VALA_CODE_VISITOR (self));
			(cmp_call == NULL ? NULL : (cmp_call = (g_object_unref (cmp_call), NULL)));
		} else {
			/* TODO: check for integer or floating point type in expr.left */
			if (!vala_semantic_analyzer_check_binary_type (self, expr, "Relational operation")) {
				return;
			}
		}
		vala_expression_set_static_type (VALA_EXPRESSION (expr), self->priv->bool_type);
	} else if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_EQUALITY || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_INEQUALITY) {
		/* relational operation */
		if (!vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_binary_expression_get_right (expr)), vala_expression_get_static_type (vala_binary_expression_get_left (expr))) && !vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_binary_expression_get_left (expr)), vala_expression_get_static_type (vala_binary_expression_get_right (expr)))) {
			char* __temp145 = NULL;
			char* __temp144 = NULL;
			char* __temp143 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), (__temp145 = g_strdup_printf ("Equality operation: `%s' and `%s' are incompatible, comparison would always evaluate to false", (__temp143 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_right (expr)))), (__temp144 = vala_type_reference_to_string (vala_expression_get_static_type (vala_binary_expression_get_left (expr)))))));
			(__temp145 == NULL ? NULL : (__temp145 = (g_free (__temp145), NULL)));
			(__temp144 == NULL ? NULL : (__temp144 = (g_free (__temp144), NULL)));
			(__temp143 == NULL ? NULL : (__temp143 = (g_free (__temp143), NULL)));
			return;
		}
		if (vala_expression_get_static_type (vala_binary_expression_get_left (expr))->type == self->priv->string_type->type && vala_expression_get_static_type (vala_binary_expression_get_right (expr))->type == self->priv->string_type->type) {
			/* string comparison: convert to a.collate (b) OP 0 */
			ValaMemberAccess* __temp146 = NULL;
			ValaInvocationExpression* __temp147 = NULL;
			ValaInvocationExpression* cmp_call = (__temp147 = g_object_new (VALA_TYPE_INVOCATION_EXPRESSION, "call", (__temp146 = g_object_new (VALA_TYPE_MEMBER_ACCESS, "inner", vala_binary_expression_get_left (expr), "member_name", "collate", NULL)), NULL), (__temp146 == NULL ? NULL : (__temp146 = (g_object_unref (__temp146), NULL))), __temp147);
			vala_invocation_expression_add_argument (cmp_call, vala_binary_expression_get_right (expr));
			vala_binary_expression_set_left (expr, VALA_EXPRESSION (cmp_call));
			ValaLiteralExpression* __temp149 = NULL;
			ValaIntegerLiteral* __temp148 = NULL;
			vala_binary_expression_set_right (expr, VALA_EXPRESSION ((__temp149 = g_object_new (VALA_TYPE_LITERAL_EXPRESSION, "literal", (__temp148 = g_object_new (VALA_TYPE_INTEGER_LITERAL, "value", "0", NULL)), NULL))));
			(__temp149 == NULL ? NULL : (__temp149 = (g_object_unref (__temp149), NULL)));
			(__temp148 == NULL ? NULL : (__temp148 = (g_object_unref (__temp148), NULL)));
			vala_code_node_accept (VALA_CODE_NODE (vala_binary_expression_get_left (expr)), VALA_CODE_VISITOR (self));
			(cmp_call == NULL ? NULL : (cmp_call = (g_object_unref (cmp_call), NULL)));
		}
		vala_expression_set_static_type (VALA_EXPRESSION (expr), self->priv->bool_type);
	} else if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_BITWISE_AND || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_BITWISE_OR) {
		/* integer type or flags type*/
		vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_binary_expression_get_left (expr)));
	} else if (vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_AND || vala_binary_expression_get_operator (expr) == VALA_BINARY_OPERATOR_OR) {
		if (vala_expression_get_static_type (vala_binary_expression_get_left (expr))->type != self->priv->bool_type->type || vala_expression_get_static_type (vala_binary_expression_get_right (expr))->type != self->priv->bool_type->type) {
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (expr)), "Operands must be boolean");
		}
		vala_expression_set_static_type (VALA_EXPRESSION (expr), self->priv->bool_type);
	} else {
		g_assert_not_reached ();
	}
}


static void vala_semantic_analyzer_real_visit_type_check (ValaCodeVisitor* base, ValaTypeCheck* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_TYPE_CHECK (expr));
	if (vala_type_check_get_type_reference (expr)->type == NULL) {
		return;
	}
	/* if type resolving didn't succeed, skip this check */
	vala_source_file_add_symbol_dependency (self->priv->current_source_file, vala_code_node_get_symbol (VALA_CODE_NODE (vala_type_check_get_type_reference (expr)->type)), VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE);
	vala_expression_set_static_type (VALA_EXPRESSION (expr), self->priv->bool_type);
}


static void vala_semantic_analyzer_real_visit_conditional_expression (ValaCodeVisitor* base, ValaConditionalExpression* expr)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_CONDITIONAL_EXPRESSION (expr));
	if (vala_expression_get_static_type (vala_conditional_expression_get_condition (expr))->type != self->priv->bool_type->type) {
		vala_code_node_set_error (VALA_CODE_NODE (expr), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_conditional_expression_get_condition (expr))), "Condition must be boolean");
		return;
	}
	/* FIXME: use greatest lower bound in the type hierarchy */
	vala_expression_set_static_type (VALA_EXPRESSION (expr), vala_expression_get_static_type (vala_conditional_expression_get_true_expression (expr)));
}


static char* vala_semantic_analyzer_get_lambda_name (ValaSemanticAnalyzer* self)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	char* result = g_strdup_printf ("__lambda%d", self->priv->next_lambda_id);
	self->priv->next_lambda_id++;
	char* __temp150 = NULL;
	char* __temp151 = NULL;
	return (__temp151 = (__temp150 = result, (__temp150 == NULL ? __temp150 : g_strdup (__temp150))), (result == NULL ? NULL : (result = (g_free (result), NULL))), __temp151);
	(result == NULL ? NULL : (result = (g_free (result), NULL)));
}


static ValaMethod* vala_semantic_analyzer_find_current_method (ValaSemanticAnalyzer* self)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), NULL);
	ValaSymbol* sym = self->priv->current_symbol;
	while (sym != NULL) {
		if (VALA_IS_METHOD (vala_symbol_get_node (sym))) {
			return VALA_METHOD (vala_symbol_get_node (sym));
		}
		sym = sym->parent_symbol;
	}
	return NULL;
}


static void vala_semantic_analyzer_real_visit_begin_lambda_expression (ValaCodeVisitor* base, ValaLambdaExpression* l)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_LAMBDA_EXPRESSION (l));
	if (vala_expression_get_expected_type (VALA_EXPRESSION (l)) == NULL || !(VALA_IS_CALLBACK (vala_expression_get_expected_type (VALA_EXPRESSION (l))->type))) {
		vala_code_node_set_error (VALA_CODE_NODE (l), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (l)), "lambda expression not allowed in this context");
		return;
	}
	ValaMethod* current_method = vala_semantic_analyzer_find_current_method (self);
	ValaCallback* cb = VALA_CALLBACK (vala_expression_get_expected_type (VALA_EXPRESSION (l))->type);
	ValaMethod* __temp155 = NULL;
	char* __temp154 = NULL;
	vala_lambda_expression_set_method (l, (__temp155 = g_object_new (VALA_TYPE_METHOD, "name", (__temp154 = vala_semantic_analyzer_get_lambda_name (self)), "return_type", vala_callback_get_return_type (cb), NULL)));
	(__temp155 == NULL ? NULL : (__temp155 = (g_object_unref (__temp155), NULL)));
	(__temp154 == NULL ? NULL : (__temp154 = (g_free (__temp154), NULL)));
	vala_method_set_instance (vala_lambda_expression_get_method (l), vala_callback_get_instance (cb) && vala_method_get_instance (current_method));
	ValaSymbol* __temp156 = NULL;
	vala_code_node_set_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l)), (__temp156 = g_object_new (VALA_TYPE_SYMBOL, "node", vala_lambda_expression_get_method (l), NULL)));
	(__temp156 == NULL ? NULL : (__temp156 = (g_object_unref (__temp156), NULL)));
	vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l)))->parent_symbol = self->priv->current_symbol;
	GList* lambda_params = vala_lambda_expression_get_parameters (l);
	GList* lambda_param_it = lambda_params;
	{
		GList * cb_param_it;
		for (cb_param_it = vala_callback_get_parameters (cb); cb_param_it != NULL; cb_param_it = cb_param_it->next) {
			ValaFormalParameter* cb_param = cb_param_it->data;
			{
				if (lambda_param_it == NULL) {
					break;
				}
				/* lambda expressions are allowed to have less parameters */
				char* lambda_param = lambda_param_it->data;
				ValaFormalParameter* param = g_object_new (VALA_TYPE_FORMAL_PARAMETER, "name", lambda_param, NULL);
				vala_formal_parameter_set_type_reference (param, vala_formal_parameter_get_type_reference (cb_param));
				ValaSymbol* __temp157 = NULL;
				vala_code_node_set_symbol (VALA_CODE_NODE (param), (__temp157 = g_object_new (VALA_TYPE_SYMBOL, "node", param, NULL)));
				(__temp157 == NULL ? NULL : (__temp157 = (g_object_unref (__temp157), NULL)));
				vala_symbol_add (vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l))), vala_formal_parameter_get_name (param), vala_code_node_get_symbol (VALA_CODE_NODE (param)));
				vala_method_add_parameter (vala_lambda_expression_get_method (l), param);
				lambda_param_it = lambda_param_it->next;
				(param == NULL ? NULL : (param = (g_object_unref (param), NULL)));
			}
		}
	}
	if (lambda_param_it != NULL) {
		/* lambda expressions may not expect more parameters */
		vala_code_node_set_error (VALA_CODE_NODE (l), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (l)), "lambda expression: too many parameters");
		(lambda_params == NULL ? NULL : (lambda_params = (g_list_free (lambda_params), NULL)));
		return;
	}
	if (vala_lambda_expression_get_expression_body (l) != NULL) {
		ValaBlock* block = g_object_new (VALA_TYPE_BLOCK, NULL);
		ValaSymbol* __temp158 = NULL;
		vala_code_node_set_symbol (VALA_CODE_NODE (block), (__temp158 = g_object_new (VALA_TYPE_SYMBOL, "node", block, NULL)));
		(__temp158 == NULL ? NULL : (__temp158 = (g_object_unref (__temp158), NULL)));
		vala_code_node_get_symbol (VALA_CODE_NODE (block))->parent_symbol = vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l)));
		if (vala_method_get_return_type (vala_lambda_expression_get_method (l))->type != NULL) {
			ValaReturnStatement* __temp159 = NULL;
			vala_block_add_statement (block, VALA_STATEMENT ((__temp159 = g_object_new (VALA_TYPE_RETURN_STATEMENT, "return_expression", vala_lambda_expression_get_expression_body (l), NULL))));
			(__temp159 == NULL ? NULL : (__temp159 = (g_object_unref (__temp159), NULL)));
		} else {
			ValaExpressionStatement* __temp160 = NULL;
			vala_block_add_statement (block, VALA_STATEMENT ((__temp160 = g_object_new (VALA_TYPE_EXPRESSION_STATEMENT, "expression", vala_lambda_expression_get_expression_body (l), NULL))));
			(__temp160 == NULL ? NULL : (__temp160 = (g_object_unref (__temp160), NULL)));
		}
		vala_method_set_body (vala_lambda_expression_get_method (l), VALA_STATEMENT (block));
		(block == NULL ? NULL : (block = (g_object_unref (block), NULL)));
	} else {
		vala_method_set_body (vala_lambda_expression_get_method (l), VALA_STATEMENT (vala_lambda_expression_get_statement_body (l)));
		vala_code_node_get_symbol (VALA_CODE_NODE (vala_method_get_body (vala_lambda_expression_get_method (l))))->parent_symbol = vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l)));
	}
	/* lambda expressions should be usable like MemberAccess of a method */
	vala_expression_set_symbol_reference (VALA_EXPRESSION (l), vala_code_node_get_symbol (VALA_CODE_NODE (vala_lambda_expression_get_method (l))));
	(lambda_params == NULL ? NULL : (lambda_params = (g_list_free (lambda_params), NULL)));
}


static void vala_semantic_analyzer_real_visit_begin_assignment (ValaCodeVisitor* base, ValaAssignment* a)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_ASSIGNMENT (a));
	ValaMemberAccess* ma = VALA_MEMBER_ACCESS (vala_assignment_get_left (a));
	if (VALA_IS_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))))) {
		ValaSignal* sig = VALA_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))));
		ValaTypeReference* __temp161 = NULL;
		vala_expression_set_expected_type (vala_assignment_get_right (a), (__temp161 = g_object_new (VALA_TYPE_TYPE_REFERENCE, NULL)));
		(__temp161 == NULL ? NULL : (__temp161 = (g_object_unref (__temp161), NULL)));
		vala_expression_get_expected_type (vala_assignment_get_right (a))->type = VALA_DATA_TYPE (vala_signal_get_callback (sig));
	}
}


static void vala_semantic_analyzer_real_visit_end_assignment (ValaCodeVisitor* base, ValaAssignment* a)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (base);
	g_return_if_fail (VALA_IS_ASSIGNMENT (a));
	if (!(VALA_IS_MEMBER_ACCESS (vala_assignment_get_left (a)))) {
		vala_code_node_set_error (VALA_CODE_NODE (a), TRUE);
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), "unsupported lvalue in assignment");
		return;
	}
	ValaMemberAccess* ma = VALA_MEMBER_ACCESS (vala_assignment_get_left (a));
	if (VALA_IS_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))))) {
		ValaSignal* sig = VALA_SIGNAL (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))));
		if (vala_expression_get_symbol_reference (vala_assignment_get_right (a)) == NULL) {
			vala_code_node_set_error (VALA_CODE_NODE (vala_assignment_get_right (a)), TRUE);
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_assignment_get_right (a))), "unsupported expression for signal handler");
			return;
		}
		ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (vala_assignment_get_right (a))));
		if (vala_method_get_instance (m) && m->access == VALA_MEMBER_ACCESSIBILITY_PUBLIC) {
			/* TODO: generate wrapper function */
			vala_code_node_set_error (VALA_CODE_NODE (ma), TRUE);
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (vala_assignment_get_right (a))), "public instance methods not yet supported as signal handlers");
			return;
		}
		if (vala_method_get_instance (m)) {
			/* instance signal handlers must have the self
			 * parameter at the end
			 * do not use G_CONNECT_SWAPPED as this would
			 * rearrange the parameters for instance
			 * methods and non-instance methods
			 */
			vala_method_set_instance_last (m, TRUE);
		}
	} else if (VALA_IS_PROPERTY (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))))) {
		ValaProperty* prop = VALA_PROPERTY (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))));
		if (vala_property_get_set_accessor (prop) == NULL) {
			vala_code_node_set_error (VALA_CODE_NODE (ma), TRUE);
			char* __temp163 = NULL;
			char* __temp162 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (ma)), (__temp163 = g_strdup_printf ("Property `%s' is read-only", (__temp162 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (prop)))))));
			(__temp163 == NULL ? NULL : (__temp163 = (g_free (__temp163), NULL)));
			(__temp162 == NULL ? NULL : (__temp162 = (g_free (__temp162), NULL)));
			return;
		}
	} else if (VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)))) && vala_expression_get_static_type (vala_assignment_get_right (a)) == NULL) {
		ValaVariableDeclarator* decl = VALA_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))));
		ValaMemberAccess* right_ma = VALA_MEMBER_ACCESS (vala_assignment_get_right (a));
		if (VALA_IS_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (right_ma)))) && VALA_IS_CALLBACK (decl->type_reference->type)) {
			ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (right_ma))));
			ValaCallback* cb = VALA_CALLBACK (decl->type_reference->type);
			/* check whether method matches callback type */
			if (!vala_callback_matches_method (cb, m)) {
				vala_code_node_set_error (VALA_CODE_NODE (decl), TRUE);
				char* __temp166 = NULL;
				char* __temp165 = NULL;
				char* __temp164 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), (__temp166 = g_strdup_printf ("declaration of method `%s' doesn't match declaration of callback `%s'", (__temp164 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))), (__temp165 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (cb)))))));
				(__temp166 == NULL ? NULL : (__temp166 = (g_free (__temp166), NULL)));
				(__temp165 == NULL ? NULL : (__temp165 = (g_free (__temp165), NULL)));
				(__temp164 == NULL ? NULL : (__temp164 = (g_free (__temp164), NULL)));
				return;
			}
			vala_expression_set_static_type (vala_assignment_get_right (a), decl->type_reference);
		} else {
			vala_code_node_set_error (VALA_CODE_NODE (a), TRUE);
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), "Assignment: Invalid callback assignment attempt");
			return;
		}
	} else if (VALA_IS_FIELD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)))) && vala_expression_get_static_type (vala_assignment_get_right (a)) == NULL) {
		ValaField* f = VALA_FIELD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma))));
		ValaMemberAccess* right_ma = VALA_MEMBER_ACCESS (vala_assignment_get_right (a));
		if (VALA_IS_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (right_ma)))) && VALA_IS_CALLBACK (vala_field_get_type_reference (f)->type)) {
			ValaMethod* m = VALA_METHOD (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (right_ma))));
			ValaCallback* cb = VALA_CALLBACK (vala_field_get_type_reference (f)->type);
			/* check whether method matches callback type */
			if (!vala_callback_matches_method (cb, m)) {
				vala_code_node_set_error (VALA_CODE_NODE (f), TRUE);
				char* __temp169 = NULL;
				char* __temp168 = NULL;
				char* __temp167 = NULL;
				vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), (__temp169 = g_strdup_printf ("declaration of method `%s' doesn't match declaration of callback `%s'", (__temp167 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (m)))), (__temp168 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (cb)))))));
				(__temp169 == NULL ? NULL : (__temp169 = (g_free (__temp169), NULL)));
				(__temp168 == NULL ? NULL : (__temp168 = (g_free (__temp168), NULL)));
				(__temp167 == NULL ? NULL : (__temp167 = (g_free (__temp167), NULL)));
				return;
			}
			vala_expression_set_static_type (vala_assignment_get_right (a), vala_field_get_type_reference (f));
		} else {
			vala_code_node_set_error (VALA_CODE_NODE (a), TRUE);
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), "Assignment: Invalid callback assignment attempt");
			return;
		}
	} else if (vala_expression_get_static_type (vala_assignment_get_left (a)) != NULL && vala_expression_get_static_type (vala_assignment_get_right (a)) != NULL) {
		if (!vala_semantic_analyzer_is_type_compatible (self, vala_expression_get_static_type (vala_assignment_get_right (a)), vala_expression_get_static_type (vala_assignment_get_left (a)))) {
			/* if there was an error on either side,
			 * i.e. a.{left|right}.static_type == null, skip type check */
			char* __temp172 = NULL;
			char* __temp171 = NULL;
			char* __temp170 = NULL;
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), (__temp172 = g_strdup_printf ("Assignment: Cannot convert from `%s' to `%s'", (__temp170 = vala_type_reference_to_string (vala_expression_get_static_type (vala_assignment_get_right (a)))), (__temp171 = vala_type_reference_to_string (vala_expression_get_static_type (vala_assignment_get_left (a)))))));
			(__temp172 == NULL ? NULL : (__temp172 = (g_free (__temp172), NULL)));
			(__temp171 == NULL ? NULL : (__temp171 = (g_free (__temp171), NULL)));
			(__temp170 == NULL ? NULL : (__temp170 = (g_free (__temp170), NULL)));
			return;
		}
		if (vala_semantic_analyzer_get_memory_management (self)) {
			if (vala_type_reference_get_is_ref (vala_expression_get_static_type (vala_assignment_get_right (a)))) {
				/* rhs transfers ownership of the expression */
				if (!vala_type_reference_get_is_lvalue_ref (vala_expression_get_static_type (vala_assignment_get_left (a)))) {
					/* lhs doesn't own the value
					 * promote lhs type if it is a local variable
					 * error if it's not a local variable */
					if (!(VALA_IS_VARIABLE_DECLARATOR (vala_symbol_get_node (vala_expression_get_symbol_reference (VALA_EXPRESSION (ma)))))) {
						vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (a)), "Invalid assignment from owned expression to unowned variable");
					}
					vala_type_reference_set_is_lvalue_ref (vala_expression_get_static_type (vala_assignment_get_left (a)), TRUE);
				}
			} else if (vala_type_reference_get_is_lvalue_ref (vala_expression_get_static_type (vala_assignment_get_left (a)))) {
			}
		}
	}
	/* lhs wants to own the value
	 * rhs doesn't transfer the ownership
	 * code generator needs to add reference
	 * increment calls */
	vala_expression_set_static_type (VALA_EXPRESSION (a), vala_expression_get_static_type (vala_assignment_get_left (a)));
}


gboolean vala_semantic_analyzer_get_memory_management (ValaSemanticAnalyzer* self)
{
	g_return_val_if_fail (VALA_IS_SEMANTIC_ANALYZER (self), FALSE);
	return self->priv->_memory_management;
}


void vala_semantic_analyzer_set_memory_management (ValaSemanticAnalyzer* self, gboolean value)
{
	g_return_if_fail (VALA_IS_SEMANTIC_ANALYZER (self));
	self->priv->_memory_management = value;
}


void vala_semantic_analyzer_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (object);
	switch (property_id) {
		case VALA_SEMANTIC_ANALYZER_MEMORY_MANAGEMENT:
		g_value_set_boolean (value, vala_semantic_analyzer_get_memory_management (self));
		break;
	}
}


void vala_semantic_analyzer_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (object);
	switch (property_id) {
		case VALA_SEMANTIC_ANALYZER_MEMORY_MANAGEMENT:
		vala_semantic_analyzer_set_memory_management (self, g_value_get_boolean (value));
		break;
	}
}


static void vala_semantic_analyzer_class_init (ValaSemanticAnalyzerClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaSemanticAnalyzerPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_semantic_analyzer_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_semantic_analyzer_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_semantic_analyzer_dispose;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_source_file = vala_semantic_analyzer_real_visit_begin_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_source_file = vala_semantic_analyzer_real_visit_end_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_namespace = vala_semantic_analyzer_real_visit_begin_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_namespace = vala_semantic_analyzer_real_visit_end_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_class = vala_semantic_analyzer_real_visit_begin_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_class = vala_semantic_analyzer_real_visit_end_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_struct = vala_semantic_analyzer_real_visit_begin_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_struct = vala_semantic_analyzer_real_visit_end_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_field = vala_semantic_analyzer_real_visit_field;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_method = vala_semantic_analyzer_real_visit_begin_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_method = vala_semantic_analyzer_real_visit_end_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_formal_parameter = vala_semantic_analyzer_real_visit_formal_parameter;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_property = vala_semantic_analyzer_real_visit_end_property;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_property_accessor = vala_semantic_analyzer_real_visit_begin_property_accessor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_property_accessor = vala_semantic_analyzer_real_visit_end_property_accessor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_constructor = vala_semantic_analyzer_real_visit_begin_constructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_constructor = vala_semantic_analyzer_real_visit_end_constructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_destructor = vala_semantic_analyzer_real_visit_begin_destructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_destructor = vala_semantic_analyzer_real_visit_end_destructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_named_argument = vala_semantic_analyzer_real_visit_named_argument;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_block = vala_semantic_analyzer_real_visit_begin_block;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_block = vala_semantic_analyzer_real_visit_end_block;
	VALA_CODE_VISITOR_CLASS (klass)->visit_variable_declarator = vala_semantic_analyzer_real_visit_variable_declarator;
	VALA_CODE_VISITOR_CLASS (klass)->visit_expression_statement = vala_semantic_analyzer_real_visit_expression_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_if_statement = vala_semantic_analyzer_real_visit_if_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_while_statement = vala_semantic_analyzer_real_visit_while_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_for_statement = vala_semantic_analyzer_real_visit_for_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_foreach_statement = vala_semantic_analyzer_real_visit_begin_foreach_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_return_statement = vala_semantic_analyzer_real_visit_return_statement;
	VALA_CODE_VISITOR_CLASS (klass)->visit_boolean_literal = vala_semantic_analyzer_real_visit_boolean_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_character_literal = vala_semantic_analyzer_real_visit_character_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_integer_literal = vala_semantic_analyzer_real_visit_integer_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_real_literal = vala_semantic_analyzer_real_visit_real_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_string_literal = vala_semantic_analyzer_real_visit_string_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_null_literal = vala_semantic_analyzer_real_visit_null_literal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_literal_expression = vala_semantic_analyzer_real_visit_literal_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_parenthesized_expression = vala_semantic_analyzer_real_visit_parenthesized_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_member_access = vala_semantic_analyzer_real_visit_member_access;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_invocation_expression = vala_semantic_analyzer_real_visit_begin_invocation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_invocation_expression = vala_semantic_analyzer_real_visit_end_invocation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_element_access = vala_semantic_analyzer_real_visit_element_access;
	VALA_CODE_VISITOR_CLASS (klass)->visit_postfix_expression = vala_semantic_analyzer_real_visit_postfix_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_object_creation_expression = vala_semantic_analyzer_real_visit_object_creation_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_unary_expression = vala_semantic_analyzer_real_visit_unary_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_cast_expression = vala_semantic_analyzer_real_visit_cast_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_binary_expression = vala_semantic_analyzer_real_visit_binary_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_type_check = vala_semantic_analyzer_real_visit_type_check;
	VALA_CODE_VISITOR_CLASS (klass)->visit_conditional_expression = vala_semantic_analyzer_real_visit_conditional_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_lambda_expression = vala_semantic_analyzer_real_visit_begin_lambda_expression;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_assignment = vala_semantic_analyzer_real_visit_begin_assignment;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_assignment = vala_semantic_analyzer_real_visit_end_assignment;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SEMANTIC_ANALYZER_MEMORY_MANAGEMENT, g_param_spec_boolean ("memory-management", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_semantic_analyzer_init (ValaSemanticAnalyzer * self)
{
	self->priv = VALA_SEMANTIC_ANALYZER_GET_PRIVATE (self);
	self->priv->next_lambda_id = 0;
}


static void vala_semantic_analyzer_dispose (GObject * obj)
{
	ValaSemanticAnalyzer * self = VALA_SEMANTIC_ANALYZER (obj);
	(self->priv->root_symbol == NULL ? NULL : (self->priv->root_symbol = (g_object_unref (self->priv->root_symbol), NULL)));
	(self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL)));
	(self->priv->current_source_file == NULL ? NULL : (self->priv->current_source_file = (g_object_unref (self->priv->current_source_file), NULL)));
	(self->priv->current_return_type == NULL ? NULL : (self->priv->current_return_type = (g_object_unref (self->priv->current_return_type), NULL)));
	(self->priv->current_using_directives == NULL ? NULL : (self->priv->current_using_directives = (g_list_free (self->priv->current_using_directives), NULL)));
	(self->priv->bool_type == NULL ? NULL : (self->priv->bool_type = (g_object_unref (self->priv->bool_type), NULL)));
	(self->priv->string_type == NULL ? NULL : (self->priv->string_type = (g_object_unref (self->priv->string_type), NULL)));
	(self->priv->int_type == NULL ? NULL : (self->priv->int_type = (g_object_unref (self->priv->int_type), NULL)));
	(self->priv->pointer_type == NULL ? NULL : (self->priv->pointer_type = (g_object_unref (self->priv->pointer_type), NULL)));
	(self->priv->initially_unowned_type == NULL ? NULL : (self->priv->initially_unowned_type = (g_object_unref (self->priv->initially_unowned_type), NULL)));
	ValaSemanticAnalyzerClass * klass;
	GObjectClass * parent_class;
	klass = VALA_SEMANTIC_ANALYZER_CLASS (g_type_class_peek (VALA_TYPE_SEMANTIC_ANALYZER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_semantic_analyzer_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaSemanticAnalyzerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_semantic_analyzer_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaSemanticAnalyzer), 0, (GInstanceInitFunc) vala_semantic_analyzer_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "ValaSemanticAnalyzer", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



