/* valasignal.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valasignal.h>
#include <valatypereference.h>
#include <valacallback.h>
#include <valasourcereference.h>
#include <valaformalparameter.h>
#include <valasymbol.h>
#include <valadatatype.h>
#include <valacodevisitor.h>

struct _ValaSignalPrivate {
	char* _name;
	ValaTypeReference* _return_type;
	GList* parameters;
	ValaCallback* generated_callback;
};
#define VALA_SIGNAL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_SIGNAL, ValaSignalPrivate))
enum  {
	VALA_SIGNAL_DUMMY_PROPERTY,
	VALA_SIGNAL_NAME,
	VALA_SIGNAL_RETURN_TYPE,
};
static void vala_signal_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_signal_dispose (GObject * obj);

/**
 * Creates a new signal.
 *
 * @param name        signal name
 * @param return_type signal return type
 * @param source      reference to source code
 * @return            newly created signal
 */
ValaSignal* vala_signal_new (char* name, ValaTypeReference* return_type, ValaSourceReference* source)
{
	g_return_val_if_fail (name != NULL, NULL);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (return_type), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_SIGNAL, "name", name, "return_type", return_type, "source_reference", source, NULL));
}


/**
 * Appends parameter to signal handler.
 *
 * @param param a formal parameter
 */
void vala_signal_add_parameter (ValaSignal* self, ValaFormalParameter* param)
{
	g_return_if_fail (VALA_IS_SIGNAL (self));
	g_return_if_fail (VALA_IS_FORMAL_PARAMETER (param));
	self->priv->parameters = g_list_append (self->priv->parameters, g_object_ref (param));
}


/**
 * Returns copy of list of signal handler parameters.
 *
 * @return parameter list
 */
GList* vala_signal_get_parameters (ValaSignal* self)
{
	g_return_val_if_fail (VALA_IS_SIGNAL (self), NULL);
	return g_list_copy (self->priv->parameters);
}


/**
 * Returns generated callback to be used for signal handlers.
 *
 * @return callback
 */
ValaCallback* vala_signal_get_callback (ValaSignal* self)
{
	g_return_val_if_fail (VALA_IS_SIGNAL (self), NULL);
	if (self->priv->generated_callback == NULL) {
		ValaCallback* __temp2 = NULL;
		self->priv->generated_callback = (__temp2 = g_object_new (VALA_TYPE_CALLBACK, "return_type", vala_signal_get_return_type (self), "instance", TRUE, NULL), (self->priv->generated_callback == NULL ? NULL : (self->priv->generated_callback = (g_object_unref (self->priv->generated_callback), NULL))), __temp2);
		ValaFormalParameter* sender_param = g_object_new (VALA_TYPE_FORMAL_PARAMETER, "name", "sender", NULL);
		ValaTypeReference* __temp3 = NULL;
		vala_formal_parameter_set_type_reference (sender_param, (__temp3 = g_object_new (VALA_TYPE_TYPE_REFERENCE, NULL)));
		(__temp3 == NULL ? NULL : (__temp3 = (g_object_unref (__temp3), NULL)));
		vala_formal_parameter_get_type_reference (sender_param)->type = VALA_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (self))->parent_symbol));
		vala_callback_add_parameter (self->priv->generated_callback, sender_param);
		{
			GList * param_it;
			for (param_it = self->priv->parameters; param_it != NULL; param_it = param_it->next) {
				ValaFormalParameter* param = param_it->data;
				{
					vala_callback_add_parameter (self->priv->generated_callback, param);
				}
			}
		}
		(sender_param == NULL ? NULL : (sender_param = (g_object_unref (sender_param), NULL)));
	}
	return self->priv->generated_callback;
}


static void vala_signal_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaSignal * self = VALA_SIGNAL (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_begin_signal (visitor, self);
	vala_code_node_accept (VALA_CODE_NODE (vala_signal_get_return_type (self)), visitor);
	{
		GList * param_it;
		for (param_it = self->priv->parameters; param_it != NULL; param_it = param_it->next) {
			ValaFormalParameter* param = param_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (param), visitor);
			}
		}
	}
	vala_code_visitor_visit_end_signal (visitor, self);
}


char* vala_signal_get_name (ValaSignal* self)
{
	g_return_val_if_fail (VALA_IS_SIGNAL (self), NULL);
	return self->priv->_name;
}


void vala_signal_set_name (ValaSignal* self, char* value)
{
	g_return_if_fail (VALA_IS_SIGNAL (self));
	char* __temp6 = NULL;
	self->priv->_name = (__temp6 = g_strdup (value), (self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL))), __temp6);
}


ValaTypeReference* vala_signal_get_return_type (ValaSignal* self)
{
	g_return_val_if_fail (VALA_IS_SIGNAL (self), NULL);
	return self->priv->_return_type;
}


void vala_signal_set_return_type (ValaSignal* self, ValaTypeReference* value)
{
	g_return_if_fail (VALA_IS_SIGNAL (self));
	ValaTypeReference* __temp8 = NULL;
	self->priv->_return_type = (__temp8 = g_object_ref (value), (self->priv->_return_type == NULL ? NULL : (self->priv->_return_type = (g_object_unref (self->priv->_return_type), NULL))), __temp8);
}


void vala_signal_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaSignal * self = VALA_SIGNAL (object);
	switch (property_id) {
		case VALA_SIGNAL_NAME:
		g_value_set_string (value, vala_signal_get_name (self));
		break;
		case VALA_SIGNAL_RETURN_TYPE:
		g_value_set_object (value, vala_signal_get_return_type (self));
		break;
	}
}


void vala_signal_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaSignal * self = VALA_SIGNAL (object);
	switch (property_id) {
		case VALA_SIGNAL_NAME:
		vala_signal_set_name (self, g_value_get_string (value));
		break;
		case VALA_SIGNAL_RETURN_TYPE:
		vala_signal_set_return_type (self, g_value_get_object (value));
		break;
	}
}


static void vala_signal_class_init (ValaSignalClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaSignalPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_signal_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_signal_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_signal_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_signal_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SIGNAL_NAME, g_param_spec_string ("name", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SIGNAL_RETURN_TYPE, g_param_spec_object ("return-type", "foo", "bar", VALA_TYPE_TYPE_REFERENCE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
}


static void vala_signal_init (ValaSignal * self)
{
	self->priv = VALA_SIGNAL_GET_PRIVATE (self);
}


static void vala_signal_dispose (GObject * obj)
{
	ValaSignal * self = VALA_SIGNAL (obj);
	(self->priv->_name == NULL ? NULL : (self->priv->_name = (g_free (self->priv->_name), NULL)));
	(self->priv->_return_type == NULL ? NULL : (self->priv->_return_type = (g_object_unref (self->priv->_return_type), NULL)));
	(self->priv->parameters == NULL ? NULL : (self->priv->parameters = (g_list_foreach (self->priv->parameters, (GFunc) g_object_unref, NULL), g_list_free (self->priv->parameters), NULL)));
	(self->priv->generated_callback == NULL ? NULL : (self->priv->generated_callback = (g_object_unref (self->priv->generated_callback), NULL)));
	ValaSignalClass * klass;
	GObjectClass * parent_class;
	klass = VALA_SIGNAL_CLASS (g_type_class_peek (VALA_TYPE_SIGNAL));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_signal_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaSignalClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_signal_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaSignal), 0, (GInstanceInitFunc) vala_signal_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_NODE, "ValaSignal", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



