/* valasourcefile.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valasourcefile.h>
#include <valanamespace.h>
#include <valasourcereference.h>
#include <valanamespacereference.h>
#include <valacodevisitor.h>
#include <valasymbol.h>
#include <valadatatype.h>
#include <valamethod.h>
#include <valafield.h>
#include <valaproperty.h>
#include <valaformalparameter.h>
#include <valatypereference.h>

struct _ValaSourceFilePrivate {
	char* _filename;
	gboolean _pkg;
	GList* using_directives;
	ValaNamespace* global_namespace;
	GList* namespaces;
	char* cheader_filename;
	char* csource_filename;
};
#define VALA_SOURCE_FILE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_SOURCE_FILE, ValaSourceFilePrivate))
enum  {
	VALA_SOURCE_FILE_DUMMY_PROPERTY,
	VALA_SOURCE_FILE_FILENAME,
	VALA_SOURCE_FILE_PKG,
};
static void vala_source_file_dispose (GObject * obj);

void vala_source_file_add_using_directive (ValaSourceFile* self, ValaNamespaceReference* ns)
{
	g_return_if_fail (VALA_IS_SOURCE_FILE (self));
	g_return_if_fail (ns == NULL || VALA_IS_NAMESPACE_REFERENCE (ns));
	ValaNamespaceReference* __temp2 = NULL;
	self->priv->using_directives = g_list_append (self->priv->using_directives, (__temp2 = ns, (__temp2 == NULL ? __temp2 : g_object_ref (__temp2))));
}


GList* vala_source_file_get_using_directives (ValaSourceFile* self)
{
	g_return_val_if_fail (VALA_IS_SOURCE_FILE (self), NULL);
	return self->priv->using_directives;
}


void vala_source_file_add_namespace (ValaSourceFile* self, ValaNamespace* ns)
{
	g_return_if_fail (VALA_IS_SOURCE_FILE (self));
	g_return_if_fail (ns == NULL || VALA_IS_NAMESPACE (ns));
	ValaNamespace* __temp4 = NULL;
	self->priv->namespaces = g_list_append (self->priv->namespaces, (__temp4 = ns, (__temp4 == NULL ? __temp4 : g_object_ref (__temp4))));
}


ValaNamespace* vala_source_file_get_global_namespace (ValaSourceFile* self)
{
	g_return_val_if_fail (VALA_IS_SOURCE_FILE (self), NULL);
	return self->priv->global_namespace;
}


GList* vala_source_file_get_namespaces (ValaSourceFile* self)
{
	g_return_val_if_fail (VALA_IS_SOURCE_FILE (self), NULL);
	return g_list_copy (self->priv->namespaces);
}


void vala_source_file_accept (ValaSourceFile* self, ValaCodeVisitor* visitor)
{
	g_return_if_fail (VALA_IS_SOURCE_FILE (self));
	g_return_if_fail (visitor == NULL || VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_begin_source_file (visitor, self);
	{
		GList * ns_ref_it;
		for (ns_ref_it = self->priv->using_directives; ns_ref_it != NULL; ns_ref_it = ns_ref_it->next) {
			ValaNamespaceReference* ns_ref = ns_ref_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (ns_ref), visitor);
			}
		}
	}
	vala_code_node_accept (VALA_CODE_NODE (self->priv->global_namespace), visitor);
	{
		GList * ns_it;
		for (ns_it = self->priv->namespaces; ns_it != NULL; ns_it = ns_it->next) {
			ValaNamespace* ns = ns_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (ns), visitor);
			}
		}
	}
	vala_code_visitor_visit_end_source_file (visitor, self);
}


char* vala_source_file_get_cheader_filename (ValaSourceFile* self)
{
	g_return_val_if_fail (VALA_IS_SOURCE_FILE (self), NULL);
	if (self->priv->cheader_filename == NULL) {
		char* basename = g_strndup (vala_source_file_get_filename (self), (g_utf8_strlen (vala_source_file_get_filename (self), -1) - g_utf8_strlen (".vala", -1)));
		char* __temp7 = NULL;
		self->priv->cheader_filename = (__temp7 = g_strdup_printf ("%s.h", basename), (self->priv->cheader_filename == NULL ? NULL : (self->priv->cheader_filename = (g_free (self->priv->cheader_filename), NULL))), __temp7);
		(basename == NULL ? NULL : (basename = (g_free (basename), NULL)));
	}
	return self->priv->cheader_filename;
}


char* vala_source_file_get_csource_filename (ValaSourceFile* self)
{
	g_return_val_if_fail (VALA_IS_SOURCE_FILE (self), NULL);
	if (self->priv->csource_filename == NULL) {
		char* basename = g_strndup (vala_source_file_get_filename (self), (g_utf8_strlen (vala_source_file_get_filename (self), -1) - g_utf8_strlen (".vala", -1)));
		char* __temp9 = NULL;
		self->priv->csource_filename = (__temp9 = g_strdup_printf ("%s.c", basename), (self->priv->csource_filename == NULL ? NULL : (self->priv->csource_filename = (g_free (self->priv->csource_filename), NULL))), __temp9);
		(basename == NULL ? NULL : (basename = (g_free (basename), NULL)));
	}
	return self->priv->csource_filename;
}


/* used for cycle detection, 0 = white (not yet visited), 1 = gray (currently visiting), 2 = black (already visited)*/
void vala_source_file_add_symbol_dependency (ValaSourceFile* self, ValaSymbol* sym, ValaSourceFileDependencyType dep_type)
{
	g_return_if_fail (VALA_IS_SOURCE_FILE (self));
	g_return_if_fail (sym == NULL || VALA_IS_SYMBOL (sym));
	ValaDataType* t = NULL;
	if (VALA_IS_DATA_TYPE (vala_symbol_get_node (sym))) {
		t = VALA_DATA_TYPE (vala_symbol_get_node (sym));
	} else if (VALA_IS_METHOD (vala_symbol_get_node (sym)) || VALA_IS_FIELD (vala_symbol_get_node (sym))) {
		if (VALA_IS_DATA_TYPE (vala_symbol_get_node (sym->parent_symbol))) {
			t = VALA_DATA_TYPE (vala_symbol_get_node (sym->parent_symbol));
		} else {
			return;
		}
	} else if (VALA_IS_PROPERTY (vala_symbol_get_node (sym))) {
		t = VALA_DATA_TYPE (vala_symbol_get_node (sym->parent_symbol));
	} else if (VALA_IS_FORMAL_PARAMETER (vala_symbol_get_node (sym))) {
		ValaFormalParameter* fp = VALA_FORMAL_PARAMETER (vala_symbol_get_node (sym));
		t = vala_formal_parameter_get_type_reference (fp)->type;
		if (t == NULL) {
			return;
		}
	} else {
		return;
	}
	/* generic type parameter */
	if (dep_type == VALA_SOURCE_FILE_DEPENDENCY_TYPE_SOURCE) {
		self->source_includes = g_list_concat (self->source_includes, vala_data_type_get_cheader_filenames (t));
		return;
	}
	if (vala_source_file_get_pkg (vala_source_reference_get_file (vala_code_node_get_source_reference (VALA_CODE_NODE (t))))) {
		/* external package */
		self->header_external_includes = g_list_concat (self->header_external_includes, vala_data_type_get_cheader_filenames (t));
		return;
	}
	if (dep_type == VALA_SOURCE_FILE_DEPENDENCY_TYPE_HEADER_FULL || !vala_data_type_is_reference_type (t)) {
		self->header_internal_includes = g_list_concat (self->header_internal_includes, vala_data_type_get_cheader_filenames (t));
		self->header_internal_full_dependencies = g_list_append (self->header_internal_full_dependencies, vala_source_reference_get_file (vala_code_node_get_source_reference (VALA_CODE_NODE (t))));
	}
	self->header_internal_dependencies = g_list_append (self->header_internal_dependencies, vala_source_reference_get_file (vala_code_node_get_source_reference (VALA_CODE_NODE (t))));
}


char* vala_source_file_get_filename (ValaSourceFile* self)
{
	g_return_val_if_fail (VALA_IS_SOURCE_FILE (self), NULL);
	return self->priv->_filename;
}


void vala_source_file_set_filename (ValaSourceFile* self, char* value)
{
	g_return_if_fail (VALA_IS_SOURCE_FILE (self));
	char* __temp13 = NULL;
	char* __temp12 = NULL;
	self->priv->_filename = (__temp13 = (__temp12 = value, (__temp12 == NULL ? __temp12 : g_strdup (__temp12))), (self->priv->_filename == NULL ? NULL : (self->priv->_filename = (g_free (self->priv->_filename), NULL))), __temp13);
}


gboolean vala_source_file_get_pkg (ValaSourceFile* self)
{
	g_return_val_if_fail (VALA_IS_SOURCE_FILE (self), FALSE);
	return self->priv->_pkg;
}


void vala_source_file_set_pkg (ValaSourceFile* self, gboolean value)
{
	g_return_if_fail (VALA_IS_SOURCE_FILE (self));
	self->priv->_pkg = value;
}


void vala_source_file_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaSourceFile * self = VALA_SOURCE_FILE (object);
	switch (property_id) {
		case VALA_SOURCE_FILE_FILENAME:
		g_value_set_string (value, vala_source_file_get_filename (self));
		break;
		case VALA_SOURCE_FILE_PKG:
		g_value_set_boolean (value, vala_source_file_get_pkg (self));
		break;
	}
}


void vala_source_file_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaSourceFile * self = VALA_SOURCE_FILE (object);
	switch (property_id) {
		case VALA_SOURCE_FILE_FILENAME:
		vala_source_file_set_filename (self, g_value_get_string (value));
		break;
		case VALA_SOURCE_FILE_PKG:
		vala_source_file_set_pkg (self, g_value_get_boolean (value));
		break;
	}
}


static void vala_source_file_class_init (ValaSourceFileClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaSourceFilePrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_source_file_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_source_file_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_source_file_dispose;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SOURCE_FILE_FILENAME, g_param_spec_string ("filename", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_SOURCE_FILE_PKG, g_param_spec_boolean ("pkg", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vala_source_file_init (ValaSourceFile * self)
{
	self->priv = VALA_SOURCE_FILE_GET_PRIVATE (self);
	self->priv->cheader_filename = NULL;
	self->priv->csource_filename = NULL;
	{
		ValaNamespace* __temp1 = NULL;
		ValaSourceReference* __temp0 = NULL;
		self->priv->global_namespace = (__temp1 = g_object_new (VALA_TYPE_NAMESPACE, "source_reference", (__temp0 = g_object_new (VALA_TYPE_SOURCE_REFERENCE, "file", self, NULL)), NULL), (self->priv->global_namespace == NULL ? NULL : (self->priv->global_namespace = (g_object_unref (self->priv->global_namespace), NULL))), __temp1);
		(__temp0 == NULL ? NULL : (__temp0 = (g_object_unref (__temp0), NULL)));
	}
}


static void vala_source_file_dispose (GObject * obj)
{
	ValaSourceFile * self = VALA_SOURCE_FILE (obj);
	(self->priv->_filename == NULL ? NULL : (self->priv->_filename = (g_free (self->priv->_filename), NULL)));
	(self->comment == NULL ? NULL : (self->comment = (g_free (self->comment), NULL)));
	(self->priv->using_directives == NULL ? NULL : (self->priv->using_directives = (g_list_foreach (self->priv->using_directives, (GFunc) g_object_unref, NULL), g_list_free (self->priv->using_directives), NULL)));
	(self->priv->global_namespace == NULL ? NULL : (self->priv->global_namespace = (g_object_unref (self->priv->global_namespace), NULL)));
	(self->priv->namespaces == NULL ? NULL : (self->priv->namespaces = (g_list_foreach (self->priv->namespaces, (GFunc) g_object_unref, NULL), g_list_free (self->priv->namespaces), NULL)));
	(self->priv->cheader_filename == NULL ? NULL : (self->priv->cheader_filename = (g_free (self->priv->cheader_filename), NULL)));
	(self->priv->csource_filename == NULL ? NULL : (self->priv->csource_filename = (g_free (self->priv->csource_filename), NULL)));
	(self->header_external_includes == NULL ? NULL : (self->header_external_includes = (g_list_free (self->header_external_includes), NULL)));
	(self->header_internal_includes == NULL ? NULL : (self->header_internal_includes = (g_list_free (self->header_internal_includes), NULL)));
	(self->source_includes == NULL ? NULL : (self->source_includes = (g_list_free (self->source_includes), NULL)));
	(self->header_internal_full_dependencies == NULL ? NULL : (self->header_internal_full_dependencies = (g_list_free (self->header_internal_full_dependencies), NULL)));
	(self->header_internal_dependencies == NULL ? NULL : (self->header_internal_dependencies = (g_list_free (self->header_internal_dependencies), NULL)));
	(self->cycle == NULL ? NULL : (self->cycle = (g_object_unref (self->cycle), NULL)));
	ValaSourceFileClass * klass;
	GObjectClass * parent_class;
	klass = VALA_SOURCE_FILE_CLASS (g_type_class_peek (VALA_TYPE_SOURCE_FILE));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_source_file_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaSourceFileClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_source_file_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaSourceFile), 0, (GInstanceInitFunc) vala_source_file_init };
		g_define_type_id = g_type_register_static (G_TYPE_OBJECT, "ValaSourceFile", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



