/* valasymbolbuilder.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <valasymbolbuilder.h>
#include <valareport.h>
#include <valasourcereference.h>
#include <valadatatype.h>
#include <valatypereference.h>
#include <valamemberaccess.h>
#include <valareturnstatement.h>
#include <valastatement.h>
#include <valaassignment.h>
#include <valaexpressionstatement.h>

struct _ValaSymbolBuilderPrivate {
	ValaSymbol* root;
	ValaSymbol* current_type;
	ValaSymbol* current_symbol;
};
#define VALA_SYMBOL_BUILDER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_SYMBOL_BUILDER, ValaSymbolBuilderPrivate))
enum  {
	VALA_SYMBOL_BUILDER_DUMMY_PROPERTY,
};
static void vala_symbol_builder_real_visit_begin_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
static void vala_symbol_builder_real_visit_end_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
static ValaSymbol* vala_symbol_builder_add_symbol (ValaSymbolBuilder* self, char* name, ValaCodeNode* node);
static void vala_symbol_builder_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_symbol_builder_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl);
static void vala_symbol_builder_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vala_symbol_builder_real_visit_end_struct (ValaCodeVisitor* base, ValaStruct* st);
static void vala_symbol_builder_real_visit_begin_interface (ValaCodeVisitor* base, ValaInterface* iface);
static void vala_symbol_builder_real_visit_end_interface (ValaCodeVisitor* base, ValaInterface* iface);
static void vala_symbol_builder_real_visit_begin_enum (ValaCodeVisitor* base, ValaEnum* en);
static void vala_symbol_builder_real_visit_end_enum (ValaCodeVisitor* base, ValaEnum* en);
static void vala_symbol_builder_real_visit_enum_value (ValaCodeVisitor* base, ValaEnumValue* ev);
static void vala_symbol_builder_real_visit_begin_callback (ValaCodeVisitor* base, ValaCallback* cb);
static void vala_symbol_builder_real_visit_end_callback (ValaCodeVisitor* base, ValaCallback* cb);
static void vala_symbol_builder_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c);
static void vala_symbol_builder_real_visit_field (ValaCodeVisitor* base, ValaField* f);
static void vala_symbol_builder_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m);
static void vala_symbol_builder_real_visit_end_method (ValaCodeVisitor* base, ValaMethod* m);
static void vala_symbol_builder_real_visit_formal_parameter (ValaCodeVisitor* base, ValaFormalParameter* p);
static void vala_symbol_builder_real_visit_begin_property (ValaCodeVisitor* base, ValaProperty* prop);
static void vala_symbol_builder_real_visit_end_property (ValaCodeVisitor* base, ValaProperty* prop);
static void vala_symbol_builder_real_visit_begin_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc);
static void vala_symbol_builder_real_visit_end_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc);
static void vala_symbol_builder_real_visit_begin_signal (ValaCodeVisitor* base, ValaSignal* sig);
static void vala_symbol_builder_real_visit_end_signal (ValaCodeVisitor* base, ValaSignal* sig);
static void vala_symbol_builder_real_visit_begin_constructor (ValaCodeVisitor* base, ValaConstructor* c);
static void vala_symbol_builder_real_visit_end_constructor (ValaCodeVisitor* base, ValaConstructor* c);
static void vala_symbol_builder_real_visit_begin_destructor (ValaCodeVisitor* base, ValaDestructor* d);
static void vala_symbol_builder_real_visit_end_destructor (ValaCodeVisitor* base, ValaDestructor* d);
static void vala_symbol_builder_real_visit_begin_block (ValaCodeVisitor* base, ValaBlock* b);
static void vala_symbol_builder_real_visit_end_block (ValaCodeVisitor* base, ValaBlock* b);
static void vala_symbol_builder_real_visit_type_parameter (ValaCodeVisitor* base, ValaTypeParameter* p);
static void vala_symbol_builder_dispose (GObject * obj);

/**
 * Build the symbol tree for the specified code context.
 *
 * @param context a code context
 */
void vala_symbol_builder_build (ValaSymbolBuilder* self, ValaCodeContext* context)
{
	g_return_if_fail (VALA_IS_SYMBOL_BUILDER (self));
	g_return_if_fail (VALA_IS_CODE_CONTEXT (context));
	ValaSymbol* __temp0 = NULL;
	self->priv->root = (__temp0 = g_object_ref (vala_code_context_get_root (context)), (self->priv->root == NULL ? NULL : (self->priv->root = (g_object_unref (self->priv->root), NULL))), __temp0);
	vala_code_context_accept (context, VALA_CODE_VISITOR (self));
}


static void vala_symbol_builder_real_visit_begin_namespace (ValaCodeVisitor* base, ValaNamespace* ns)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_NAMESPACE (ns));
	if (vala_namespace_get_name (ns) == NULL) {
		vala_code_node_set_symbol (VALA_CODE_NODE (ns), self->priv->root);
	}
	if (vala_code_node_get_symbol (VALA_CODE_NODE (ns)) == NULL) {
		vala_code_node_set_symbol (VALA_CODE_NODE (ns), vala_symbol_lookup (self->priv->root, vala_namespace_get_name (ns)));
	}
	if (vala_code_node_get_symbol (VALA_CODE_NODE (ns)) == NULL) {
		ValaSymbol* __temp1 = NULL;
		vala_code_node_set_symbol (VALA_CODE_NODE (ns), (__temp1 = g_object_new (VALA_TYPE_SYMBOL, "node", ns, NULL)));
		(__temp1 == NULL ? NULL : (__temp1 = (g_object_unref (__temp1), NULL)));
		vala_symbol_add (self->priv->root, vala_namespace_get_name (ns), vala_code_node_get_symbol (VALA_CODE_NODE (ns)));
	}
	ValaSymbol* __temp3 = NULL;
	ValaSymbol* __temp2 = NULL;
	self->priv->current_symbol = (__temp3 = (__temp2 = vala_code_node_get_symbol (VALA_CODE_NODE (ns)), (__temp2 == NULL ? __temp2 : g_object_ref (__temp2))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp3);
}


static void vala_symbol_builder_real_visit_end_namespace (ValaCodeVisitor* base, ValaNamespace* ns)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_NAMESPACE (ns));
	ValaSymbol* __temp5 = NULL;
	ValaSymbol* __temp4 = NULL;
	self->priv->current_symbol = (__temp5 = (__temp4 = self->priv->current_symbol->parent_symbol, (__temp4 == NULL ? __temp4 : g_object_ref (__temp4))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp5);
}


static ValaSymbol* vala_symbol_builder_add_symbol (ValaSymbolBuilder* self, char* name, ValaCodeNode* node)
{
	g_return_val_if_fail (VALA_IS_SYMBOL_BUILDER (self), NULL);
	g_return_val_if_fail (name != NULL, NULL);
	g_return_val_if_fail (VALA_IS_CODE_NODE (node), NULL);
	if (vala_symbol_lookup (self->priv->current_symbol, name) != NULL) {
		vala_code_node_set_error (node, TRUE);
		char* __temp7 = NULL;
		char* __temp6 = NULL;
		vala_report_error (vala_code_node_get_source_reference (node), (__temp7 = g_strdup_printf ("`%s' already contains a definition for `%s'", (__temp6 = vala_symbol_get_full_name (self->priv->current_symbol)), name)));
		(__temp7 == NULL ? NULL : (__temp7 = (g_free (__temp7), NULL)));
		(__temp6 == NULL ? NULL : (__temp6 = (g_free (__temp6), NULL)));
		return NULL;
	}
	ValaSymbol* __temp9 = NULL;
	vala_code_node_set_symbol (node, (__temp9 = g_object_new (VALA_TYPE_SYMBOL, "node", node, NULL)));
	(__temp9 == NULL ? NULL : (__temp9 = (g_object_unref (__temp9), NULL)));
	vala_symbol_add (self->priv->current_symbol, name, vala_code_node_get_symbol (node));
	return vala_code_node_get_symbol (node);
}


static void vala_symbol_builder_real_visit_begin_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	if (vala_symbol_builder_add_symbol (self, vala_data_type_get_name (VALA_DATA_TYPE (cl)), VALA_CODE_NODE (cl)) == NULL) {
		return;
	}
	ValaSymbol* __temp12 = NULL;
	ValaSymbol* __temp11 = NULL;
	self->priv->current_symbol = (__temp12 = (__temp11 = vala_code_node_get_symbol (VALA_CODE_NODE (cl)), (__temp11 == NULL ? __temp11 : g_object_ref (__temp11))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp12);
}


static void vala_symbol_builder_real_visit_end_class (ValaCodeVisitor* base, ValaClass* cl)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_CLASS (cl));
	if (vala_code_node_get_error (VALA_CODE_NODE (cl))) {
		return;
	}
	/* skip classes with errors */
	ValaSymbol* __temp14 = NULL;
	ValaSymbol* __temp13 = NULL;
	self->priv->current_symbol = (__temp14 = (__temp13 = self->priv->current_symbol->parent_symbol, (__temp13 == NULL ? __temp13 : g_object_ref (__temp13))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp14);
}


static void vala_symbol_builder_real_visit_begin_struct (ValaCodeVisitor* base, ValaStruct* st)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_STRUCT (st));
	if (vala_symbol_builder_add_symbol (self, vala_data_type_get_name (VALA_DATA_TYPE (st)), VALA_CODE_NODE (st)) == NULL) {
		return;
	}
	ValaSymbol* __temp16 = NULL;
	ValaSymbol* __temp15 = NULL;
	self->priv->current_symbol = (__temp16 = (__temp15 = vala_code_node_get_symbol (VALA_CODE_NODE (st)), (__temp15 == NULL ? __temp15 : g_object_ref (__temp15))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp16);
}


static void vala_symbol_builder_real_visit_end_struct (ValaCodeVisitor* base, ValaStruct* st)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_STRUCT (st));
	if (vala_code_node_get_error (VALA_CODE_NODE (st))) {
		return;
	}
	/* skip structs with errors */
	ValaSymbol* __temp18 = NULL;
	ValaSymbol* __temp17 = NULL;
	self->priv->current_symbol = (__temp18 = (__temp17 = self->priv->current_symbol->parent_symbol, (__temp17 == NULL ? __temp17 : g_object_ref (__temp17))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp18);
}


static void vala_symbol_builder_real_visit_begin_interface (ValaCodeVisitor* base, ValaInterface* iface)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_INTERFACE (iface));
	if (vala_symbol_builder_add_symbol (self, vala_data_type_get_name (VALA_DATA_TYPE (iface)), VALA_CODE_NODE (iface)) == NULL) {
		return;
	}
	ValaSymbol* __temp20 = NULL;
	ValaSymbol* __temp19 = NULL;
	self->priv->current_symbol = (__temp20 = (__temp19 = vala_code_node_get_symbol (VALA_CODE_NODE (iface)), (__temp19 == NULL ? __temp19 : g_object_ref (__temp19))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp20);
}


static void vala_symbol_builder_real_visit_end_interface (ValaCodeVisitor* base, ValaInterface* iface)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_INTERFACE (iface));
	if (vala_code_node_get_error (VALA_CODE_NODE (iface))) {
		return;
	}
	/* skip interfaces with errors */
	ValaSymbol* __temp22 = NULL;
	ValaSymbol* __temp21 = NULL;
	self->priv->current_symbol = (__temp22 = (__temp21 = self->priv->current_symbol->parent_symbol, (__temp21 == NULL ? __temp21 : g_object_ref (__temp21))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp22);
}


static void vala_symbol_builder_real_visit_begin_enum (ValaCodeVisitor* base, ValaEnum* en)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_ENUM (en));
	if (vala_symbol_builder_add_symbol (self, vala_data_type_get_name (VALA_DATA_TYPE (en)), VALA_CODE_NODE (en)) == NULL) {
		return;
	}
	ValaSymbol* __temp24 = NULL;
	ValaSymbol* __temp23 = NULL;
	self->priv->current_symbol = (__temp24 = (__temp23 = vala_code_node_get_symbol (VALA_CODE_NODE (en)), (__temp23 == NULL ? __temp23 : g_object_ref (__temp23))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp24);
}


static void vala_symbol_builder_real_visit_end_enum (ValaCodeVisitor* base, ValaEnum* en)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_ENUM (en));
	if (vala_code_node_get_error (VALA_CODE_NODE (en))) {
		return;
	}
	/* skip enums with errors */
	ValaSymbol* __temp26 = NULL;
	ValaSymbol* __temp25 = NULL;
	self->priv->current_symbol = (__temp26 = (__temp25 = self->priv->current_symbol->parent_symbol, (__temp25 == NULL ? __temp25 : g_object_ref (__temp25))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp26);
}


static void vala_symbol_builder_real_visit_enum_value (ValaCodeVisitor* base, ValaEnumValue* ev)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_ENUM_VALUE (ev));
	ValaSymbol* __temp27 = NULL;
	vala_code_node_set_symbol (VALA_CODE_NODE (ev), (__temp27 = g_object_new (VALA_TYPE_SYMBOL, "node", ev, NULL)));
	(__temp27 == NULL ? NULL : (__temp27 = (g_object_unref (__temp27), NULL)));
	vala_symbol_add (self->priv->current_symbol, vala_enum_value_get_name (ev), vala_code_node_get_symbol (VALA_CODE_NODE (ev)));
}


static void vala_symbol_builder_real_visit_begin_callback (ValaCodeVisitor* base, ValaCallback* cb)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_CALLBACK (cb));
	if (vala_symbol_builder_add_symbol (self, vala_data_type_get_name (VALA_DATA_TYPE (cb)), VALA_CODE_NODE (cb)) == NULL) {
		return;
	}
	ValaSymbol* __temp29 = NULL;
	ValaSymbol* __temp28 = NULL;
	self->priv->current_symbol = (__temp29 = (__temp28 = vala_code_node_get_symbol (VALA_CODE_NODE (cb)), (__temp28 == NULL ? __temp28 : g_object_ref (__temp28))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp29);
}


static void vala_symbol_builder_real_visit_end_callback (ValaCodeVisitor* base, ValaCallback* cb)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_CALLBACK (cb));
	if (vala_code_node_get_error (VALA_CODE_NODE (cb))) {
		return;
	}
	/* skip enums with errors */
	ValaSymbol* __temp31 = NULL;
	ValaSymbol* __temp30 = NULL;
	self->priv->current_symbol = (__temp31 = (__temp30 = self->priv->current_symbol->parent_symbol, (__temp30 == NULL ? __temp30 : g_object_ref (__temp30))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp31);
}


static void vala_symbol_builder_real_visit_constant (ValaCodeVisitor* base, ValaConstant* c)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_CONSTANT (c));
	vala_symbol_builder_add_symbol (self, vala_constant_get_name (c), VALA_CODE_NODE (c));
}


static void vala_symbol_builder_real_visit_field (ValaCodeVisitor* base, ValaField* f)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_FIELD (f));
	vala_symbol_builder_add_symbol (self, vala_field_get_name (f), VALA_CODE_NODE (f));
}


static void vala_symbol_builder_real_visit_begin_method (ValaCodeVisitor* base, ValaMethod* m)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_METHOD (m));
	if (vala_symbol_builder_add_symbol (self, vala_method_get_name (m), VALA_CODE_NODE (m)) == NULL) {
		return;
	}
	if (vala_method_get_instance (m)) {
		if (!(VALA_IS_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (m))->parent_symbol)))) {
			vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (m)), "instance methods not allowed outside of data types");
			vala_code_node_set_error (VALA_CODE_NODE (m), TRUE);
			return;
		}
		ValaFormalParameter* __temp33 = NULL;
		ValaTypeReference* __temp32 = NULL;
		vala_method_set_this_parameter (m, (__temp33 = g_object_new (VALA_TYPE_FORMAL_PARAMETER, "name", "this", "type_reference", (__temp32 = g_object_new (VALA_TYPE_TYPE_REFERENCE, NULL)), NULL)));
		(__temp33 == NULL ? NULL : (__temp33 = (g_object_unref (__temp33), NULL)));
		(__temp32 == NULL ? NULL : (__temp32 = (g_object_unref (__temp32), NULL)));
		vala_formal_parameter_get_type_reference (vala_method_get_this_parameter (m))->type = VALA_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (m))->parent_symbol));
		ValaSymbol* __temp34 = NULL;
		vala_code_node_set_symbol (VALA_CODE_NODE (vala_method_get_this_parameter (m)), (__temp34 = g_object_new (VALA_TYPE_SYMBOL, "node", vala_method_get_this_parameter (m), NULL)));
		(__temp34 == NULL ? NULL : (__temp34 = (g_object_unref (__temp34), NULL)));
		vala_symbol_add (self->priv->current_symbol, vala_formal_parameter_get_name (vala_method_get_this_parameter (m)), vala_code_node_get_symbol (VALA_CODE_NODE (vala_method_get_this_parameter (m))));
	}
	ValaSymbol* __temp36 = NULL;
	ValaSymbol* __temp35 = NULL;
	self->priv->current_symbol = (__temp36 = (__temp35 = vala_code_node_get_symbol (VALA_CODE_NODE (m)), (__temp35 == NULL ? __temp35 : g_object_ref (__temp35))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp36);
}


static void vala_symbol_builder_real_visit_end_method (ValaCodeVisitor* base, ValaMethod* m)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_METHOD (m));
	if (vala_code_node_get_error (VALA_CODE_NODE (m))) {
		return;
	}
	/* skip methods with errors */
	ValaSymbol* __temp38 = NULL;
	ValaSymbol* __temp37 = NULL;
	self->priv->current_symbol = (__temp38 = (__temp37 = self->priv->current_symbol->parent_symbol, (__temp37 == NULL ? __temp37 : g_object_ref (__temp37))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp38);
}


static void vala_symbol_builder_real_visit_formal_parameter (ValaCodeVisitor* base, ValaFormalParameter* p)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_FORMAL_PARAMETER (p));
	if (!vala_formal_parameter_get_ellipsis (p)) {
		vala_symbol_builder_add_symbol (self, vala_formal_parameter_get_name (p), VALA_CODE_NODE (p));
	}
}


static void vala_symbol_builder_real_visit_begin_property (ValaCodeVisitor* base, ValaProperty* prop)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_PROPERTY (prop));
	if (vala_symbol_builder_add_symbol (self, vala_property_get_name (prop), VALA_CODE_NODE (prop)) == NULL) {
		return;
	}
	ValaSymbol* __temp40 = NULL;
	ValaSymbol* __temp39 = NULL;
	self->priv->current_symbol = (__temp40 = (__temp39 = vala_code_node_get_symbol (VALA_CODE_NODE (prop)), (__temp39 == NULL ? __temp39 : g_object_ref (__temp39))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp40);
	ValaFormalParameter* __temp42 = NULL;
	ValaTypeReference* __temp41 = NULL;
	prop->this_parameter = (__temp42 = g_object_new (VALA_TYPE_FORMAL_PARAMETER, "name", "this", "type_reference", (__temp41 = g_object_new (VALA_TYPE_TYPE_REFERENCE, NULL)), NULL), (prop->this_parameter == NULL ? NULL : (prop->this_parameter = (g_object_unref (prop->this_parameter), NULL))), __temp42);
	(__temp41 == NULL ? NULL : (__temp41 = (g_object_unref (__temp41), NULL)));
	vala_formal_parameter_get_type_reference (prop->this_parameter)->type = VALA_DATA_TYPE (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (prop))->parent_symbol));
	ValaSymbol* __temp43 = NULL;
	vala_code_node_set_symbol (VALA_CODE_NODE (prop->this_parameter), (__temp43 = g_object_new (VALA_TYPE_SYMBOL, "node", prop->this_parameter, NULL)));
	(__temp43 == NULL ? NULL : (__temp43 = (g_object_unref (__temp43), NULL)));
	vala_symbol_add (self->priv->current_symbol, vala_formal_parameter_get_name (prop->this_parameter), vala_code_node_get_symbol (VALA_CODE_NODE (prop->this_parameter)));
}


static void vala_symbol_builder_real_visit_end_property (ValaCodeVisitor* base, ValaProperty* prop)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_PROPERTY (prop));
	if (vala_code_node_get_error (VALA_CODE_NODE (prop))) {
		return;
	}
	/* skip properties with errors */
	ValaSymbol* __temp45 = NULL;
	ValaSymbol* __temp44 = NULL;
	self->priv->current_symbol = (__temp45 = (__temp44 = self->priv->current_symbol->parent_symbol, (__temp44 == NULL ? __temp44 : g_object_ref (__temp44))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp45);
}


static void vala_symbol_builder_real_visit_begin_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_PROPERTY_ACCESSOR (acc));
	ValaSymbol* __temp46 = NULL;
	vala_code_node_set_symbol (VALA_CODE_NODE (acc), (__temp46 = g_object_new (VALA_TYPE_SYMBOL, "node", acc, NULL)));
	(__temp46 == NULL ? NULL : (__temp46 = (g_object_unref (__temp46), NULL)));
	vala_code_node_get_symbol (VALA_CODE_NODE (acc))->parent_symbol = self->priv->current_symbol;
	ValaSymbol* __temp48 = NULL;
	ValaSymbol* __temp47 = NULL;
	self->priv->current_symbol = (__temp48 = (__temp47 = vala_code_node_get_symbol (VALA_CODE_NODE (acc)), (__temp47 == NULL ? __temp47 : g_object_ref (__temp47))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp48);
	if (vala_property_accessor_get_writable (acc) || vala_property_accessor_get_construct_ (acc)) {
		ValaFormalParameter* __temp49 = NULL;
		acc->value_parameter = (__temp49 = g_object_new (VALA_TYPE_FORMAL_PARAMETER, "name", "value", "type_reference", vala_property_get_type_reference ((VALA_PROPERTY (vala_symbol_get_node (self->priv->current_symbol->parent_symbol)))), NULL), (acc->value_parameter == NULL ? NULL : (acc->value_parameter = (g_object_unref (acc->value_parameter), NULL))), __temp49);
		ValaSymbol* __temp50 = NULL;
		vala_code_node_set_symbol (VALA_CODE_NODE (acc->value_parameter), (__temp50 = g_object_new (VALA_TYPE_SYMBOL, "node", acc->value_parameter, NULL)));
		(__temp50 == NULL ? NULL : (__temp50 = (g_object_unref (__temp50), NULL)));
		vala_symbol_add (self->priv->current_symbol, vala_formal_parameter_get_name (acc->value_parameter), vala_code_node_get_symbol (VALA_CODE_NODE (acc->value_parameter)));
	}
	if (vala_property_accessor_get_body (acc) == NULL) {
		/* no accessor body specified, insert default body */
		ValaProperty* prop = VALA_PROPERTY (vala_symbol_get_node (vala_code_node_get_symbol (VALA_CODE_NODE (acc))->parent_symbol));
		ValaBlock* block = g_object_new (VALA_TYPE_BLOCK, NULL);
		if (vala_property_accessor_get_readable (acc)) {
			ValaReturnStatement* __temp53 = NULL;
			ValaMemberAccess* __temp52 = NULL;
			char* __temp51 = NULL;
			vala_block_add_statement (block, VALA_STATEMENT ((__temp53 = g_object_new (VALA_TYPE_RETURN_STATEMENT, "return_expression", (__temp52 = g_object_new (VALA_TYPE_MEMBER_ACCESS, "member_name", (__temp51 = g_strdup_printf ("_%s", vala_property_get_name (prop))), NULL)), NULL))));
			(__temp53 == NULL ? NULL : (__temp53 = (g_object_unref (__temp53), NULL)));
			(__temp52 == NULL ? NULL : (__temp52 = (g_object_unref (__temp52), NULL)));
			(__temp51 == NULL ? NULL : (__temp51 = (g_free (__temp51), NULL)));
		} else {
			ValaExpressionStatement* __temp58 = NULL;
			ValaAssignment* __temp57 = NULL;
			ValaMemberAccess* __temp56 = NULL;
			ValaMemberAccess* __temp55 = NULL;
			char* __temp54 = NULL;
			vala_block_add_statement (block, VALA_STATEMENT ((__temp58 = g_object_new (VALA_TYPE_EXPRESSION_STATEMENT, "expression", (__temp57 = g_object_new (VALA_TYPE_ASSIGNMENT, "left", (__temp55 = g_object_new (VALA_TYPE_MEMBER_ACCESS, "member_name", (__temp54 = g_strdup_printf ("_%s", vala_property_get_name (prop))), NULL)), "right", (__temp56 = g_object_new (VALA_TYPE_MEMBER_ACCESS, "member_name", "value", NULL)), NULL)), NULL))));
			(__temp58 == NULL ? NULL : (__temp58 = (g_object_unref (__temp58), NULL)));
			(__temp57 == NULL ? NULL : (__temp57 = (g_object_unref (__temp57), NULL)));
			(__temp56 == NULL ? NULL : (__temp56 = (g_object_unref (__temp56), NULL)));
			(__temp55 == NULL ? NULL : (__temp55 = (g_object_unref (__temp55), NULL)));
			(__temp54 == NULL ? NULL : (__temp54 = (g_free (__temp54), NULL)));
		}
		vala_property_accessor_set_body (acc, VALA_STATEMENT (block));
		(block == NULL ? NULL : (block = (g_object_unref (block), NULL)));
	}
}


static void vala_symbol_builder_real_visit_end_property_accessor (ValaCodeVisitor* base, ValaPropertyAccessor* acc)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_PROPERTY_ACCESSOR (acc));
	ValaSymbol* __temp60 = NULL;
	ValaSymbol* __temp59 = NULL;
	self->priv->current_symbol = (__temp60 = (__temp59 = self->priv->current_symbol->parent_symbol, (__temp59 == NULL ? __temp59 : g_object_ref (__temp59))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp60);
}


static void vala_symbol_builder_real_visit_begin_signal (ValaCodeVisitor* base, ValaSignal* sig)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_SIGNAL (sig));
	if (vala_symbol_builder_add_symbol (self, vala_signal_get_name (sig), VALA_CODE_NODE (sig)) == NULL) {
		return;
	}
	ValaSymbol* __temp62 = NULL;
	ValaSymbol* __temp61 = NULL;
	self->priv->current_symbol = (__temp62 = (__temp61 = vala_code_node_get_symbol (VALA_CODE_NODE (sig)), (__temp61 == NULL ? __temp61 : g_object_ref (__temp61))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp62);
}


static void vala_symbol_builder_real_visit_end_signal (ValaCodeVisitor* base, ValaSignal* sig)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_SIGNAL (sig));
	if (vala_code_node_get_error (VALA_CODE_NODE (sig))) {
		return;
	}
	/* skip signals with errors */
	ValaSymbol* __temp64 = NULL;
	ValaSymbol* __temp63 = NULL;
	self->priv->current_symbol = (__temp64 = (__temp63 = self->priv->current_symbol->parent_symbol, (__temp63 == NULL ? __temp63 : g_object_ref (__temp63))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp64);
}


static void vala_symbol_builder_real_visit_begin_constructor (ValaCodeVisitor* base, ValaConstructor* c)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_CONSTRUCTOR (c));
	ValaSymbol* __temp65 = NULL;
	vala_code_node_set_symbol (VALA_CODE_NODE (c), (__temp65 = g_object_new (VALA_TYPE_SYMBOL, "node", c, NULL)));
	(__temp65 == NULL ? NULL : (__temp65 = (g_object_unref (__temp65), NULL)));
	vala_code_node_get_symbol (VALA_CODE_NODE (c))->parent_symbol = self->priv->current_symbol;
	ValaSymbol* __temp67 = NULL;
	ValaSymbol* __temp66 = NULL;
	self->priv->current_symbol = (__temp67 = (__temp66 = vala_code_node_get_symbol (VALA_CODE_NODE (c)), (__temp66 == NULL ? __temp66 : g_object_ref (__temp66))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp67);
}


static void vala_symbol_builder_real_visit_end_constructor (ValaCodeVisitor* base, ValaConstructor* c)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_CONSTRUCTOR (c));
	ValaSymbol* __temp69 = NULL;
	ValaSymbol* __temp68 = NULL;
	self->priv->current_symbol = (__temp69 = (__temp68 = self->priv->current_symbol->parent_symbol, (__temp68 == NULL ? __temp68 : g_object_ref (__temp68))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp69);
}


static void vala_symbol_builder_real_visit_begin_destructor (ValaCodeVisitor* base, ValaDestructor* d)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_DESTRUCTOR (d));
	ValaSymbol* __temp70 = NULL;
	vala_code_node_set_symbol (VALA_CODE_NODE (d), (__temp70 = g_object_new (VALA_TYPE_SYMBOL, "node", d, NULL)));
	(__temp70 == NULL ? NULL : (__temp70 = (g_object_unref (__temp70), NULL)));
	vala_code_node_get_symbol (VALA_CODE_NODE (d))->parent_symbol = self->priv->current_symbol;
	ValaSymbol* __temp72 = NULL;
	ValaSymbol* __temp71 = NULL;
	self->priv->current_symbol = (__temp72 = (__temp71 = vala_code_node_get_symbol (VALA_CODE_NODE (d)), (__temp71 == NULL ? __temp71 : g_object_ref (__temp71))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp72);
}


static void vala_symbol_builder_real_visit_end_destructor (ValaCodeVisitor* base, ValaDestructor* d)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_DESTRUCTOR (d));
	ValaSymbol* __temp74 = NULL;
	ValaSymbol* __temp73 = NULL;
	self->priv->current_symbol = (__temp74 = (__temp73 = self->priv->current_symbol->parent_symbol, (__temp73 == NULL ? __temp73 : g_object_ref (__temp73))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp74);
}


static void vala_symbol_builder_real_visit_begin_block (ValaCodeVisitor* base, ValaBlock* b)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_BLOCK (b));
	ValaSymbol* __temp75 = NULL;
	vala_code_node_set_symbol (VALA_CODE_NODE (b), (__temp75 = g_object_new (VALA_TYPE_SYMBOL, "node", b, NULL)));
	(__temp75 == NULL ? NULL : (__temp75 = (g_object_unref (__temp75), NULL)));
	vala_code_node_get_symbol (VALA_CODE_NODE (b))->parent_symbol = self->priv->current_symbol;
	ValaSymbol* __temp77 = NULL;
	ValaSymbol* __temp76 = NULL;
	self->priv->current_symbol = (__temp77 = (__temp76 = vala_code_node_get_symbol (VALA_CODE_NODE (b)), (__temp76 == NULL ? __temp76 : g_object_ref (__temp76))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp77);
}


static void vala_symbol_builder_real_visit_end_block (ValaCodeVisitor* base, ValaBlock* b)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_BLOCK (b));
	ValaSymbol* __temp79 = NULL;
	ValaSymbol* __temp78 = NULL;
	self->priv->current_symbol = (__temp79 = (__temp78 = self->priv->current_symbol->parent_symbol, (__temp78 == NULL ? __temp78 : g_object_ref (__temp78))), (self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL))), __temp79);
}


static void vala_symbol_builder_real_visit_type_parameter (ValaCodeVisitor* base, ValaTypeParameter* p)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (base);
	g_return_if_fail (VALA_IS_TYPE_PARAMETER (p));
	vala_symbol_builder_add_symbol (self, vala_type_parameter_get_name (p), VALA_CODE_NODE (p));
}


void vala_symbol_builder_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (object);
	switch (property_id) {
	}
}


void vala_symbol_builder_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (object);
	switch (property_id) {
	}
}


static void vala_symbol_builder_class_init (ValaSymbolBuilderClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaSymbolBuilderPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_symbol_builder_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_symbol_builder_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_symbol_builder_dispose;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_namespace = vala_symbol_builder_real_visit_begin_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_namespace = vala_symbol_builder_real_visit_end_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_class = vala_symbol_builder_real_visit_begin_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_class = vala_symbol_builder_real_visit_end_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_struct = vala_symbol_builder_real_visit_begin_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_struct = vala_symbol_builder_real_visit_end_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_interface = vala_symbol_builder_real_visit_begin_interface;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_interface = vala_symbol_builder_real_visit_end_interface;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_enum = vala_symbol_builder_real_visit_begin_enum;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_enum = vala_symbol_builder_real_visit_end_enum;
	VALA_CODE_VISITOR_CLASS (klass)->visit_enum_value = vala_symbol_builder_real_visit_enum_value;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_callback = vala_symbol_builder_real_visit_begin_callback;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_callback = vala_symbol_builder_real_visit_end_callback;
	VALA_CODE_VISITOR_CLASS (klass)->visit_constant = vala_symbol_builder_real_visit_constant;
	VALA_CODE_VISITOR_CLASS (klass)->visit_field = vala_symbol_builder_real_visit_field;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_method = vala_symbol_builder_real_visit_begin_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_method = vala_symbol_builder_real_visit_end_method;
	VALA_CODE_VISITOR_CLASS (klass)->visit_formal_parameter = vala_symbol_builder_real_visit_formal_parameter;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_property = vala_symbol_builder_real_visit_begin_property;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_property = vala_symbol_builder_real_visit_end_property;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_property_accessor = vala_symbol_builder_real_visit_begin_property_accessor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_property_accessor = vala_symbol_builder_real_visit_end_property_accessor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_signal = vala_symbol_builder_real_visit_begin_signal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_signal = vala_symbol_builder_real_visit_end_signal;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_constructor = vala_symbol_builder_real_visit_begin_constructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_constructor = vala_symbol_builder_real_visit_end_constructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_destructor = vala_symbol_builder_real_visit_begin_destructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_destructor = vala_symbol_builder_real_visit_end_destructor;
	VALA_CODE_VISITOR_CLASS (klass)->visit_begin_block = vala_symbol_builder_real_visit_begin_block;
	VALA_CODE_VISITOR_CLASS (klass)->visit_end_block = vala_symbol_builder_real_visit_end_block;
	VALA_CODE_VISITOR_CLASS (klass)->visit_type_parameter = vala_symbol_builder_real_visit_type_parameter;
}


static void vala_symbol_builder_init (ValaSymbolBuilder * self)
{
	self->priv = VALA_SYMBOL_BUILDER_GET_PRIVATE (self);
}


static void vala_symbol_builder_dispose (GObject * obj)
{
	ValaSymbolBuilder * self = VALA_SYMBOL_BUILDER (obj);
	(self->priv->root == NULL ? NULL : (self->priv->root = (g_object_unref (self->priv->root), NULL)));
	(self->priv->current_type == NULL ? NULL : (self->priv->current_type = (g_object_unref (self->priv->current_type), NULL)));
	(self->priv->current_symbol == NULL ? NULL : (self->priv->current_symbol = (g_object_unref (self->priv->current_symbol), NULL)));
	ValaSymbolBuilderClass * klass;
	GObjectClass * parent_class;
	klass = VALA_SYMBOL_BUILDER_CLASS (g_type_class_peek (VALA_TYPE_SYMBOL_BUILDER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_symbol_builder_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaSymbolBuilderClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_symbol_builder_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaSymbolBuilder), 0, (GInstanceInitFunc) vala_symbol_builder_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "ValaSymbolBuilder", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



