/* valatypereference.vala
 *
 * Copyright (C) 2006  Jürg Billeter, Raffaele Sandrini
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 *	Raffaele Sandrini <rasa@gmx.ch>
 */

#include <valatypereference.h>
#include <valasourcereference.h>
#include <valaexpression.h>
#include <valamemberaccess.h>
#include <valacodevisitor.h>
#include <valadatatype.h>
#include <valareport.h>
#include <valaarray.h>
#include <valasymbol.h>
#include <valatypeparameter.h>

struct _ValaTypeReferencePrivate {
	char* _namespace_name;
	char* _type_name;
	gboolean _is_ref;
	gboolean _is_lvalue_ref;
	gboolean _is_weak;
	gboolean _is_out;
	gboolean _array;
	gboolean _array_own;
	gboolean _non_null;
	gboolean _floating_reference;
	GList* type_argument_list;
};
#define VALA_TYPE_REFERENCE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_TYPE_REFERENCE, ValaTypeReferencePrivate))
enum  {
	VALA_TYPE_REFERENCE_DUMMY_PROPERTY,
	VALA_TYPE_REFERENCE_NAMESPACE_NAME,
	VALA_TYPE_REFERENCE_TYPE_NAME,
	VALA_TYPE_REFERENCE_IS_REF,
	VALA_TYPE_REFERENCE_IS_LVALUE_REF,
	VALA_TYPE_REFERENCE_IS_WEAK,
	VALA_TYPE_REFERENCE_IS_OUT,
	VALA_TYPE_REFERENCE_ARRAY,
	VALA_TYPE_REFERENCE_ARRAY_OWN,
	VALA_TYPE_REFERENCE_NON_NULL,
	VALA_TYPE_REFERENCE_FLOATING_REFERENCE,
};
static void vala_type_reference_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static void vala_type_reference_dispose (GObject * obj);

ValaTypeReference* vala_type_reference_new (char* ns, char* type_name, ValaSourceReference* source)
{
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	return (g_object_new (VALA_TYPE_TYPE_REFERENCE, "namespace_name", ns, "type_name", type_name, "source_reference", source, NULL));
}


ValaTypeReference* vala_type_reference_new_from_expression (ValaExpression* expr, ValaSourceReference* source)
{
	g_return_val_if_fail (expr == NULL || VALA_IS_EXPRESSION (expr), NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	char* ns = NULL;
	char* type_name = NULL;
	if (VALA_IS_MEMBER_ACCESS (expr)) {
		ValaMemberAccess* ma = VALA_MEMBER_ACCESS (expr);
		if (vala_member_access_get_inner (ma) != NULL) {
			if (VALA_IS_MEMBER_ACCESS (vala_member_access_get_inner (ma))) {
				ValaMemberAccess* simple = VALA_MEMBER_ACCESS (vala_member_access_get_inner (ma));
				return (g_object_new (VALA_TYPE_TYPE_REFERENCE, "namespace_name", vala_member_access_get_member_name (simple), "type_name", vala_member_access_get_member_name (ma), "source_reference", source, NULL));
			}
		} else {
			return (g_object_new (VALA_TYPE_TYPE_REFERENCE, "type_name", vala_member_access_get_member_name (ma), "source_reference", source, NULL));
		}
	}
	return NULL;
}


/* FIXME: raise error */
void vala_type_reference_add_type_argument (ValaTypeReference* self, ValaTypeReference* arg)
{
	g_return_if_fail (VALA_IS_TYPE_REFERENCE (self));
	g_return_if_fail (VALA_IS_TYPE_REFERENCE (arg));
	self->priv->type_argument_list = g_list_append (self->priv->type_argument_list, g_object_ref (arg));
}


GList* vala_type_reference_get_type_arguments (ValaTypeReference* self)
{
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (self), NULL);
	return g_list_copy (self->priv->type_argument_list);
}


static void vala_type_reference_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaTypeReference * self = VALA_TYPE_REFERENCE (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	{
		GList * type_arg_it;
		for (type_arg_it = self->priv->type_argument_list; type_arg_it != NULL; type_arg_it = type_arg_it->next) {
			ValaTypeReference* type_arg = type_arg_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (type_arg), visitor);
			}
		}
	}
	vala_code_visitor_visit_type_reference (visitor, self);
}


char* vala_type_reference_get_cname (ValaTypeReference* self, gboolean var_type)
{
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (self), NULL);
	if (self->type == NULL && self->type_parameter == NULL) {
		if (var_type) {
			return g_strdup ("gpointer");
		} else {
			return g_strdup ("void");
		}
	}
	char* ptr = NULL;
	char* arr = NULL;
	if (self->type_parameter == NULL && !vala_data_type_is_reference_type (self->type) && !vala_type_reference_get_is_ref (self)) {
		ptr = "";
	} else if (((self->type_parameter != NULL || vala_data_type_is_reference_type (self->type)) && !vala_type_reference_get_is_out (self)) || vala_type_reference_get_is_ref (self)) {
		ptr = "*";
	} else {
		ptr = "**";
	}
	if (self->type != NULL) {
		return g_strconcat (vala_data_type_get_cname (self->type), ptr, arr, NULL);
	} else if (self->type_parameter != NULL) {
		return g_strconcat ("gpointer", ptr, arr, NULL);
	} else {
		/* raise error */
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (self)), "unresolved type reference");
		return NULL;
	}
}


char* vala_type_reference_get_const_cname (ValaTypeReference* self)
{
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (self), NULL);
	char* ptr = NULL;
	ValaDataType* t = NULL;
	/* FIXME: dirty hack to make constant arrays possible */
	if (VALA_IS_ARRAY (self->type)) {
		t = vala_array_get_element_type ((VALA_ARRAY (self->type)));
	} else {
		t = self->type;
	}
	if (!vala_data_type_is_reference_type (t)) {
		ptr = "";
	} else {
		ptr = "*";
	}
	return g_strdup_printf ("const %s%s", vala_data_type_get_cname (t), ptr);
}


char* vala_type_reference_get_upper_case_cname (ValaTypeReference* self, char* infix)
{
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (self), NULL);
	return vala_data_type_get_upper_case_cname (self->type, infix);
}


char* vala_type_reference_to_string (ValaTypeReference* self)
{
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (self), NULL);
	if (self->type != NULL) {
		return vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (self->type)));
	} else if (self->type_parameter != NULL) {
		char* __temp13 = NULL;
		return (__temp13 = vala_type_parameter_get_name (self->type_parameter), (__temp13 == NULL ? __temp13 : g_strdup (__temp13)));
	} else {
		return g_strdup ("null");
	}
}


ValaTypeReference* vala_type_reference_copy (ValaTypeReference* self)
{
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (self), NULL);
	ValaTypeReference* result = g_object_new (VALA_TYPE_TYPE_REFERENCE, NULL);
	vala_type_reference_set_is_ref (result, vala_type_reference_get_is_ref (self));
	vala_type_reference_set_is_lvalue_ref (result, vala_type_reference_get_is_lvalue_ref (self));
	vala_type_reference_set_is_weak (result, vala_type_reference_get_is_weak (self));
	vala_type_reference_set_is_out (result, vala_type_reference_get_is_out (self));
	vala_type_reference_set_array (result, vala_type_reference_get_array (self));
	vala_type_reference_set_array_own (result, vala_type_reference_get_array_own (self));
	vala_type_reference_set_non_null (result, vala_type_reference_get_non_null (self));
	result->type = self->type;
	ValaTypeParameter* __temp17 = NULL;
	ValaTypeParameter* __temp16 = NULL;
	result->type_parameter = (__temp17 = (__temp16 = self->type_parameter, (__temp16 == NULL ? __temp16 : g_object_ref (__temp16))), (result->type_parameter == NULL ? NULL : (result->type_parameter = (g_object_unref (result->type_parameter), NULL))), __temp17);
	ValaTypeReference* __temp18 = NULL;
	ValaTypeReference* __temp19 = NULL;
	return (__temp19 = (__temp18 = result, (__temp18 == NULL ? __temp18 : g_object_ref (__temp18))), (result == NULL ? NULL : (result = (g_object_unref (result), NULL))), __temp19);
	(result == NULL ? NULL : (result = (g_object_unref (result), NULL)));
}


gboolean vala_type_reference_equals (ValaTypeReference* self, ValaTypeReference* type2)
{
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (self), FALSE);
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (type2), FALSE);
	if (vala_type_reference_get_is_ref (type2) != vala_type_reference_get_is_ref (self)) {
		return FALSE;
	}
	if (vala_type_reference_get_is_lvalue_ref (type2) != vala_type_reference_get_is_lvalue_ref (self)) {
		return FALSE;
	}
	if (vala_type_reference_get_is_weak (type2) != vala_type_reference_get_is_weak (self)) {
		return FALSE;
	}
	if (vala_type_reference_get_is_out (type2) != vala_type_reference_get_is_out (self)) {
		return FALSE;
	}
	if (vala_type_reference_get_array (type2) != vala_type_reference_get_array (self)) {
		return FALSE;
	}
	if (vala_type_reference_get_array_own (type2) != vala_type_reference_get_array_own (self)) {
		return FALSE;
	}
	if (vala_type_reference_get_non_null (type2) != vala_type_reference_get_non_null (self)) {
		return FALSE;
	}
	if (type2->type != self->type) {
		return FALSE;
	}
	if (type2->type_parameter != self->type_parameter) {
		return FALSE;
	}
	if (vala_type_reference_get_floating_reference (type2) != vala_type_reference_get_floating_reference (self)) {
		return FALSE;
	}
	return TRUE;
}


char* vala_type_reference_get_namespace_name (ValaTypeReference* self)
{
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (self), NULL);
	return self->priv->_namespace_name;
}


void vala_type_reference_set_namespace_name (ValaTypeReference* self, char* value)
{
	g_return_if_fail (VALA_IS_TYPE_REFERENCE (self));
	char* __temp33 = NULL;
	char* __temp32 = NULL;
	self->priv->_namespace_name = (__temp33 = (__temp32 = value, (__temp32 == NULL ? __temp32 : g_strdup (__temp32))), (self->priv->_namespace_name == NULL ? NULL : (self->priv->_namespace_name = (g_free (self->priv->_namespace_name), NULL))), __temp33);
}


char* vala_type_reference_get_type_name (ValaTypeReference* self)
{
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (self), NULL);
	return self->priv->_type_name;
}


void vala_type_reference_set_type_name (ValaTypeReference* self, char* value)
{
	g_return_if_fail (VALA_IS_TYPE_REFERENCE (self));
	char* __temp36 = NULL;
	char* __temp35 = NULL;
	self->priv->_type_name = (__temp36 = (__temp35 = value, (__temp35 == NULL ? __temp35 : g_strdup (__temp35))), (self->priv->_type_name == NULL ? NULL : (self->priv->_type_name = (g_free (self->priv->_type_name), NULL))), __temp36);
}


gboolean vala_type_reference_get_is_ref (ValaTypeReference* self)
{
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (self), FALSE);
	return self->priv->_is_ref;
}


void vala_type_reference_set_is_ref (ValaTypeReference* self, gboolean value)
{
	g_return_if_fail (VALA_IS_TYPE_REFERENCE (self));
	self->priv->_is_ref = value;
}


gboolean vala_type_reference_get_is_lvalue_ref (ValaTypeReference* self)
{
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (self), FALSE);
	return self->priv->_is_lvalue_ref;
}


void vala_type_reference_set_is_lvalue_ref (ValaTypeReference* self, gboolean value)
{
	g_return_if_fail (VALA_IS_TYPE_REFERENCE (self));
	self->priv->_is_lvalue_ref = value;
}


gboolean vala_type_reference_get_is_weak (ValaTypeReference* self)
{
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (self), FALSE);
	return self->priv->_is_weak;
}


void vala_type_reference_set_is_weak (ValaTypeReference* self, gboolean value)
{
	g_return_if_fail (VALA_IS_TYPE_REFERENCE (self));
	self->priv->_is_weak = value;
}


gboolean vala_type_reference_get_is_out (ValaTypeReference* self)
{
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (self), FALSE);
	return self->priv->_is_out;
}


void vala_type_reference_set_is_out (ValaTypeReference* self, gboolean value)
{
	g_return_if_fail (VALA_IS_TYPE_REFERENCE (self));
	self->priv->_is_out = value;
}


gboolean vala_type_reference_get_array (ValaTypeReference* self)
{
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (self), FALSE);
	return self->priv->_array;
}


void vala_type_reference_set_array (ValaTypeReference* self, gboolean value)
{
	g_return_if_fail (VALA_IS_TYPE_REFERENCE (self));
	self->priv->_array = value;
}


gboolean vala_type_reference_get_array_own (ValaTypeReference* self)
{
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (self), FALSE);
	return self->priv->_array_own;
}


void vala_type_reference_set_array_own (ValaTypeReference* self, gboolean value)
{
	g_return_if_fail (VALA_IS_TYPE_REFERENCE (self));
	self->priv->_array_own = value;
}


gboolean vala_type_reference_get_non_null (ValaTypeReference* self)
{
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (self), FALSE);
	return self->priv->_non_null;
}


void vala_type_reference_set_non_null (ValaTypeReference* self, gboolean value)
{
	g_return_if_fail (VALA_IS_TYPE_REFERENCE (self));
	self->priv->_non_null = value;
}


gboolean vala_type_reference_get_floating_reference (ValaTypeReference* self)
{
	g_return_val_if_fail (VALA_IS_TYPE_REFERENCE (self), FALSE);
	return self->priv->_floating_reference;
}


void vala_type_reference_set_floating_reference (ValaTypeReference* self, gboolean value)
{
	g_return_if_fail (VALA_IS_TYPE_REFERENCE (self));
	self->priv->_floating_reference = value;
}


void vala_type_reference_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaTypeReference * self = VALA_TYPE_REFERENCE (object);
	switch (property_id) {
		case VALA_TYPE_REFERENCE_NAMESPACE_NAME:
		g_value_set_string (value, vala_type_reference_get_namespace_name (self));
		break;
		case VALA_TYPE_REFERENCE_TYPE_NAME:
		g_value_set_string (value, vala_type_reference_get_type_name (self));
		break;
		case VALA_TYPE_REFERENCE_IS_REF:
		g_value_set_boolean (value, vala_type_reference_get_is_ref (self));
		break;
		case VALA_TYPE_REFERENCE_IS_LVALUE_REF:
		g_value_set_boolean (value, vala_type_reference_get_is_lvalue_ref (self));
		break;
		case VALA_TYPE_REFERENCE_IS_WEAK:
		g_value_set_boolean (value, vala_type_reference_get_is_weak (self));
		break;
		case VALA_TYPE_REFERENCE_IS_OUT:
		g_value_set_boolean (value, vala_type_reference_get_is_out (self));
		break;
		case VALA_TYPE_REFERENCE_ARRAY:
		g_value_set_boolean (value, vala_type_reference_get_array (self));
		break;
		case VALA_TYPE_REFERENCE_ARRAY_OWN:
		g_value_set_boolean (value, vala_type_reference_get_array_own (self));
		break;
		case VALA_TYPE_REFERENCE_NON_NULL:
		g_value_set_boolean (value, vala_type_reference_get_non_null (self));
		break;
		case VALA_TYPE_REFERENCE_FLOATING_REFERENCE:
		g_value_set_boolean (value, vala_type_reference_get_floating_reference (self));
		break;
	}
}


void vala_type_reference_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaTypeReference * self = VALA_TYPE_REFERENCE (object);
	switch (property_id) {
		case VALA_TYPE_REFERENCE_NAMESPACE_NAME:
		vala_type_reference_set_namespace_name (self, g_value_get_string (value));
		break;
		case VALA_TYPE_REFERENCE_TYPE_NAME:
		vala_type_reference_set_type_name (self, g_value_get_string (value));
		break;
		case VALA_TYPE_REFERENCE_IS_REF:
		vala_type_reference_set_is_ref (self, g_value_get_boolean (value));
		break;
		case VALA_TYPE_REFERENCE_IS_LVALUE_REF:
		vala_type_reference_set_is_lvalue_ref (self, g_value_get_boolean (value));
		break;
		case VALA_TYPE_REFERENCE_IS_WEAK:
		vala_type_reference_set_is_weak (self, g_value_get_boolean (value));
		break;
		case VALA_TYPE_REFERENCE_IS_OUT:
		vala_type_reference_set_is_out (self, g_value_get_boolean (value));
		break;
		case VALA_TYPE_REFERENCE_ARRAY:
		vala_type_reference_set_array (self, g_value_get_boolean (value));
		break;
		case VALA_TYPE_REFERENCE_ARRAY_OWN:
		vala_type_reference_set_array_own (self, g_value_get_boolean (value));
		break;
		case VALA_TYPE_REFERENCE_NON_NULL:
		vala_type_reference_set_non_null (self, g_value_get_boolean (value));
		break;
		case VALA_TYPE_REFERENCE_FLOATING_REFERENCE:
		vala_type_reference_set_floating_reference (self, g_value_get_boolean (value));
		break;
	}
}


static void vala_type_reference_class_init (ValaTypeReferenceClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaTypeReferencePrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_type_reference_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_type_reference_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_type_reference_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_type_reference_real_accept;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_TYPE_REFERENCE_NAMESPACE_NAME, g_param_spec_string ("namespace-name", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_TYPE_REFERENCE_TYPE_NAME, g_param_spec_string ("type-name", "foo", "bar", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_TYPE_REFERENCE_IS_REF, g_param_spec_boolean ("is-ref", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_TYPE_REFERENCE_IS_LVALUE_REF, g_param_spec_boolean ("is-lvalue-ref", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_TYPE_REFERENCE_IS_WEAK, g_param_spec_boolean ("is-weak", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_TYPE_REFERENCE_IS_OUT, g_param_spec_boolean ("is-out", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_TYPE_REFERENCE_ARRAY, g_param_spec_boolean ("array", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_TYPE_REFERENCE_ARRAY_OWN, g_param_spec_boolean ("array-own", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_TYPE_REFERENCE_NON_NULL, g_param_spec_boolean ("non-null", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_TYPE_REFERENCE_FLOATING_REFERENCE, g_param_spec_boolean ("floating-reference", "foo", "bar", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_type_reference_init (ValaTypeReference * self)
{
	self->priv = VALA_TYPE_REFERENCE_GET_PRIVATE (self);
}


static void vala_type_reference_dispose (GObject * obj)
{
	ValaTypeReference * self = VALA_TYPE_REFERENCE (obj);
	(self->priv->_namespace_name == NULL ? NULL : (self->priv->_namespace_name = (g_free (self->priv->_namespace_name), NULL)));
	(self->priv->_type_name == NULL ? NULL : (self->priv->_type_name = (g_free (self->priv->_type_name), NULL)));
	(self->type_parameter == NULL ? NULL : (self->type_parameter = (g_object_unref (self->type_parameter), NULL)));
	(self->priv->type_argument_list == NULL ? NULL : (self->priv->type_argument_list = (g_list_foreach (self->priv->type_argument_list, (GFunc) g_object_unref, NULL), g_list_free (self->priv->type_argument_list), NULL)));
	ValaTypeReferenceClass * klass;
	GObjectClass * parent_class;
	klass = VALA_TYPE_REFERENCE_CLASS (g_type_class_peek (VALA_TYPE_TYPE_REFERENCE));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_type_reference_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaTypeReferenceClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_type_reference_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaTypeReference), 0, (GInstanceInitFunc) vala_type_reference_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_CODE_NODE, "ValaTypeReference", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



